      subroutine sm_3d_df
     +
     +   (nelm,npts
     +   ,Ichoice
     +   ,epsilon
     +   ,f
     +   )

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c==========================================

c----------------------------------------
c Smoothing of the function f defined over
c a closed surface
c
c SYMBOLS
c -------
c
c f:       function to be smoothed
c
c rlplcn:  flat-Laplacian
c rbilap:  flat-bi-Laplacian
c
c Ichoice  = 1: smooth with the laplacian
c            2: smooth with the bi-Laplacian
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension      p(1026,3)
      Dimension     ne(1026,7)
      Dimension       f(1026),rlplcn(1026),rbilpl(1026)
      Dimension itally(1026)

      Dimension     n(512,6), nbe(512,3)
      Dimension alpha(512),  beta(512), gamma(512)

      Dimension  xxi(6), eet(6)

      Dimension xiq(20),etq(20),wq(20)

c---
c common blocks
c---

      common/points/p,ne
      common/elmnts/n,nbe
      common/albega/alpha,beta,gamma

      common/trq/xiq,etq,wq

c------------------------------------------
c Compute the pseudo-Laplacian at the nodes
c------------------------------------------

c---
c initialize
c---

      Do i=1,npts
        rlplcn(i) = 0.0
        itally(i) = 0
      End Do

c---
c loop over elements and over nodes
c---

      Do k=1,nelm    ! loop over elements

       i1 = n(k,1)     ! global node labels
       i2 = n(k,2)
       i3 = n(k,3)
       i4 = n(k,4)
       i5 = n(k,5)
       i6 = n(k,6)

       al = alpha(k)
       be = beta (k)
       ga = gamma(k)

       alc = 1.0D0-al
       bec = 1.0D0-be
       gac = 1.0D0-ga

c--
c triangle coordinates
c of the nodes
c---

       xxi(1) = 0.0
       eet(1) = 0.0

       xxi(2) = 1.0
       eet(2) = 0.0

       xxi(3) = 0.0
       eet(3) = 1.0

       xxi(4) = al
       eet(4) = 0.0

       xxi(5) = ga
       eet(5) = gac

       xxi(6) = 0.0
       eet(6) = be

c------------------------------------------------
c compute the pseudo-Laplacian of f at the nodes
c by interpolation
c------------------------------------------------

c---
c triangle coordinates
c of the nodes
c---

      Do i=1,6

        m = n(k,i)   ! global index of local node i on element k

        xi  = xxi(i)
        eta = eet(i)

        call sm_3d_df_interp
     + 
     +     (p(i1,1),p(i1,2),p(i1,3)
     +     ,p(i2,1),p(i2,2),p(i2,3)
     +     ,p(i3,1),p(i3,2),p(i3,3)
     +     ,p(i4,1),p(i4,2),p(i4,3)
     +     ,p(i5,1),p(i5,2),p(i5,3)
     +     ,p(i6,1),p(i6,2),p(i6,3)
     + 
     +     ,al,be,ga
     +     ,f(i1),f(i2),f(i3)
     +     ,f(i4),f(i5),f(i6)
     + 
     +     ,xi,eta
     +     ,rlapl
     +     )

        rlplcn(m) = rlplcn(m) + rlapl
        itally(m) = itally(m) + 1

      End Do

      End Do

c----
c average the flat-Laplacian
c at the nodes
c----

      Do i=1,npts

        par = float(itally(i))
        rlplcn(i) = rlplcn(i)/par

      End Do

c-------------------------------------------
c compute the flat bi-Laplacian at the nodes
c-------------------------------------------

      if(Ichoice.eq.2) then

c---
c initialize
c---

      Do i=1,npts
        rbilpl(i) = 0.0
        itally(i) = 0
      End Do

c---
c loop over elements and over nodes
c---

      Do k=1,nelm    ! loop over elements

       i1 = n(k,1)     ! global node labels
       i2 = n(k,2)
       i3 = n(k,3)
       i4 = n(k,4)
       i5 = n(k,5)
       i6 = n(k,6)

       al = alpha(k)
       be = beta (k)
       ga = gamma(k)

       alc = 1.0D0-al
       bec = 1.0D0-be
       gac = 1.0D0-ga

c--
c triangle coordinates
c of the nodes
c---

       xxi(1) = 0.0D0
       eet(1) = 0.0D0

       xxi(2) = 1.0D0
       eet(2) = 0.0D0

       xxi(3) = 0.0D0
       eet(3) = 1.0D0

       xxi(4) = al
       eet(4) = 0.0D0

       xxi(5) = ga
       eet(5) = gac

       xxi(6) = 0.0D0
       eet(6) = be

c------------------------------------------------
c compute the pseudo-bi-Laplacian of f at the nodes
c------------------------------------------------

c---
c triangle coordinates
c of the nodes
c---

      Do i=1,6

        m = n(k,i)   ! global index of local node i on element k

        xi  = xxi(i)
        eta = eet(i)

        call sm_3d_df_interp
     + 
     +      (p(i1,1),p(i1,2),p(i1,3)
     +      ,p(i2,1),p(i2,2),p(i2,3)
     +      ,p(i3,1),p(i3,2),p(i3,3)
     +      ,p(i4,1),p(i4,2),p(i4,3)
     +      ,p(i5,1),p(i5,2),p(i5,3)
     +      ,p(i6,1),p(i6,2),p(i6,3)
     + 
     +      ,al,be,ga
     +      ,rlplcn(i1),rlplcn(i2),rlplcn(i3)
     +      ,rlplcn(i4),rlplcn(i5),rlplcn(i6)
     + 
     +      ,xi,eta
     +      ,rbilap
     +      )

        rbilpl(m) = rbilpl(m) + rbilap
        itally(m) = itally(m) + 1

      End Do

      End Do

c----
c Average value of the flat-bi-Laplacian
c at the nodes
c----

      Do i=1,npts
        par = float(itally(i))
        rbilpl(i) = rbilpl(i)/par
      End Do

c-----------
      end if     ! End of computing the bi-Laplacian
c-----------

c----
c smooth
c---

      Do i=1,npts

       if(Ichoice.eq.1) then
        f(i) = f(i) + epsilon*rlplcn(i)
       else
        f(i) = f(i) - epsilon*rbilpl(i)
       end if

      End Do

c---
c done
c---

      return
      end
