      program sm_3d_df_dr

c========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c========================================

c-----------------------------------------------
c  Smoothing of a scalar function over a closed surface
c  described by triangulation.
c
c  Smoothing is effected by diffusion
c
c  SYMBOLS:
c  -------
c
c  p(i,j): coordinates of node i (j=1,2,3)
c
c  n(k,i): connectivity table: 
c          global labels of points for element k
c          i = 1, ..., 6
c
c  nbe(k,j): the three neighboring elements of element k (j=1,2,3)
c
c  ne(k,j):  ne(k,1) is the number of elements adjacent to point k
c            ne(k,2), ... are the elements numbers, j = 2, ..., 7
c            for this triangulation, up to six
c
c  npts:  total number of points
c  nelm:  total number of elements
c
c  alpha, beta, gamma: triangle mapping coefficients
c
c  vna: components of the averaged normal vector
c
c  f_sm: function to be smoothed
c------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension    p(1026,3)
      Dimension   ne(1026,7)
      Dimension  vna(1026,3)

      Dimension    n(512,6),nbe(512,3)
      Dimension alpha(512),beta(512),gamma(512)
      Dimension  arel(512),xmom(512), ymom(512),zmom(512)
      Dimension  crvm(512)

      Dimension zz(20),ww(20)
      Dimension xiq(20),etq(20),wq(20)

      Dimension  f_sm(1026),f_orig(1026)

c---
c common blocks
c---

      common/points/p,ne
      common/elmnts/n,nbe
      common/albega/alpha,beta,gamma
      common/geo1/arel,crvm
      common/geo2/vna

      common/zwl/zz,ww
      common/trq/xiq,etq,wq

      common/pii/pi,piq,pih,pi2,pi4,pi6,pi8

c---
c constants
c---

      pi  = 3.14159 265358D0

      piq = 0.25D0*pi
      pih = 0.50D0*pi
      pi2 = 2.00D0*pi
      pi4 = 4.00D0*pi
      pi6 = 6.00D0*pi
      pi8 = 8.00D0*pi

      null = 0
      oot  = 1.0D0/3.0D0

c-----------
c Input data
c-----------

  91  Continue

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to discretize an octahedron"
      write (6,*) " 2 to discretize an icosahedron"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Ioctaicos

      if(Ioctaicos.eq.0) Go to 99

      write (6,*)
      write (6,*) " Select the level of triangulation"
      write (6,*)
      write (6,*) " enter 0, 1, 2, 3; 99 to quit"
      write (6,*) " ----------------------------"
      read  (5,*) ndiv 

      if(ndiv.eq.99) Go to 99

      if(ndiv.gt.3)  then
        write (6,*)
        write (6,*) 'out of range; please try again.'
        write (6,*)
        Go to 91
      end if

      write (6,*)
      write (6,*) " Will use the m-point gauss-triangle rule"
      write (6,*) " for integration over each triangle"
      write (6,*)
      write (6,*) " Please enter m"
      write (6,*)
      write (6,*) " choose from 3, 4, 5, 6, 9, 12, 13"
      write (6,*) " 0 to quit"
      write (6,*) "----------"
      read  (5,*) mint

c----------------------------
c triangulate the unit sphere
c to generate the connectivity list
c----------------------------

      if(Ioctaicos.eq.1) then

       call trgl6_octa
     +
     +  (ndiv
     +  ,npts,nelm
     +  )

      else

      call trgl6_icos
     +
     +  (ndiv
     +  ,npts,nelm
     +  )

      end if
      
      write (6,*)
      write (6,*) "Number of points:   ",npts
      write (6,*)
      write (6,*) "Number of elements: ",nelm
      write (6,*)

c---------------------------------
c weights for triangle integration
c---------------------------------

      call gauss_trgl
     +
     +   (mint
     +   ,xiq,etq,wq
     +   )

c-----------------
c open output file
c-----------------

      open (1,file="sm_3d_df.out")

c-----------
c smoothing
c-----------

   92 Continue

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to smooth with the Laplacian"
      write (6,*) " 2 to smooth with the bi-Laplacian"
      write (6,*) " 0 to quit"
      write (6,*) "----------"
      read  (5,*) Ichoice

      if(Ichoice.eq.0) Go to 99

c-----------------------------------
c specify the function to be smoothed
c-------------------------------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to generate the surface function"
      write (6,*) "   on the unit sphere"
      write (6,*) " 2 to read from file sm_3d_df.inp"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Iread

      if(Iread.eq.0) Go to 99

c------------------------
      if(Iread.eq.1) then   ! read data
c------------------------

       write (6,*)
       write (6,*) " Select the function to be smoothed"
       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for f(x) = 1*(1+random noise)"
       write (6,*) " 2 for f(x) = x*(1+random noise)"
       write (6,*) " 3 for f(x) = y*(1+random noise)"
       write (6,*) " 4 for f(x) = z*(1+random noise)"
       write (6,*) " 0 to quit"
       write (6,*) "----------"
       read  (5,*) Icase

      if(Icase.eq.0) Go to 99

       write (6,*)
       write (6,*) " Please enter the amplitude of the random noise"
       write (6,*) "-----------------------------------------------"
       read  (5,*) amp_rn


        Do i=1,npts

         stoch = rand(0)
c        write (6,*) stoch

         if(Icase.eq.1) then
          f_sm(i) = 1.0*(1.0+amp_rn*stoch)
         elseif(Icase.eq.2) then
          f_sm(i) = p(i,1)*(1.0+amp_rn*stoch)
         elseif(Icase.eq.3) then
          f_sm(i) = p(i,2)*(1.0+amp_rn*stoch)
         elseif(Icase.eq.4) then
          f_sm(i) = p(i,3)*(1.0+amp_rn*stoch)
         end if

        End Do

c----------
      else if(Iread.eq.2) then   ! read data
c----------

       open (3,file="sm_3d_df.inp")

        read (3,*) npts
        Do i=1,npts
         read (3,*) idle,p(i,1),p(i,2),p(i,3),f_sm(i)
        End Do

       close (3)
    
c------------
      end if
c------------

c----------------------------
c Compute alpha, beta, gamma,
c for the xi-eta mapping
c over each triangle
c---------------------------

      Do k=1,nelm

       i1 = n(k,1)
       i2 = n(k,2)
       i3 = n(k,3)
       i4 = n(k,4)
       i5 = n(k,5)
       i6 = n(k,6)

       call abc 
     +
     +   (p(i1,1),p(i1,2),p(i1,3)
     +   ,p(i2,1),p(i2,2),p(i2,3)
     +   ,p(i3,1),p(i3,2),p(i3,3)
     +   ,p(i4,1),p(i4,2),p(i4,3)
     +   ,p(i5,1),p(i5,2),p(i5,3)
     +   ,p(i6,1),p(i6,2),p(i6,3)
     +   ,alpha(k),beta(k),gamma(k)
     +   )

      End Do

c------------------------------------------------
c compute:
c
c     surface area of the individual elements
c     x, y, and z surface moments over each element
c     total surface area and volume
c     mean curvature over each element
c     average normal vector at the nodes
c------------------------------------------------

      call elm_geom 
     +
     +  (nelm,npts,mint
     +  ,xmom,ymom,zmom
     +  ,area,vlm
     +  ,cx,cy,cz
     +  )

      area = area/(pi4)        ! normalize
      vlm  = vlm /(pi4/3.0)    ! normalize

      write (6,*)
      write (6,110) area
      write (6,111) vlm
      write (6,*)

c------------------------------------
c display points and surface function
c------------------------------------
c
c     write (1,*) npts," before smoothing"
c
c      Do i=1,npts
c      write (6,100) i,p(i,1),p(i,2),p(i,3)
c    +              ,f_sm(i)
c      write (1,100) i,p(i,1),p(i,2),p(i,3)
c    +              ,f_sm(i)
c     End Do
c

c---------------------------------
c save the function to be smoothed
c---------------------------------

      Do i=1,npts
       f_orig(i) = f_sm(i)
      End Do

c----------------
c begin smoothing
c----------------

      write (6,*)
      write (6,*) " Enter the smoothing coefficient"
      write (6,*) " 99 to quit"
      write (6,*) "-----------"
      read  (5,*) epsilon

      if(epsilon.eq.99) Go to 99

      write (6,*)
      write (6,*) " Enter the number of smoothing passes "
      write (6,*) " 0 to quit"
      write (6,*) "----------"
      read  (5,*) Npass

      if(Npass.eq.0) Go to 99

      Do Ipass=1,Npass

        call sm_3d_df
     +
     +    (nelm,npts
     +    ,Ichoice
     +    ,epsilon
     +    ,f_sm
     +    )

      End Do

c---------------
c done smoothing
c---------------

      write (1,*) npts," points, function before and after smoothing"

      Do i=1,npts
       write (1,100) i,p(i,1),p(i,2),p(i,3),f_orig(i),f_sm(i)
      End Do

      write (6,*)
      write (6,*) " Display on screen ?"
      write (6,*) " -------------------"
      write (6,*)
      read  (5,*) Idisp

      if(Idisp.eq.1) then

       write (6,*) npts," points, before and after smoothing"
       Do i=1,npts
        write (6,100) i,p(i,1),p(i,2),p(i,3),f_orig(i),f_sm(i)
       End Do

      end if

c--------------------------
c Replace smoothed function
c with the original function
c
c and return to the main menu
c----------------------------

      Do i=1,npts
       f_sm(i) = f_orig(i)
      End Do

      Go to 92
     
c---
c Done
c---

  99  Continue

      write (1,100) null
      close (1)

  100 Format(1x,i4,10(1x,f8.4))
  101 Format(10(1x,f12.5))
  102 Format(1x,i4,1x,i4,10(f15.10))

  110 Format(" Surface Area :",F15.10)
  111 Format(" Volume       :",F15.10)

      Stop
      End
