      program sm_3d_dr

c================================================
c FDLIB, BEMLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c----------------------------------------

c-----------------------------------------------
c  Smoothing of a scalar function 
c  defined over a closed surface
c  described by triangulation.
c
c  The smoothing is done by mapping the surface onto
c  the unit sphere, and then applying spectrum truncation
c
c  SYMBOLS:
c  -------
c
c  p(i,j): coordinates of node i (j=1,2,3)
c
c  n(k,i): connectivity table: 
c          global labels of points for element k
c          i = 1, ..., 6
c
c  nbe(k,j): the three neighboring elements of element k (j=1,2,3)
c
c  ne(k,j):  ne(k,1) is the number of elements adjacent to point k
c            ne(k,2), ... are the elements numbers, j = 2, ..., 7
c            for this triangulation, up to six
c
c  npts:  total number of points
c  nelm:  total number of elements
c
c  alpha, beta, gamma: triangle mapping coefficients
c
c  f_sm: function to be smoothed
c
c  afl, bfl: coefficient of the Fourier-Legendre
c            expansion
c
c smm:  smoothing matrix
c------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension    p(1026,3)
      Dimension   ne(1026,7)

      Dimension    n(512,6),nbe(512,3)
      Dimension alpha(512),beta(512),gamma(512)

      Dimension xiq(20),etq(20),wq(20)

      Dimension  phi(1026)
      Dimension rmpp(1026,0:32,0:32)

      Dimension  f(1026),f_sm(1026)

      Dimension  smm(1026,1026)

      Dimension afl(0:32,0:32),bfl(0:32,0:32)

c--------------
c common blocks
c--------------

      common/points/p,ne
      common/elmnts/n,nbe

      common/albega/alpha,beta,gamma

      common/trq/xiq,etq,wq

      common/fleg/phi,rmpp
      common/flegc/afl,bfl

      common/pii/pi,piq,pih,pi2,pi4,pi6,pi8

c----------
c constants
c----------

      pi = 3.14159 265358 D0

      piq = 0.25D0*pi
      pih = 0.50D0*pi
      pi2 = 2.00D0*pi
      pi4 = 4.00D0*pi
      pi6 = 6.00D0*pi
      pi8 = 8.00D0*pi

      null = 0

c-----------
c input data
c-----------

  91  Continue

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to discretize an octahedron"
      write (6,*) " 2 to discretize an icosahedron"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Ioctaicos

      if(Ioctaicos.eq.0) Go to 99

      write (6,*)
      write (6,*) " Select the level of triangulation"
      write (6,*)
      write (6,*) " Enter 0, 1, 2, 3; 99 to quit"
      write (6,*) " ----------------------------"
      read  (5,*) Ndiv 

      if(ndiv.eq.99) Go to 99

      if(ndiv.gt.3)  then
        write (6,*)
        write (6,*) 'out of range; please try again.'
        write (6,*)
        Go to 91
      end if

      write (6,*)
      write (6,*) " Will use the m-point gauss-triangle rule"
      write (6,*) " for integration over each triangle"
      write (6,*)
      write (6,*) " Please enter m"
      write (6,*) " choose from 1, 3, 4, 6, 7, 9, 12, 13"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) mint

      if(mint.eq.0) Go to 99

      write (6,*)
      write (6,*) " Display the function on screen?"
      write (6,*) " -------------------------------"
      write (6,*)
      read  (5,*) Idisp

c-----------------
c open output file
c-----------------

      open (1,file="sm_3d_tr.out")

c---------------------------------
c weights for triangle integration
c---------------------------------

      call gauss_trgl
     +
     +  (mint
     +  ,xiq,etq,wq
     +  )

c----------------------------
c triangulate the unit sphere
c----------------------------

      if(Ioctaicos.eq.1) then

       call trgl6_octa
     +
     +  (ndiv
     +  ,npts,nelm
     +  )

      else

      call trgl6_icos
     +
     +  (ndiv
     +  ,npts,nelm
     +  )

      end if

      write (6,*)
      write (6,*) "Number of points:   ",npts
      write (6,*)
      write (6,*) "Number of elements: ",nelm
      write (6,*)

c----------------------------
c Compute alpha, beta, gamma,
c for the xi-eta mapping
c over each triangle
c---------------------------

      Do k=1,nelm

       i1 = n(k,1)
       i2 = n(k,2)
       i3 = n(k,3)
       i4 = n(k,4)
       i5 = n(k,5)
       i6 = n(k,6)

       call abc 
     +
     +   (p(i1,1),p(i1,2),p(i1,3)
     +   ,p(i2,1),p(i2,2),p(i2,3)
     +   ,p(i3,1),p(i3,2),p(i3,3)
     +   ,p(i4,1),p(i4,2),p(i4,3)
     +   ,p(i5,1),p(i5,2),p(i5,3)
     +   ,p(i6,1),p(i6,2),p(i6,3)
     +   ,alpha(k),beta(k),gamma(k)
     +   )

      End Do

c-----------------------------------
c specify the function to be smoothed
c-------------------------------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*) 
      write (6,*) " 1 to generate the surface function"
      write (6,*) " 2 to read from file sm_3d.inp"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Iread

      if(Iread.eq.0) Go to 99

c------------------------
      if(Iread.eq.1) then   ! read data
c------------------------

       write (6,*)
       write (6,*) " Select the function to be smoothed"
       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for f(x) = 1"
       write (6,*) " 2 for f(x) = Full Series"
       write (6,*) " 0 to quit"
       write (6,*) "----------"
       read  (5,*) Icase

       if(Icase.eq.0) Go to 99

c-----
       if(Icase.eq.1) then
c-----

        Do i=1,npts
          f(i) = 1.0D0
        End Do

c-----
       else if(Icase.eq.2) then
c-----

        j_max = 30

        call fleg_prep (npts,j_max)

        j_max = 10

        Do j=0,j_max
          Do m=0,j
           afl(j,m) = 1.0D0
           bfl(j,m) = 1.0D0
         End Do
        End Do

        Do i=1,npts
         f(i) = 0.5D0*afl(0,0)*rmpp(i,0,0)
         Do j=1,j_max
          f(i) = f(i) + 0.5D0*afl(j,0)*rmpp(i,j,0)
          Do m=1,j
           arg = m*phi(i)
           f(i) = f(i) + rmpp(i,j,m)*(afl(j,m)*dcos(arg)
     +                               +bfl(j,m)*dsin(arg))
          End Do
         End Do
        End Do

c-----
       end if
c-----

c-----------------------------
      else if(Iread.eq.2) then   ! read data
c-----------------------------

       open (3,file="sm_3d_tr.inp")

        read (3,*) npts
        Do i=1,npts
         read (3,*) idle,ccc,ccc,ccc,f(i)
        End Do

       close (3)
    
c------------
       end if
c------------

c------------------------------------
c Display points and surface function
c------------------------------------
c
c     write (1,*) npts," before smoothing"
c
c      Do i=1,npts
c      write (6,100) i,p(i,1),p(i,2),p(i,3)
c    +              ,f(i)
c      write (1,100) i,p(i,1),p(i,2),p(i,3)
c    +              ,f(i)
c     End Do
c
c---------------------------------

c-----------
c smoothing
c-----------

   92 Continue

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to smooth by direct truncation"
      write (6,*) " 2 to use a smoothing matrix"
      write (6,*) " 0 to quit"
      write (6,*) "----------"
      read  (5,*) Ichoice

      if(Ichoice.eq.0) Go to 99

c--------------------------
      if(Ichoice.eq.1) then             ! direct smoothing
c--------------------------

      advice   = (-3.0+sqrt((4.0*npts-8.0)+9.0))/2.0
      J_advice = Nint(advice)

  93  Continue

      write (6,*)
      write (6,*) " Enter the j truncation level"
      write (6,*) " for the Legendre decomposition"
      write (6,*) " in the range: 0, 1, ...",J_advice
      write (6,*) " 99 to quit"
      write (6,*) "-----------"
      read  (5,*) j_max

      if(j_max.eq.99) Go to 99

c--------
c prepare
c--------

      call fleg_prep (npts,j_max)

c---------------------------------
c compute the Fourier coefficients
c---------------------------------

       call fleg_cf
     +
     +  (nelm,npts
     +  ,mint
     +  ,f
     +  ,j_max
     +  )

c---
c display
c---

      write (6,*)
      write (6,*)  " Cosine coefficients:"
      write (6,*)

      Do j=0,j_max
       write (6,601) (afl(j,m),m=0,j)
      End Do

      write (6,*)
      write (6,*)  " Sine coefficients:"
      write (6,*)

      Do j=0,j_max
       write (6,601) (bfl(j,m),m=0,j)
      End Do

c-------
c smooth
c-------

      j_trunc = j_max

      call sm_3d_tr
     +
     +   (npts
     +   ,f
     +   ,j_trunc
     +   ,f_sm
     +   )

c---
c display
c---

      if(Idisp.eq.1) then

       write (6,*)
       write (6,*) npts," points, before and after smoothing"
       write (6,*)

       Do i=1,npts
        write (6,100) i,p(i,1),p(i,2),p(i,3),f(i),f_sm(i)
       End Do

      End If

      Go to 93

c-------------------------------
      else if(Ichoice.eq.2) then
c-------------------------------

c------------------------------
c smm: smoothing matrix
c
c    Smoothing is effected by the projection:
c
c    f_sm(i) = smm(i,j) f(j)
c
c--------------------------------

      if(Ndiv.eq.2) open (2,file="sm_3d_tr.tbl2")
      if(Ndiv.eq.3) open (2,file="sm_3d_tr.tbl3")

      write (6,*) " reading the smoothing matrix"

      read (2,*) j_trunc

       Do i=1,npts
        Do k=1,npts
          read (2,*) idle,idle,smm(k,i)
        End Do
       End Do

       close (2)

      write (6,*) " Done"

c---
c smoothing
c---

      write (6,*) " Smoothing"

      Do i=1,npts
       f_sm(i) = 0.0D0
       Do j=1,npts
        f_sm(i) = f_sm(i) + smm(i,j)*f(j)
       End Do
      End Do

c---
c display
c---

      write (6,*) " Done"

      write (6,*)
      write (6,*) " Display on screen ?"
      write (6,*) " -------------------"
      write (6,*)
      read  (5,*) Idisp

      if(Idisp.eq.1) then

       write (6,*)
       write (6,*) npts," points, before and after smoothing"
       write (6,*)

       Do i=1,npts
        write (6,100) i,p(i,1),p(i,2),p(i,3),f(i),f_sm(i)
       End Do

      end if
     
c-----------
      end if
c-----------

c---------------
c Done smoothing
c---------------

c--------------------------
c Replace the smoothed function
c with the original function
c
c and return to the main menu
c----------------------------

c     Do i=1,npts
c      f(i) = f_sm(i)
c     End Do

c-----------------
c return to repeat
c-----------------

      Go to 92
     
c-----
c done
c-----

  99  Continue

      write (1,100) null
      close (1)

  100 Format(1x,i4,40(1x,f10.6))
  601 Format(10(1x,f12.5))
  102 Format(1x,i4,1x,i4,40(f15.10))

  110 Format(" Surface Area :",F15.10)
  111 Format(" Volume       :",F15.10)

  120 Format(1x,i4,1x,i4,1x,f16.11)

      stop
      end
