%----
clear all;
close all;
%----

%----
% Solution branches of a sessile or pendant drop
%----

%----
% parameters
%----

a = 1.0;      % drop radius
Isp = -1;     % pendant drop
psi_max =0.25*pi;

npts1 = 32;
npts1 = 24;
npts2 = 48;

ratio = 0.9;
maxiter = 10;  % for secant iterations
tol = 0.0000001;  % for secant iterations

%---
% prepare
%---

 volume = 4.0*pi*a^3/3.0;

%-----------------
% family parameters
%-----------------

%---
capls=8.0;
Nloop=2*2*2*2*2*2*64;
Nplot=2*32;
Bmin=1.00;
Bmax=2.6;
DB=0.0020;
%---

%---
capls=2.5;
Nloop=2*2*2*2*2*2*128;
Nplot=32;
Bmin=1.25;
Bmax=3.5;
DB=0.0020;
%---

%---
capls=2.5;
Nloop=2*2*2*2*2*2*128;
Nplot=2*32;
Bmin=1.25;
Bmax=3.5;
DB=0.0010;
%---

%---
capls=1.0;
Nloop=2*2*2*2*2*2*2*128;
Nplot=4*2*32;
Bmin=1.5;
Bmax=5.0;
DB=0.0020;
%---

%---
capls=1.5;
Nloop=2*2*2*2*2*2*2*128;
Nplot=4*2*32;
Bmin=1.2;
Bmin=1.2;
Bmax=4.5;
DB=0.0020;
%
%npts2=16;
%psi_max =0.10*pi;
%---

%---
capls=16.0;
Nloop=2*2*2*2*2*2*64;
Nplot=2*32;
Bmin=1.00;
Bmax=2.4;
DB=0.0020;
%---

%---
capls=4.0;
Nloop=2*2*2*2*2*2*64;
Nplot=4*32;
Bmin=1.00;
Bmax=3.25;
DB=0.0020;
DB=0.0010;
%---

%---
capls = 2.0;
Nloop = 2*2*2*2*2*2*128;
Nplot = 32;
Bmin = 1.00;
Bmax = 4.0;
DB = 0.0020;
Nplot = 2*2*2*32;
Nplot = 500;
Nplot = 350;
%---

%---
% prepare to plot
%---

 figure(1)
 hold on
 xlabel('y/a','fontsize',15)
 ylabel('x/a','fontsize',15)
 set(gca,'fontsize',15)
 axis equal
 box on

 xwall(1) =-1.8;
 ywall(1) = 0;
 xwall(2) = 1.8;
 ywall(2) = 0;

 if(Isp==1)
  patch([xwall xwall(2) xwall(1)] ,[ywall ywall(2)-0.2 ywall(1)-0.2],'r')
 else
  patch([xwall xwall(2) xwall(1)] ,[ywall ywall(2)+0.2 ywall(1)+0.2],'r')
 end
 plot(xwall,ywall,'k')

%---
% prepare to loop
%---

B = Bmin;
Next(1) = 2.0;

Iplot = Nplot;

%============
for Iloop=1:Nloop  % loop over B
%============

 B = B+DB;

 Ic = 1; % counter

 [x,s,Ntot,vlm,slope] = drop_ax1_ode ...
   ...
   (npts1 ...
   ,capls ...
   ,Isp ...
   ,psi_max ...
   ,B ...
   ,ratio ...
   ,Next(1) ...
   ,npts2 ...
   );

  error(Ic) = vlm-volume;

  Ic=2;
  Next(Ic) = Next(Ic-1)+0.10;

 [x,s,Ntot,vlm,slope] = drop_ax1_ode ...
   ...
   (npts1 ...
   ,capls ...
   ,Isp ...
   ,psi_max ...
   ,B ...
   ,ratio ...
   ,Next(2) ...
   ,npts2 ...
   );

 error(Ic) = vlm-volume;

%---------------------------------------
% iterate on Next using the secant method
% until convergence
%---------------------------------------

   Iconverged = 0;

   for iter=1:maxiter

    Ic = Ic+1;

%---
% secant updating
%---

    Icb = Ic-2;
    Ica = Ic-1;

    dedc = (error(Ica)-error(Icb))/(Next(Ica)-Next(Icb));
    Next(Ic) = Next(Ica)-error(Ica)/dedc;

   [x,s,Ntot,vlm,slope] = drop_ax1_ode ...
    ...
    (npts1 ...
    ,capls ...
    ,Isp ...
    ,psi_max ...
    ,B ...
    ,ratio ...
    ,Next(Ic) ...
    ,npts2 ...
    );

     error(Ic) = vlm-volume;
     err = abs(error(Ic));

    if(err<tol)
     Iconverged = 1;
     break
    end

%----
  end
%----

  err
%  Iconverged
%  pause

 Next(1) = Next(Ic);

 xplot(Iloop) = a*B;
 yplot(Iloop) = atan(slope)/pi+0.5;

% if(abs(yplot(Iloop)-0.5)<0.01)
% if(abs(yplot(Iloop)-0.75)<0.01)
% if(abs(yplot(Iloop)-0.6)<0.01)
  if(abs(yplot(Iloop)-0.3)<0.01)
   atan(slope)/pi+0.5
%   pause
  end

 if(Iloop>1)
  if(abs(yplot(Iloop)-yplot(Iloop-1))>0.1)
%  figure
%  hold on
  end
 end

 if(Iplot==Nplot)
  if(Iconverged==1)
   figure(1)
   x = x-x(Ntot+1); % shift the profile
   plot( s,Isp*x,'k-','LineWidth',3)
   plot(-s,Isp*x,'k-','LineWidth',3)
%   plot( s,Isp*x,'o-')
%   plot(-s,Isp*x,'o-')
  end
  Iplot=0;
 end

 Iplot = Iplot+1;

 if(B>Bmax)
  break
 end

%============
end
%============

figure(2)
plot(xplot,yplot,'k-')
xlabel('aB','fontsize',15)
ylabel('\alpha/\pi','fontsize',15)
set(gca,'fontsize',15)
axis([1 Bmax 0 1])

%-------
% record
%-------

file1 = fopen('drop_ax1.out','w');

fprintf(file1,'\n');
fprintf(file1,'x=[');
fprintf(file1,'\n');

for i=1:Iloop
 fprintf(file1,'%12.8f ',xplot(i));
 fprintf(file1,'\n');
end

fprintf(file1,'];');
fprintf(file1,'\n');
fprintf(file1,'y=[');
fprintf(file1,'\n');

for i=1:Iloop
 fprintf(file1,'%12.8f ',yplot(i));
 fprintf(file1,'\n');
end

fprintf(file1,'];');
fclose(file1);
