      program men_2d_plate

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c----------------------------------------------
c
c Shape of a semi-infinite meniscus attached to
c an inclined plate that is partially immersed in a pool.
c
c Procedure is described in section 4.3 
c of Pozrikidis (1997, pp. 166-169)
c
c----------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(800),y(800)

c----------
c constants
c----------

      pi  = 3.14159 265359

      piq = 0.25*pi
      pih = 0.50*pi

      null = 0
      none = 1
      ntwo = 2

      zero = 0.0
      two  = 2.0

c--------
c prepare
c--------

      open(1,file="PLOTDAT",status="unknown")

c------------
c Preferences
c------------

      write (6,*)
      write (6,*) " Choose the data input mode"
      write (6,*) 
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 to read data from file: men_2d_plate.dat"
      write (6,*) " 2 to type in data"
      write (6,*) " -------------------------------------------"
      read  (5,*) Ienrd

      If(Ienrd.eq.0) Go to 99

c------------------------
      If(Ienrd.eq.2) then
c------------------------

      write (6,*)
      write (6,*) " Please enter the acceleration of gravity"
      write (6,*) "                               0  to quit"
      write (6,*) " ----------------------------------------"
      read  (5,*) gac

      If(gac.eq.0) Go to 99

      write (6,*)
      write (6,*) 'Please enter the density of the pool fluid'
      write (6,*) '------------------------------------------'
      read  (5,*) rhop

      write (6,*)
      write (6,*) 'Please enter the density of the ambient fluid'
      write (6,*) '---------------------------------------------'
      read  (5,*) rhoa

      write (6,*)
      write (6,*) 'Please enter the interfacial tension'
      write (6,*) '------------------------------------'
      read  (5,*) gamma

      write (6,*)
      write (6,*) 'Enter the plate inclination angle beta'
      write (6,*) '                  in multiples of pi/2'
      write (6,*) '--------------------------------------'
      read  (5,*) beta

      write (6,*)
      write (6,*) 'Please enter contact angle alpha'
      write (6,*) '            in multiples of pi/2'
      write (6,*) '--------------------------------'
      read  (5,*) alpha

      write (6,*) 'Enter the number of interfacial marker points'
      write (6,*) '---------------------------------------------'
      read  (5,*) npts

c----------
       Else
c----------

        open (2,file="men_2d_plate.dat")

         read (2,*) gac
         read (2,*) rhop
         read (2,*) rhoa
         read (2,*) gamma
         read (2,*) beta
         read (2,*) alpha
         read (2,*) npts

        close (2)

c----------
      End If
c----------

c-----------------------------
c check the density difference
c-----------------------------

      drho = rhop-rhoa

      If(drho.lt.0.0) then
       write (6,*)
       write (6,*) " Sorry: density of the pool must be higher"
       write (6,*) "        than that of the ambient fluid"
       write (6,*)
       Go to 99
      End If

c-------------
c preparations
c-------------

      npts1 = npts+1

      capls = gamma/(gac*abs(drho))   ! square of the 
                                      !capillary length
      capl  = sqrt(capls)

      alpha = alpha*pih
      beta  = beta *pih

c------------------
c more preparations
c------------------

  94  Continue

      thcl  = alpha+beta
      cst   = cos(thcl)

      If(cst.gt.0.9999999.or.cst.le.-0.99999999) then
       write (6,*)
       write (6,*) " Free surface is flat"
       write (6,*)
       write (6,*) " No need for further computation"
       write (6,*)
       stop
      End If

c-------------------------------------
c compute magnitude of meniscus rise h
c-------------------------------------

      If(thcl.gt.pih.and.thcl.lt.3.0*pih) then
       h = sqrt(2.0*capls*(1.0-abs(cst)))
      Else
       h = sqrt(2.0*capls*(1.0+abs(cst)))
      End If

c---
c sign of meniscus rise
c---

      If(thcl.lt.pi) then
       h = abs(h)            ! meniscus goes up
      Else
       h =-abs(h)            ! meniscus goes down
      End If

      write (6,*)
      write (6,*) 'Meniscus rise = ',h
      write (6,*)

c------------------------------------------
c integrate meniscus equation dx/df = G(f)
c from f=h to f=0
c using the modified Euler method with constant step
c------------------------------------------

      df  = h/npts        ! note that df can be positive or negative
      dfh = 0.5*df

      y(1) = h
      x(1) = 0.0

      Do i=1,npts-1

       fred  = y(i)/capl
       freds = fred**2
       xp    = (2.0-freds)/sqrt(4.0-freds)/fred

       xsv  = x(i)          ! save
       xpsv = xp

       i1 = i+1

       y(i1) = y(i)-df
       x(i1) = x(i)+xp*df

       fred  = y(i1)/capl
       freds = fred**2
       xp    = (2.0-freds)/(fred*sqrt(4.0-freds))

       x(i1) = xsv+(xpsv+xp)*dfh

      End Do

c-----------------
c printing session
c-----------------

c-------------------------
c print the inclined plate
c-------------------------

      xxpp = -h/tan(beta)

      write (1,100) ntwo
      write (1,100) none,xxpp,zero
      write (1,100) ntwo,zero,h

c---------------
c print the pool
c---------------

      tmp1 = -h/tan(beta)-0.5
      write (1,100) ntwo
      write (1,100) none,tmp1,zero
      write (1,100) ntwo,x(npts1),zero

c---------------------------
c print the meniscus profile
c---------------------------

      write (1,100) npts
      Do i=1,npts
       write (1,100) i,x(i),y(i)
c      write (6,100) i,x(i),y(i)
      End Do

c--------------------------
c Display converged profile
c and allow for printing
c--------------------------

      write (6,*) "beta=",beta
      write (6,*) "alpha=",alpha

c-------
c repeat
c-------

      write (6,*)
      write (6,*) " Enter new contact angle in multiples of pi/2"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " --------------------------------------------"
      read  (5,*) alpha

      If(alpha.eq.0) Go to 99

      Go to 94

c-----------
c pack it in
c-----------

  99  Continue

      write (1,100) null
      close (1)

c-----
c Done
c-----

  100 Format (1x,i3,5(2x,f15.10))

      Stop
      End
