%-------
clear all
close all
%------

global A
global Nxi Net
global xi0 ximax snhxi0 cshxi0 
global xi cshxi snhxi
global eta cseta sneta
global Niter tolit
global capls
global x y z J
global forcex forcey forcez torqux torquy torquz
global cang
global cangle

%=======================================
% meniscus in the exterior of an ellipse
% in the yz plane
%=======================================

Iplot_grid = 1;
Iplot_grid = 0;

Iplot_shape = 0;
Iplot_shape = 1;

%---
% parameters
%---

%---
problem = 1;  % fixed contact line
%---

%---
problem = 2;  % fixed contact angle
cang = 0.25*pi; % specify the contact angle for problem 2
%---

tolit = 0.000001; % tolerance
Niter = 20000;

c = 1.0; % second ellipse (major) axis

xcline = 1.0; % elevation of the contact line

%---
% case B: alpha = 0.125pi
%
%capl =  5.6295*c;
%elv  = 2.0774*c;
capl = 4.5036*c;
elv = 1.8845*c;
%---

%---
% case C: alpha = 0.375pi
%
%capl =  5.6295*c;
%elv = 0.7539*c;
capl = 4.5036*c;
elv = 0.6792*c;
%---

%---
% case A: alpha = 0.25pi
%
capl = 4.5036*c;
elv = 1.3210*c;
%---

%=====
% prepare
%=====

capls = capl*capl;  % square of the capillary length

%---
% prepare for graphics
%---

[xcyl,ycyl,zcyl] = cylinder(1.0,16);

%=============
%for loopb=1:6  % over minor axis
%for loopb=3:6  % over minor axis
%for loopb=1:1  % over minor axis
 for loopb=2:2  % over minor axis
%=============

 if(loopb==1)
   b = 0.50*c; % first axis
 elseif(loopb==2)
   b = 0.60*c; % first axis
 elseif(loopb==3)
   b = 0.70*c; % first axis
 elseif(loopb==4)
   b = 0.80*c; % first axis
 elseif(loopb==5)
   b = 0.90*c; % first axis
 elseif(loopb==6)
   b = 0.99*c; % first axis
 end

%---
% compute xi0 and A
%---

xi0 = atanh(b/c);
snhxi0 = sinh(xi0);
cshxi0 = cosh(xi0);
A = b/snhxi0;

%---
% set ximax
%---

ximax = log(32.0*c/A);

%=============
%for ngrid=1:3
 for ngrid=2:2
%for ngrid=1:1
%=============

if(ngrid==1) Np=1; end
if(ngrid==2) Np=2; end
if(ngrid==3) Np=4; end

%----------
% divisions
%----------

Nxi = Np*8;
Net = Np*16;

%---
% grid
%---

 Dxi = (ximax-xi0)/Nxi;

 for i=1:Nxi+1
  xi(i) = xi0+(i-1.0)*Dxi;
  snhxi(i) = sinh(xi(i));
  cshxi(i) = cosh(xi(i));
 end

 Deta = 2.0*pi/Net;

 for j=1:Net+1
  eta(j) = (j-1.0)*Deta;
  cseta(j) = cos(eta(j));
  sneta(j) = sin(eta(j));
 end

%---
% grid points
%---

 for i=1:Nxi+1
  for j=1:Net+1
   y(i,j) = A*snhxi(i)*cseta(j);
   z(i,j) = A*cshxi(i)*sneta(j);
   J(i,j) = A*sqrt(cshxi(i)^2-sneta(j)^2);
  end
 end

%---
% polar angle around the cylinder
%---

 ycnt = 0.0;
 zcnt = 0.0;

 for j=1:Net
  rr = sqrt((y(1,j)-ycnt)^2+(z(1,j)-zcnt)^2);
  theta(j) = acos((y(1,j)-ycnt)/rr);
  if(z(1,j)<0.0)
   theta(j) = 2*pi-theta(j);
  end
 end

 theta(Net+1) = 2*pi+theta(1);

%---
% initialize the interfacial elevation
%---

for i=1:Nxi+1
 for j=1:Net+2
  x(i,j) = 0.0;
 end
end

%---
% plot the grid
%---

if(Iplot_grid==1)

 figure(100)
 colormap('gray')

 hold on;

 for i=1:Nxi+1
  plot(y(i,:),z(i,:),'k')
 end
 for j=1:Net
  plot(y(:,j),z(:,j),'k')
 end

%for i=1:Nxi+1
% plot3(y(i,:),z(i,:),J(i,:))
%end
%for j=1:Net
% plot3(y(:,j),z(:,j),j(:,j))
%end
%for j=1:Net
% plot3(y(:,j),z(:,j),J(:,j))
%end

 xlabel('y','fontsize',15);
 ylabel('z','fontsize',15);
 zlabel('x','fontsize',15);
 set(gca,'fontsize',15)
 axis square
 axis([-2 2 -2 2 -2 2])
%view(140,30)

%---
end
%---

%=========
% contact line
%=========

for i=1:Net
 xcontact(i) = xcline*y(1,i);  % rotated
 xcontact(i) = elv;            % constant height
end

%==============
 if(problem==1)
%==============

 obj = fellipse(xcontact);

  figure(loopb)
  hold on
  plot(theta(1:Net+1)/(2*pi),cangle(1:Net+1)/pi,'k')
  axis([0 1 0.0 0.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\alpha/\pi','fontsize',15);
  set(gca,'fontsize',15)
  box on

%==============
 elseif(problem==2)
%==============

 Jeps = 0.01;
 obj = fellipse(xcontact);
 Iamdone = 0;

 % Newton iterations

 %~~
 for iterate = 1:10
 %~~

  iterate

  stck = 0.0;

  for i=1:Net
   tasty = abs(obj(i));
   if(tasty>stck) stck=tasty; end
  end

  disp(['men_ell: Newton residual= ',num2str(stck)]);

  if(stck<0.0001)
    Iamdone = 1;
   break;
  end

%if(iterate==1)

  for j=1:Net
   xcontact(j) = xcontact(j) + Jeps;
   objtmp = fellipse(xcontact);
   for i=1:Net
    Jac(i,j) = (objtmp(i)-obj(i))/Jeps;
   end
   xcontact(j) = xcontact(j) - Jeps;
  end

%end

 xcontact = xcontact - obj/Jac';

 obj = fellipse(xcontact);

%---
% plot the contact angle
%---

  figure(loopb)
  hold on
  plot(theta(1:Net+1)/(2*pi),cangle(1:Net+1)/pi,'k')
  plot(theta(1:Net+1)/(2*pi),cangle(1:Net+1)/pi,'k--')
  axis([0 1 0.0 0.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\alpha/\pi','fontsize',15);
  set(gca,'fontsize',15)
  box on

 %~~
  end % of iterate
 %~~

  if(Iamdone==0) disp('men_ell: outer iterations did not converge'); end

%===
 end % of problem
%===

  forcex

%---
% hold
%---

 for j=1:Net+1
  hold_Net(ngrid) = Net;
  hold_theta(ngrid,j) = theta(j);
  hold_cangle(ngrid,j) = cangle(j);
 end

 hold_ft(ngrid,:) = [forcex forcey forcez ...
                     torqux torquy torquz];

%===========
end % of Np loop
%===========

%---
% Richardson extrapolation
% for the force
%---

 if(ngrid==3)

  fx_extra = rich (hold_ft(1,1),hold_ft(2,1),hold_ft(3,1));
  ftext = [fx_extra]

 end

%---
% plot the contact angle or contact line elevation
% distribution for the finest solution
%---

if(problem ==1)

  figure(110)
  hold on
  plot(theta(1:Net+1)/(2*pi),cangle(1:Net+1)/pi,'k')
  axis([0 1 0.25 0.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\alpha/\pi','fontsize',15);
  set(gca,'fontsize',15)
  box on

elseif(problem ==2)

  figure(110)
  hold on
  plot(theta(1:Net+1)/(2*pi),x(1,1:Net+1)/c,'k')
  axis([0 1 0.0 1.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\zeta/c','fontsize',15);
  set(gca,'fontsize',15)
  box on

end

%-------
% plot the finest solution
%-------

 if(Iplot_shape==1)

 figure(loopb+20)

 colormap('gray')
 colormap('bone')
 hold on;

%for i=1:Nxi+1
% plot3(y(i,:),z(i,:),x(i,1:Net+1))
%end
%for j=1:Net
% plot3(y(:,j),z(:,j),x(1:Nxi+1,j))
%end

 for i=1:Nxi
  for j=1:Net
   patch([y(i,j), y(i,j+1), y(i+1,j+1),  y(i+1,j)] ...
        ,[z(i,j), z(i,j+1), z(i+1,j+1),  z(i+1,j)] ...
        ,[x(i,j), x(i,j+1), x(i+1,j+1),  x(i+1,j)] ...
        ,'y')
%       [x(i,j), x(i,j+1), x(i+1,j+1),  x(i+1,j)]);
  end
 end

 colormap('copper')

 for j=1:Net
   ptc = patch([y(1,j), y(1,j+1), y(1,j+1),  y(1,j)] ...
              ,[z(1,j), z(1,j+1), z(1,j+1),  z(1,j)] ...
              ,[0, 0, 0.5+x(1,j+1),  0.5+x(1,j)] ...
              ,100*[0, 0, 0.5+x(1,j+1),  0.5+x(1,j)] ...
              );
 %set(ptc,'facecolor','red','edgecolor','none');
% set(ptc,'edgecolor','none');
  set(ptc,'edgecolor','k');
  %camlight; lighting gouraud;
 end

 xlabel('y/c','fontsize',15);
 ylabel('z/c','fontsize',15);
 zlabel('x/c','fontsize',15);
 set(gca,'fontsize',15)

% cylinder

%surf(b*ycyl, c*xcyl, 2.0*(zcyl-0.0),(zcyl-0.1)+0.0)

 axis equal
 axis([-3 3 -3 3 -3 3])
 view(162, 12)
 axis off

 %-------
 % plot the solution in parametric space
 %-------

 figure(loopb+30)
 colormap('bone')
 surf(eta(1:Net+1)/(2*pi),xi(1:Nxi+1)/xi0,x(1:Nxi+1,1:Net+1))
 xlabel('\eta/(2\pi)','fontsize',15);
 ylabel('\xi/\xi_0','fontsize',15);
 zlabel('\zeta/c','fontsize',15);
 set(gca,'fontsize',15)
 axis([0 1 1 ximax/xi0 0 2])
 view(150, 24)
 box on

end

%===========
end % of loopb
%===========
