function obj = men_3d_2p_fun1(xcontact)

%===================
% solve the Laplace-Young equation
%
% NOTATION:
%
% xcontact: elevation around the contact angle
% obj:      contact angle objective function
%===================

%=====
 global ab
 global Nxi Neta
 global Dxi Deta
 global xi eta
 global x y z J
 global dydxi dydeta dzdxi dzdeta
 global a delta

 global Niter tol
 global capls
 global forcex forcey forcez torqux torquy torquz
 global cang
 global cangle
 global elv
 global cang
 global drhosign
%=====

%---
% prepare
%---

 Dxi2 = 2.0*Dxi;
 Deta2 = 2.0*Deta;

%---
% assign boundary values for i=1
%---

 for j=1:Neta+1
  x(1,j) = xcontact(j);
 end

%----
% prepare for iterations
%----

 nonlinear = 0;
 omega = 1.0;
 Iflag = 0;

%======
for iterations=1:Niter
%======

 %~~~~~
 % wrap
 %~~~~~

 for j=1:Neta+1
  x(Nxi+2,j) = x(Nxi,j);   % right
 end

 for i=1:Nxi+1
  x(i,Neta+2) = x(i,Neta);    % top
  x(i,Neta+3) = x(i,Neta-1);  % top
 end

 xold = x;  % to compute the error
 xneo = x;  % to update by Jacobi

 %---
 % compute the first derivatives
 % dxdy and dxdz
 %---

 %~~~
 for i=2:Nxi+1
 %~~~
 jmax = Neta+2;
 if(i==Nxi+1) jmax = Neta; end
 %~~~
 for j=1:jmax
 %~~~

  MAT(1,1) =  dydxi(i,j); MAT(1,2) =  dzdxi(i,j);
  MAT(2,1) = dydeta(i,j); MAT(2,2) = dzdeta(i,j);

  if(i==Nxi+1)
   RHS(1) = 0.0;
  else
   RHS(1) = (x(i+1,j)-x(i-1,j))/Dxi2;
  end

  if(j==1)
   RHS(2) = 0.0;
  else
   RHS(2) = (x(i,j+1)-x(i,j-1))/Deta2;
  end

  SOL = RHS/MAT';
  dxdy(i,j) = SOL(1);
  dxdz(i,j) = SOL(2);

 %~~~
 end
 %~~~
 end
 %~~~

 %---
 % boundary values at i=1
 % needed for the second derivatives
 %---

 for j=1:Neta+2
  dxdy(1,j)    = 2.0*dxdy(2,j) -dxdy(3,j);
  dxdz(1,j)    = 2.0*dxdz(2,j) -dxdz(3,j);

  dxdy(1,j) = 3.0*dxdy(2,j)-3.0*dxdy(3,j)+dxdy(4,j);
  dxdz(1,j) = 3.0*dxdz(2,j)-3.0*dxdz(3,j)+dxdz(4,j);
 end

 %---
 % wrap at i=Nxi+2
 %---

 for j=1:Neta+2
  dxdy(Nxi+2,j) = -dxdy(Nxi,j);
  dxdz(Nxi+2,j) =  dxdz(Nxi,j);
 end

 dxdz(Nxi+1,Neta+1) = (4.0*dxdz(Nxi+1,Neta)-dxdz(Nxi+1,Neta-1))/3.0;
 dxdy(Nxi+1,Neta+1) = dxdz(Nxi+1,Neta+1);

  Ishow = 1;
  Ishow = 0;

%---
  if(Ishow==1)
%---

  figure(33)
  surf(eta(1:Neta+2),xi(1:Nxi+2),dxdy(1:Nxi+2,1:Neta+2))
  ylabel('\xi')
  xlabel('\eta')

  figure(34)
  surf(eta(1:Neta+2),xi(1:Nxi+2),dxdz(1:Nxi+2,1:Neta+2))
  xlabel('\eta')
  ylabel('\xi')

  pause
% figure(33)
% clf
% figure(34)
% clf

  end

%---
% compute the second derivatives
%---

 for i=2:Nxi+1
 jmax = Neta+1;
 if(i==Nxi+1) jmax = Neta; end
 for j=1:jmax
  MAT(1,1) =  dydxi(i,j); MAT(1,2) =  dzdxi(i,j);
  MAT(2,1) = dydeta(i,j); MAT(2,2) = dzdeta(i,j);
  RHS(1) = (dxdy(i+1,j)-dxdy(i-1,j))/Dxi2;
  if(j==1)
    RHS(2) = (dxdy(i,j+1)-dxdy(i,2))/Deta2;
  else
    RHS(2) = (dxdy(i,j+1)-dxdy(i,j-1))/Deta2;
  end
  SOL = RHS/MAT';
  dxdyy(i,j) = SOL(1);
  dxdyz(i,j) = SOL(2);
  RHS(1) = (dxdz(i+1,j)-dxdz(i-1,j))/Dxi2;
  if(j==1)
    RHS(2) = (dxdz(i,j+1)+dxdz(i,2))/Deta2;
  else
    RHS(2) = (dxdz(i,j+1)-dxdz(i,j-1))/Deta2;
  end
  SOL = RHS/MAT';
  dxdzy(i,j) = SOL(1);
  dxdzz(i,j) = SOL(2);

% [dxdyy dxdyz dxdzy dxdzz]
% pause

 end

 dxdyy(i,Neta+1) = 2*dxdyy(i,Neta)-dxdyy(i,Neta-1);
 dxdyz(i,Neta+1) = 2*dxdyz(i,Neta)-dxdyz(i,Neta-1);
 dxdzy(i,Neta+1) = 2*dxdzy(i,Neta)-dxdzy(i,Neta-1);
 dxdzz(i,Neta+1) = 2*dxdzz(i,Neta)-dxdzz(i,Neta-1);

end

  Ishow = 1;
  Ishow = 0;

  if(Ishow==1)

  figure(35)
  surf(eta(1:Neta+1),xi(1:Nxi+1),dxdyy(1:Nxi+1,1:Neta+1))
  ylabel('\xi')
  xlabel('\eta')

  figure(36)
  surf(eta(1:Neta+1),xi(1:Nxi+1),dxdyz(1:Nxi+1,1:Neta+1))
  ylabel('\xi')
  xlabel('\eta')

  figure(37)
  surf(eta(1:Neta+1),xi(1:Nxi+1),dxdzy(1:Nxi+1,1:Neta+1))
  ylabel('\xi')
  xlabel('\eta')

  figure(38)
  surf(eta(1:Neta+1),xi(1:Nxi+1),dxdzz(1:Nxi+1,1:Neta+1))
  ylabel('\xi')
  xlabel('\eta')

  pause

  end

%---
% done computing derivatives
%---

 %~~~
 for i=2:Nxi
 %~~~
 jmax = Neta+1;
 %~~~
 for j=1:jmax
 %~~~

  JS = J(i,j)*J(i,j);

  if(j==1)
   xb = x(i,2);
  else
   xb = x(i,j-1);
  end

  xnew = (x(i+1,j)+x(i-1,j))/Dxi^2 + (x(i,j+1)+xb)/Deta^2;
  xnew = xnew/JS;

%~~~
  if(nonlinear==1)
   xnew = xnew +  dxdz(i,j)^2               *dxdyy(i,j);
%  xnew = xnew - 2.0*dxdy(i,j)*dxdz(i,j)    *dxdyz(i,j);
   xnew = xnew - dxdy(i,j)*dxdz(i,j)   * (dxdyz(i,j) + dxdzy(i,j));
   xnew = xnew +  dxdy(i,j)^2               *dxdzz(i,j);
  end
%~~~

  tmp = 1.0+dxdy(i,j)^2+dxdz(i,j)^2;
  tmp = tmp*tmp*tmp;
  tmp = sqrt(tmp);

  xnew = xnew + tmp*delta/a;

  G = 2.0*(1.0/Dxi^2+1.0/Deta^2)/JS + drhosign*tmp/capls;

  xnew = xnew/G;

  xneo(i,j) = x(i,j)+omega*(xnew-x(i,j));

 %~~~
 end
 %~~~
 end
 %~~~

 x = xneo;

 % upper right corner

%aa = x(Nxi,Neta+1);
%bb = x(Nxi+1,Neta);
%x(Nxi+1,Neta+1) = (aa/Dxi+bb/Deta)/(1/Dxi+1/Deta);

 % reset the origin:

%x = x- x(Nxi+1,Neta+1);

 % reset the contact line:

 for j=1:Neta+1
  x(1,j) = xcontact(j);
 end

%-----
% compute the error
%-----

 err = 0.0;

 for i=2:Nxi+1
  for j=1:Neta+1
   diff = abs(xold(i,j)-x(i,j));
   if(diff>err) err = diff; end
  end
 end

%[nonlinear err]

%---
% stopping check
%---

if(err<10.0*tol)
  nonlinear = 1;
  omega = 0.1;
end

if(err<tol)
  Iflag = 1;
  break;
end

%======
end % of iterations
%======

if(Iflag == 0)
 disp('iterations did not coverge')
end

%============
% Compute
% the surface normal and tension vector
% around the contact line
%============

x0 = 0.0;
y0 = 0.0;
z0 = 0.0;

for j=1:Neta+1

 dxdxi(1,j) = (-x(3,j)+4.0*x(2,j)-3.0*x(1,j))/Dxi2;

 if(j==1)
  dxdeta(1,j) = 0.0;
 else
  dxdeta(1,j) = (x(1,j+1)-x(1,j-1))/Deta2;
 end

 vnx(j) = dydxi(1,j)*dzdeta(1,j)-dzdxi(1,j)*dydeta(1,j);
 vny(j) = dzdxi(1,j)*dxdeta(1,j)-dxdxi(1,j)*dzdeta(1,j);
 vnz(j) = dxdxi(1,j)*dydeta(1,j)-dydxi(1,j)*dxdeta(1,j);
 vnm(j) = sqrt(vnx(j)^2+ vny(j)^2+vnz(j)^2);
 vnx(j) = vnx(j)/vnm(j);
 vny(j) = vny(j)/vnm(j);
 vnz(j) = vnz(j)/vnm(j);

 % tangential vector

 tngx(j) = dydeta(1,j)*vnz(j)-dzdeta(1,j)*vny(j);
 tngy(j) = dzdeta(1,j)*vnx(j)-dxdeta(1,j)*vnz(j);
 tngz(j) = dxdeta(1,j)*vny(j)-dydeta(1,j)*vnx(j);

 % cross of tangential vector with position vector

 crsx(j) = (y(1,j)-y0)*tngz(j)-(z(1,j)-z0)*tngy(j);
 crsy(j) = (z(1,j)-z0)*tngx(j)-(x(1,j)-x0)*tngz(j);
 crsz(j) = (x(1,j)-x0)*tngy(j)-(y(1,j)-y0)*tngx(j);

end

%---
% cylinder normal vector
%---

ycnt = 0;

for j=1:Neta+1
 vnpx(j) = 0.0;
 vnpy(j) = dydxi(1,j);
 vnpz(j) = dzdxi(1,j);
 vnpm(j) = sqrt(vnpx(j)^2+vnpy(j)^2+vnpz(j)^2);
 vnpx(j) = vnpx(j)/vnpm(j);
 vnpy(j) = vnpy(j)/vnpm(j);
 vnpz(j) = vnpz(j)/vnpm(j);
end

%---
% contact angle
%---

for j=1:Neta+1
  prj = vnx(j)*vnpx(j) ...
       +vny(j)*vnpy(j) ...
       +vnz(j)*vnpz(j);
  cangle(j) = acos(prj);
end

%---
% object function
%---

for j=1:Neta+1
  obj(j) = cangle(j)-cang;
end

%---
% force and torque
%---

forcex = 0.0;
forcey = 0.0;
forcez = 0.0;
torqux = 0.0;
torquy = 0.0;
torquz = 0.0;

for j=1:Neta
  forcex = forcex+tngx(j);
  forcey = forcey+tngy(j);
  forcez = forcez+tngz(j);
  torqux = torqux+crsx(j);
  torquy = torquy+crsy(j);
  torquz = torquz+crsz(j);
end
forcex = forcex*Deta;
forcey = forcey*Deta;
forcez = forcez*Deta;

torqux = torqux*Deta;
torquy = torquy*Deta;
torquz = torquz*Deta;

%[forcex, forcey, forcez]
%[torqux, torquy, torquz]

%---
% done
%---

return

