       program men_ax

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the lisensing agreement
c==========================================

c----------------------------------------------
c Axisymmetric meniscus
c inside a vertical circular tube
c computed by a shooting method
c
c The updates are done using the secant method.
c
c Legend:
c ------
c
c npts: number of points along the interface.
c
c x,s:  polar cylindrical coordinates of marker points
c       along the trace of the interface in an
c       azimuthal plane
c
c h:      meniscus height at the centerline
c error:  residual of object function
c
c tol:    accuracy tolerance
c Itmax:  maximum number of iterations
c----------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(800),s(800)
      Dimension h(30),error(300)

      parameter (tol=0.00001,Itmax = 1000)

c----------
c constants
c----------

      pi  = 3.14159 265358

      piq = 0.25*pi
      pih = 0.50*pi

      null = 0
      none = 1
      ntwo = 2

      zero = 0.0
      one  = 1.0
      onem =-1.0

      two  = 2.0
      twom =-2.0

c--------
c prepare
c--------

      open (1,file="men_ax.out",status="unknown")

c-----------
c Preferences
c------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to read data from file: men_2d.dat"
      write (6,*) " 2 to type in data"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Ienrd

      If(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.2) then
c------------------------

      write (6,*)
      write (6,*) " Enter the the acceleration of gravity"
      write (6,*) " 99 to quit"
      write (6,*) " ----------"
      read  (5,*) gac

      if(gac.eq.99) Go to 99

      write (6,*) 
      write (6,*) " Enter the tube radius"
      write (6,*) " ---------------------"
      read  (5,*) a

      write (6,*) 
      write (6,*) " Enter the density of the pool"
      write (6,*) " -----------------------------"
      read  (5,*) rhop

      write (6,*) 
      write (6,*) " Enter the density of the ambient fluid"
      write (6,*) " --------------------------------------"
      read  (5,*) rhoa

      write (6,*) 
      write (6,*) " Enter the interfacial tension"
      write (6,*) " -----------------------------"
      read  (5,*) gamma

      write (6,*) 
      write (6,*) " Enter contact angle in multiples of pi/2"
      write (6,*) " ----------------------------------------"
      read  (5,*) alpha

      write (6,*) 
      write (6,*) "Enter the number of integration intervals"
      write (6,*) "-----------------------------------------"
      read  (5,*) npts

c----------
       else
c----------

        open (2,file="men_ax.dat")

         read (2,*) gac
         read (2,*) a
         read (2,*) rhop
         read (2,*) rhoa
         read (2,*) gamma
         read (2,*) alpha
         read (2,*) npts

        close (2)

c----------
      end if
c----------

c-----------------------------
c check the density difference
c-----------------------------

      drho = rhop-rhoa

      if(drho.lt.0.0) then
       write (6,*)
       write (6,*) " Sorry: density of the pool must be higher "
       write (6,*) "        than that of the ambient fluid"
       write (6,*)
       Go to 99
      end if

c--------
c prepare
c--------

      npts1 = npts+1
      am = -a

      capls = gamma/(gac*abs(drho))    ! square of capillary length

      alpha = pih*alpha

c-------------------------------------------
c initial guess for h
c computed by assuming a spherical interface
c-------------------------------------------

      h(1) = 2.0*capls*cos(alpha)/a

      write (6,*)
      write (6,*) " Shooting parameter is the meniscus height"
      write (6,*) " initial value set to: ",h(1)
      write (6,*)
c     write (6,*) " Please 0 to proceed, 1 to change it"
c     write (6,*) " -----------------------------------"
c     read  (5,*) Ichange

      Ichange = 0

      if(Ichange.eq.1) then
       write (6,*)
       write (6,*) " Please enter the new value"
       read (5,*) h(1)
      end if

c-----------------
c return to repeat
c-----------------

  94  Continue

      dthet = (pih-alpha)/(npts1-1.0)

c-------------------
c prepare to display
c-------------------

      write (6,*)
      write (6,*) " Radius, shooting parameter, error" 
      write (6,*)

c-----------------------------------
c Compute first solution of the odes
c to start up the secant method
c-----------------------------------

      Ic = 1                 ! counter

      call men_ax_ode
     +
     +  (npts
     +  ,capls
     +  ,dthet
     +  ,h(Ic)
     +  ,x,s
     +  )

      error(Ic) = s(npts1)-a
      err       = abs(error(Ic))

      write (6,100) Ic,s(npts1),h(Ic),err

c-------------------------
c second start-up solution 
c-------------------------

      Ic = 2

      h(2) = h(1)+0.01

      call men_ax_ode
     +
     +  (npts
     +  ,capls
     +  ,dthet
     +  ,h(Ic)
     +  ,x,s
     +  )

      error(Ic) = s(npts1)-a
      err       = abs(error(Ic))

      write (6,100) Ic,s(npts1),h(Ic),err

c-------------------------------------------------------
c iterate on h using the secant method until convergence
c-------------------------------------------------------

      Iconverge = 0

      Do while(err.gt.tol)

       Ic = Ic+1

       If(Ic.gt.Itmax) then
        write (6,*) 
        write (6,*) 'Sorry: iterations did not converge'
        write (6,*) 
        Iconverge = 1
        Go to 97
       End If

c---
c Secant update
c---
    
      Icb = Ic-2
      Ica = Ic-1

      dedh = (error(Ica)-error(Icb))
     +      /(    h(Ica)-    h(Icb))

      h(Ic) = h(Ica)-error(Ica)/dedh

       call men_ax_ode
     +
     +   (npts
     +   ,capls
     +   ,dthet
     +   ,h(Ic)
     +   ,x,s
     +   )

      error(Ic) = s(npts1)-a
      err       = Dabs(error(Ic))

      write (6,100) Ic,s(npts1),h(Ic),err

      End Do

  97  Continue

      cr = h(Ic)     ! capillary rise

c-----------------
c printing session
c-----------------

c---
c draw tube walls
c---

       height  = abs(h(Ic))+abs(x(npts1))+0.1
       heightm = -height

       write (1,100) Ntwo
       write (1,100) none,am,heightm
       write (1,100) ntwo,am,height
       write (1,100) Ntwo
       write (1,100) none, a,heightm
       write (1,100) ntwo, a,height

c---
c draw the pool
c---

       apo  = a+1.0
       apom = - apo

       write (1,100) Ntwo
       write (1,100) none,apom,zero
       write (1,100) ntwo,  am,zero

       write (1,100) Ntwo
       write (1,100) none,  a,zero
       write (1,100) ntwo,apo,zero

c---
c printing session
c draw meniscus and its reflection
c---

       shift = -cr    ! shift to reset the origin at the pool

       write (1,100) npts1

       Do i=1,Npts1
        write (1,100) i,s(i),x(i)-shift
       End Do

       write (1,100) npts1

       Do i=1,Npts1
        write (1,100) i,-s(i),x(i)-shift
       End Do

c-------
c repeat
c-------

      write (6,*) "a=",a,"  alpha/pi=",alpha/pi

      write (6,*)
      write (6,*) " Enter new contact angle in multiples of pi/2"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) alpha

      If(alpha.eq.0) Go to 99

       h(1) = 2.0*capls*cos(alpha)/a

      Go to 94

c-----
c Done
c-----

  99  Continue

      write (1,100) null
      close (1)

  100 Format (1x,i3,5(2x,f15.10))

      stop
      end
