function [Iflag,x,s,hout] = men_ax ...
 ...
  (a,gac,gamma,rhop,rhoa ...
  ,alpha,npts ...
  ,epsilon,maxiter ...
  ,tol ...
  ,hin ...
  )

%=================================
% FDLIB
%
% shape of an axisymmetric meniscus 
% inside a tube of radius computed
% by a shooting method for h
%=================================

%-----------------------------------
% shape of an axisymmetric meniscus
% inside a tube of radius computed
% by a shooting method for h
%
% hin: initial guess for h
%-----------------------------------


 Iflag = 0;  % flag for success

%----
% prepare
%----

 drho  = rhop-rhoa ;             % density difference
 capls = gamma/(gac*abs(drho));  % square of the capillary length

 npts1 = npts+1;

 dthet = (0.5*pi-alpha)/npts;

%-----------------------------------------
% initial guess for h
% computed by assuming a circular interface
%-----------------------------------------

h(1) = hin;

%---------
% compute the first solution of the odes
% to start-up the secant method
%---------

  Ic = 1;   % counter
  [x,s] = men_ax_ode(npts,capls,a,dthet,h(Ic));

  error(Ic) = s(npts1)-a;

%-------------------------
% second start-up solution 
%-------------------------

  Ic = 2;
  h(2) = h(1)+epsilon;
  [x,s] = men_ax_ode(npts,capls,a,dthet,h(Ic));

  error(Ic) = s(npts1)-a;

%---------------------------------------
% iterate hshp using the secant method
% until convergence
%---------------------------------------

 for iter=1:maxiter

  Ic = Ic+1;

%---
% secant updating
%---
     
  Icb = Ic-2;
  Ica = Ic-1;

  dedh = (error(Ica)-error(Icb))/(h(Ica)-h(Icb));
  h(Ic) = h(Ica) - error(Ica)/dedh;

  [x,s] = men_ax_ode(npts,capls,a,dthet,h(Ic));

  error(Ic) = s(npts1)-a;

  if(abs(error(Ic))<tol)
    break
  end

%---
   end
%---

   if(iter==maxiter)
    disp('men_ax: ODE solver failed')
    Iflag=1;
    return
   end

   hout = h(Ic);

%---
% done
%---
 
 return
