%-------
clear all
clear global
close all
%------

%==============================
% FDLIB
%
% meniscus in the exterior
% of two circular contact lines
% attached to two circular cylinders
%==============================

global d a1 a2 A
global Nxi Net
global xi1 xi2
global xi cshxi snhxi
global eta cseta sneta
global Niter tol
global capls
global x y z J Ikeep
global ycnt1 ycnt2
global forcex1 forcey1 forcez1 torqux1 torquy1 torquz1
global forcex2 forcey2 forcez2 torqux2 torquy2 torquz2
global cang1 cang2
global cangle1 cangle2

%---
% plotting flags
%---

Iplot_grid = 1;
Iplot_grid = 0;

Iplot_metric = 1;
Iplot_metric = 0;

Iplot_shape = 0;
Iplot_shape = 1;

%---
% parameters
%---

%---
problem = 1;
%---

%---
problem = 2;
cang1 = 0.25*pi;
cang2 = 0.25*pi;
%---

tol = 0.000001;
Niter = 200000;

a1 = 1.0;  % first radius
a2 = 1.0;  % second radius

Jmax = 200.0;  % maximum metric
Jmax = 20.0;

elv1 = 0.25*a1; % contact line elevation
elv2 = 0.25*a2;

elv1 = 1.0*a1;
elv2 = 1.0*a2;

%---
% capillary length
%---

capl = 2.0*a1;

%---
% case A: alpha = 0.25pi
%
capl = 4.5036*a1;
elv1 = 1.3210*a1;
elv2 = 1.3210*a2;
%---

%---
% case B: alpha = 0.125pi
%
capl = 4.5036*a1;
elv1 = 1.8845*a1;
elv2 = 1.8845*a2;
%---

%---
% prepare
%---

im = sqrt(-1);

capls = capl*capl;

%---
% prepare for graphics
%---

[xsph,ysph,zsph] = sphere(16);
[xcyl,ycyl,zcyl] = cylinder(1.0,16);

%~~~~
%for loopd=1:5
 for loopd=1:1
%for loopd=3:3
%for loopd=2:2
%for loopd=4:4
%for loopd=6:6
%for loopd=7:7
%for loopd=8:8
%for loopd=9:9
%~~~~

if(loopd==1) d=2.0; end
if(loopd==2) d=2.5; end
if(loopd==3) d=3.0; end
if(loopd==4) d=3.5; end
if(loopd==5) d=4.0; end
if(loopd==6) d=4.5; end
if(loopd==7) d=5.0; end
if(loopd==8) d=5.5; end
if(loopd==9) d=6.0; end

%=====
% compute xi1 xi2 A
%=====

xi1 = -1.0;
xi2 =  1.0;
vitamin(1) = xi1;
vitamin(2) = xi2;
vitamin = fsolve(@myfun,vitamin);
xi1 = vitamin(1);
xi2 = vitamin(2);

%---
% circle centers along the y axis
%---

ycnt1 = A*coth(xi1);
ycnt2 = A*coth(xi2);

%=============
%for ngrid=1:3
%for ngrid=1:1
 for ngrid=2:2
%=============

if(ngrid==1) Np=1; end
if(ngrid==2) Np=2; end
if(ngrid==3) Np=4; end

%---
Nxi = Np*8+1;
Net = Np*16;
%---

%---
% grid
%---

 Dxi = (xi2-xi1)/Nxi;

 for i=1:Nxi+1
     xi(i) = xi1+(i-1.0)*Dxi;
  cshxi(i) = cosh(xi(i));
  snhxi(i) = sinh(xi(i));
 end

 Deta = 2.0*pi/Net;

 for j=1:Net+1
     eta(j) = (j-0.5)*Deta;
   cseta(j) = cos(eta(j));
   sneta(j) = sin(eta(j));
 end

%---
% mapping 
%---

 for i=1:Nxi+1
  for j=1:Net+1

   dn = cshxi(i)+cseta(j);
   yy = A*snhxi(i)/dn;
   zz = A*sneta(j)/dn;
   y(i,j) = yy;
   z(i,j) = zz;
   J(i,j) = A/dn;

   Ikeep(i,j) = 1;
   Ikeep((Nxi+1)/2,Net/2 ) = 0;
   Ikeep((Nxi+1)/2,Net/2+1) = 0;
   Ikeep((Nxi+1)/2+1,Net/2 ) = 0;
   Ikeep((Nxi+1)/2+1,Net/2+1) = 0;

%  if(J(i,j)>Jmax)
%    Ikeep(i,j) = 0;
%  end;

% xe = (pi-eta(j)+im*xi(i))/2.0;
% [A*im*cot(xe), yy+im*zz]
% [0.5*abs(1+cot(0.5*xe)^2), 1/den]
% pause

  end
 end

%---
% angle around the first cylinder
%---

 for j=1:Net
  rr = sqrt((y(1,j)-ycnt1)^2+z(1,j)^2);
  theta1(j) = acos((y(1,j)-ycnt1)/rr);
  if(z(1,j)<0.0) 
  theta1(j) = 2*pi-theta1(j);
  end
 end

 theta1(Net+1) = theta1(1)+2*pi;

%---
% wrap
%---

 for i=1:Nxi+1
   Ikeep(i,Net+2) = Ikeep(i,2);
 end

%---
% initialize
%---

 for i=1:Nxi+1
  for j=1:Net+1
   x(i,j) = 0.0;
  end
 end

%=========
% plot the grid
%=========

if(Iplot_grid==1)

 figure(100)
 colormap('bone')

 hold on;

 for i=1:Nxi
  plot(y(i,:),z(i,:),'k-')
 end

 for j=1:Net+1
  plot(y(:,j),z(:,j),'k-')
 end

 for i=1:Nxi
  for j=1:Net+1
   if(Ikeep(i,j)==1)
     plot(y(i,j),z(i,j),'k.')
   end
  end
 end

 plot([ycnt1, ycnt2],[0,0],'k','LineWidth',1)
 plot(y(1,:),z(1,:),'k','LineWidth',2)
 plot(y(Nxi+1,:),z(Nxi+1,:),'k','LineWidth',2)

 xlabel('y/a_1','fontsize',15);
 ylabel('z/a_1','fontsize',15);
 set(gca,'fontsize',15)

 axis equal
% axis([2*ycnt1 2*ycnt2 2*ycnt1 2*ycnt2])
 box on
%view(140,30)

 pause

end
%=============

%=============
% plot the metric
%=============

if(Iplot_metric==1)

 figure(101)
 colormap('gray')
 hold on;

% wire frame

 Ido = 0;
 Ido = 1;

 if(Ido==1)
  for i=1:Nxi
%  plot3(y(i,:),z(i,:),J(i,:),'k-')
   plot(y(i,:),z(i,:),'k-')
  end
  for j=1:Net
%  plot3(y(:,j),z(:,j),J(:,j),'k-')
   plot(y(:,j),z(:,j),'k-')
  end
 end

% patches

 for i=1:Nxi
  for j=1:Net
    if(Ikeep(i,j)==1|Ikeep(i+1,j)==1 ...
      |Ikeep(i+1,j+1)==1|Ikeep(i,j+1)==1 )
%   patch([y(i,j),y(i+1,j),y(i+1,j+1),y(i,j+1),y(i,j)] ...
%        ,[z(i,j),z(i+1,j),z(i+1,j+1),z(i,j+1),z(i,j)] ...
%        ,[J(i,j),J(i+1,j),J(i+1,j+1),J(i,j+1),J(i,j)] ...
%        ,'0.3' ...
%       );
%   plot3([y(i,j),y(i+1,j),y(i+1,j+1),y(i,j+1),y(i,j)] ...
%        ,[z(i,j),z(i+1,j),z(i+1,j+1),z(i,j+1),z(i,j)] ...
%        ,[J(i,j),J(i+1,j),J(i+1,j+1),J(i,j+1),J(i,j)],'k');
    plot([y(i,j),y(i+1,j),y(i+1,j+1),y(i,j+1),y(i,j)] ...
        ,[z(i,j),z(i+1,j),z(i+1,j+1),z(i,j+1),z(i,j)],'k');
   end
  end
 end

 xlabel('y/a_1','fontsize',15);
 ylabel('z/a_1','fontsize',15);
 zlabel('J/a_1','fontsize',15);
 set(gca,'fontsize',15)
 axis equal
 axis([2*ycnt1 2*ycnt2 2*ycnt1 2*ycnt2])
%view(140,30)

 end
%=============

%=============
% plot a solution
%=============

Ido=1;
Ido=0;

if(Ido==1)

 figure(102)
 hold on;

 for i=1:Nxi+1
  for j=1:Net+1
   x(i,j) = 1.0 -(xi(i)-xi1)/(xi2-xi1);
  end
 end

 for i=1:Nxi
  plot3(y(i,:),z(i,:),x(i,:),'k-')
 end
 for j=1:Net+1
  plot3(y(:,j),z(:,j),x(:,j),'k-')
 end

 xlabel('y/a_1','fontsize',15);
 ylabel('z/a_1','fontsize',15);
 set(gca,'fontsize',15)

 axis([-6 6 -6 6 0 1 ])
 box on
%view(140,30)

end
%=============

%=========
% contact lines
%=========

for i=1:Net
 xcontact(i)     = elv1; %*(1+0.1*rand);
 xcontact(i+Net) = elv2; %*(1+0.2*rand);
end

%==============
 if(problem==1)
%==============

  obj = fcircles(xcontact);

% [forcex1, forcey1, forcez1, torqux1, torquy1, torquz1]
% [forcex2, forcey2, forcez2, torqux2, torquy2, torquz2]

  figure(loopd)
  hold on
  plot(theta1(1:Net)/(2*pi),cangle1(1:Net)/pi,'k')
  plot(theta1(1:Net)/(2*pi),cangle2(1:Net)/pi,'k--')
  axis([0 1 0.25 0.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\alpha/\pi','fontsize',15);
  set(gca,'fontsize',15)
  box on

%===========================
 elseif(problem==2)
%===========================

 Jeps = 0.001;
 obj = fcircles(xcontact);

 %~~~
 for Iterate = 1:3
 %~~~

 for j=1:2*Net

  xcontact(j) = xcontact(j) + Jeps;

  obj1 = fcircles(xcontact);

  for i=1:2*Net
   Jac(i,j) = (obj1(i)-obj(i))/Jeps;
  end

  xcontact(j) = xcontact(j) - Jeps;

 end

 xcontact = xcontact - obj/Jac';

 obj = fcircles(xcontact);

%-----
% plot the contact angle distribution
%-----

  figure(loopd)
  hold on
  plot(theta1(1:Net)/(2*pi),cangle1(1:Net)/pi,'k')
  plot(theta1(1:Net)/(2*pi),cangle2(1:Net)/pi,'k--')
  axis([0 1 0.00 0.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\alpha/\pi','fontsize',15);
  set(gca,'fontsize',15)
  box on

 %~~~
 end % of iterate
 %~~~

 [forcex1, forcey1, forcez1, torqux1, torquy1, torquz1]
 [forcex2, forcey2, forcez2, torqux2, torquy2, torquz2]

%==========================
 end % of problem selection
%==========================

%---
% hold
%---

 for j=1:Net+1
  hold_Net(ngrid) = Net;
  hold_theta1(ngrid,j) = theta1(j);
  hold_cangle1(ngrid,j) = cangle1(j);
  hold_cangle2(ngrid,j) = cangle2(j);
 end

 hold_ft1(ngrid,:) = [forcex1 forcey1 forcez1 ...
                      torqux1 torquy1 torquz1];

 hold_ft2(ngrid,:) = [forcex2 forcey2 forcez2 ...
                      torqux2 torquy2 torquz2];

%==========
end  % of ngrid resolution 
%==========

%---
% extrapolate the force and torque
%---

 if(ngrid==3)

 fx1_extra = rich(hold_ft1(1,1),hold_ft1(2,1),hold_ft1(3,1));
 fy1_extra = rich(hold_ft1(1,2),hold_ft1(2,2),hold_ft1(3,2));
 tz1_extra = rich(hold_ft1(1,6),hold_ft1(2,6),hold_ft1(3,6));

 ftext = [fx1_extra, fy1_extra, tz1_extra]

 end

%---
% plot the contact angle or contact line
% distribution of the finest solution
%---

if(problem ==1)

  figure(110)
  hold on
  plot(theta1(1:Net+1)/(2*pi),cangle1(1:Net+1)/pi,'k')
  plot(theta1(1:Net+1)/(2*pi),cangle2(1:Net+1)/pi,'k--')
  axis([0 1 0.25 0.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\alpha/\pi','fontsize',15);
  set(gca,'fontsize',15)
  box on

elseif(problem ==2)

  figure(110)
  hold on
  plot(theta1(1:Net+1)/(2*pi),x(1,1:Net+1)/a1,'k')
  axis([0 1 0.0 2.50])
  xlabel('\theta/2\pi','fontsize',15);
  ylabel('\zeta/a_1','fontsize',15);
  set(gca,'fontsize',15)
  box on

end

%=================
% plot the finest solution
% in physical space
%=================

%---
if(Iplot_shape==1)
%---

 figure(loopd+20)

 colormap('gray')
 colormap('bone')
 hold on;

% for i=1:Nxi
%  plot3(y(i,1:Net+1),z(i,1:Net+1),x(i,1:Net+1),'k-')
% end
% for j=1:Net+1
%  plot3(y(1:Nxi+1,j),z(1:Nxi+1,j),x(1:Nxi+1,j),'k-')
% end

 fc = 1.0;

 Rmax = 1.5*d;

 for i=1:Nxi+1
  for j=1:Net+1
   r = sqrt(yy*yy+zz*zz);
   Jkeep(i,j) = 1;
   if(r>Rmax)
     Jkeep(i,j) = 0;
   end;
  end
 end

 for i=1:Nxi+1
   Jkeep(i,Net+2) = Jkeep(i,2);
 end

 for i=1:Nxi
  for j=1:Net
    if( Jkeep(i,j)==1 & Jkeep(i+1,j)==1 ...
      & Jkeep(i+1,j+1)==1 & Jkeep(i,j+1)==1 )
    patch([y(i,j),y(i+1,j),y(i+1,j+1),y(i,j+1),y(i,j)] ...
        ,[z(i,j),z(i+1,j),z(i+1,j+1),z(i,j+1),z(i,j)] ...
        ,fc*[x(i,j),x(i+1,j),x(i+1,j+1),x(i,j+1),x(i,j)] ...
        ,'y' ...
        )
%   plot3([y(i,j),y(i+1,j),y(i+1,j+1),y(i,j+1),y(i,j)] ...
%        ,[z(i,j),z(i+1,j),z(i+1,j+1),z(i,j+1),z(i,j)] ...
%        ,fc*[x(i,j),x(i+1,j),x(i+1,j+1),x(i,j+1),x(i,j)] ...
%       ,'k' ...
%       )
   end
   end
  end


 colormap('copper')

 for j=1:Net
   ptc = patch([y(1,j), y(1,j+1), y(1,j+1),  y(1,j)] ...
              ,[z(1,j), z(1,j+1), z(1,j+1),  z(1,j)] ...
              ,[0, 0, 0.5+x(1,j+1),  0.5+x(1,j)] ...
              ,100*[0, 0, 0.5+x(1,j+1),  0.5+x(1,j)] ...
              );
 %set(ptc,'facecolor','red','edgecolor','none');
% set(ptc,'edgecolor','none');
  set(ptc,'edgecolor','k');
  %camlight; lighting gouraud;
   ptc = patch([y(Nxi+1,j), y(Nxi+1,j+1), y(Nxi+1,j+1), y(Nxi+1,j)] ...
              ,[z(1,j), z(Nxi+1,j+1), z(Nxi+1,j+1),  z(Nxi+1,j)] ...
              ,[0, 0, 0.5+x(Nxi+1,j+1),  0.5+x(Nxi+1,j)] ...
              ,100*[0, 0, 0.5+x(Nxi+1,j+1),  0.5+x(Nxi+1,j)] ...
              );
 end


% -------
% spheres
% -------
% surf(ycnt1+a1*ysph, a1*xsph, elv1+a1*zsph)
% surf(ycnt2+a2*ysph, a2*xsph, elv2+a2*zsph)

% -------
% cylinders
% -------

 Ido = 0;

 %---
 if(Ido==1)
 %---
 surf(ycnt1+a1*ycyl, a1*xcyl, 0.5*elv1+2.0*(zcyl-0.0) ...
                            ,  elv1+0.0*(zcyl-0.1)+1.0)
 surf(ycnt2+a2*ycyl, a2*xcyl, 0.5*elv2+2.0*(zcyl-0.0) ...
                            , elv2+0.0*(zcyl-0.1)+1.0)
 %---
 end
 %---

 for i=1:Nxi
  for j=1:Net+1
%  plot3(y(i,j),z(i,j),x(i,j),'b+')
   if(Ikeep(i,j)==1)
%    plot3(y(i,j),z(i,j),x(i,j),'ro')
   end
  end
 end

 xlabel('y/a_1','fontsize',15);
 ylabel('z/a_1','fontsize',15);
 zlabel('x/a_1','fontsize',15);
 set(gca,'fontsize',15)
 axis equal
 axis([-Rmax Rmax -Rmax Rmax -Rmax Rmax])
 view(140,30)
 axis off
 box off

%----
% plot the solution in parametric space
%----

 figure(loopd+30)
 colormap('bone')
 surf(eta/(2*pi),xi/xi2,x(1:Nxi+1,1:Net+1))
 xlabel('\eta/(2\pi)','fontsize',15);
 ylabel('\xi/\xi_2','fontsize',15);
 zlabel('\zeta/a_1','fontsize',15);
 set(gca,'fontsize',15)
 axis([0 1 -1 1 0 2*elv1])
 box on

end
%---

%=============
end % of loopd (cylinder separation)
%=============

