   function u = chan_2d_osc ...
 ...
      (h ...
      ,visc ...
      ,den ...
      ,Vamp ...
      ,omega ...
      ,ot ...
      ,y ...
      )

%=========================================
% FDLIB
%
% Copyright by C. Pozrikidis, 1999
% All rights reserved
%
% This program is to be used only under the
% stipulations of the licensing agreement
%=========================================

%=========================================
% Flow in a channel due to an oscillating wall
% The walls are located at y = 0, h
% The lower wall is oscillating with amplitude: Vamp
% so that the wall velocity is:
%
% V_wall = Vamp * cos(omega*time)
%
% ot: Omega*t
%=========================================

%--------
% prepare
%--------

 rnu   = visc/den;               % kinematic viscosity
 delta = sqrt(2.0D0*rnu/omega);  % Stokes layer thickness
 cs = cos(ot);
 sn = sin(ot);
 hr   = h/delta;               % reduced height
 hr2  = 2.0*hr;
 tmp1 = exp(-hr2);
 denr = 1.0-tmp1*cos(hr2);    % real part of the denominator
 deni =    -tmp1*sin(hr2);    % imag part of the denominator
 dens = denr*denr+deni*deni;
 aar  =  denr/dens;            % real part of 1/denominator
 aai  = -deni/dens;            % imag part of 1/denominator

%---
% compute the velocity
%---

  yr   = y/delta;
  tmp1 = exp(-yr);
  tmp2 = 2.0*hr-yr;
  tmp3 = exp(-tmp2);
  bbr  = tmp1*cos(yr)-tmp3*cos(tmp2);
  bbi  = tmp1*sin(yr)-tmp3*sin(tmp2);

  fr  = aar*bbr-aai*bbi;
  fi  = aar*bbi+aai*bbr;

  u = Vamp*( cs*fr + sn*fi );

%-----
% done
%-----

return
