      program chan_2d_trans_dr

c=========================================
c FDLIB, FDLIBM
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement
c=========================================

c=========================================
c Flow in a channel due to the sudden 
c sudden application of a constant pressure gradient
c or body force.
c
c The channel is confined between two parallel walls
c located at y = 0 and h
c=========================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      Null = 0
      None = 1
      Ntwo = 2
      zero = 0.0

c---
c input
c---

      open (1,file="chan_2d_trans.dat",status="unknown")

       read (1,*) nprof    ! number of points for a profile
       read (1,*) h        ! channel width 
       read (1,*) visc     ! viscosity
       read (1,*) den      ! density
       read (1,*) pg       ! pressure gradient
       read (1,*) maxsm    ! truncation limit for summation

      close (1)

c---------------
c prepare to run
c---------------

      open(2,file='chan_2d_trans.out')

      nprof1 = nprof+1
      step   = h/(nprof1-1.0)

c-----------------------------------------
c generate data for plotting channel walls
c-----------------------------------------

      write (2,900) Ntwo
      write (2,900) None,-h,zero
      write (2,900) Ntwo, h,zero
      write (2,900) Ntwo
      write (2,900) None,-h,h
      write (2,900) Ntwo, h,h

c-----
c menu
c-----

      write (6,*)
      write (6,*)  " Enter 1 for one profile"
      write (6,*)  " 0 to quit"
      write (6,*)  "----------"
      read  (5,*) menu

      if(menu.eq.0) Go to 99

c------------
c One profile
c------------

      If(menu.eq.1) then

 94   Continue

      write (6,*)
      write (6,*) " Enter the time"
      write (6,*) " 99 to quit"
      write (6,*) " ----------"
      read (5,*) time

      if(time.eq.99) Go to 99

c---
c generate a profile
c---

      write (6,*)
      write (6,*) "u, y"
      write (6,*)

      write (2,900) nprof1

      y = 0.0D0

      Do i=1,nprof1

       call chan_2d_trans
     +
     +     (h
     +     ,visc
     +     ,den
     +     ,pg
     +     ,maxsm
     +     ,time
     +     ,y
     +     ,u
     +     )

       write (2,900) i,u,y
       write (6,900) i,u,y

       y = y+step

      End Do

c---
c done with the profile
c---

       Go to 94

      End If

c-----
c Done
c-----

  99  Continue

      write (2,900) Null
      close (2)

 900  Format (1x,i3,4(1x,f10.5))

      Stop
      End
