%---
close all
hold on
%---

%==================================================
% Flow in a channel due to the sudden 
% sudden application of a constant pressure gradient
% or body force.
%
% The channel is confined between two parallel walls
% located at y = 0 and h
%==================================================

nprof = 32;    % number of points for a profile
h = 1;        % channel width 
visc = 1.0;    % viscosity
den = 1.0;     % density
pg = 1.0;      % pressure gradient
maxsm = 32;      % truncation limit for summation

%---
% maximum velocity at steady state
%---

U = pg*h/(8*visc);

step=h/nprof;

%---
 for iloop=1:32
%---

     if(iloop==1) time=0.001;
 elseif(iloop==2) time=0.005;
 elseif(iloop==3) time=0.010;
 else
   time=time+0.01;
 end

 y=0;

 for i=1:nprof+1

  u = chan_2d_trans  ...
 ...
     (h ...
     ,visc ...
     ,den ...
     ,pg ...
     ,maxsm ...
     ,time ...
     ,y ...
     );

  xplot(i)=u/U;
  yplot(i)=y/h;

  y=y+step;


 end

 y=0;
 for i=1:nprof+1
  parabolic(i)=0.5*pg*y*(h-y)/visc/U;
  y=y+step;
 end

 plot(xplot,yplot,'k')

%---
 end
%---

 plot(parabolic,yplot,'k--')

 xmin=-0.05; xmax=1.1;
 ymin= -0.2; ymax=1.1;
 axis([xmin xmax ymin ymax])
 plot([0 0],[0 h],'k');
 plot([xmin xmax],[0 0],'r','linewidth',4);
 plot([xmin xmax],[h h],'r','linewidth',4);
 xlabel('u_x/U','fontsize',15)
 ylabel('y/h','fontsize',15)
 set(gca,'fontsize',15)
 box

