function u = chan_2d_wom ...
       ...
  (h ...
  ,visc ...
  ,den ...
  ,pgamp ...
  ,omega ...
  ,ot ...
  ,y ...
  )

%==============================================
% Flow in a  channel due to an oscillatory
% pressure gradient:
%
% dp/dx = -pgamp * sin(omega*time)
%
% The channel walls are located at y = 0, h
%==============================================

%--------
% prepare
%--------

  rnu  = visc/den;  %   kinematic viscosity

  delta = sqrt(2.0*rnu/omega);
  hr    = h/delta;
  hrh   = 0.50*hr;
  denr  = cosh(hrh)*cos(hrh);
  deni  =-sinh(hrh)*sin(hrh);
  dens  = denr*denr+deni*deni;
  aar   =  denr/dens;
  aai   = -deni/dens;

%---
% compute the velocity
%---

  yr  = y/delta;          % reduced position
  tmp = yr-0.5*hr;
  bbr = cosh(tmp)*cos(tmp);
  bbi =-sinh(tmp)*sin(tmp);
  ccr = aar*bbr-aai*bbi;
  cci = aar*bbi+aai*bbr;

  ccr = ccr-1.0;
  cci = cci;

  cs  = cos(ot);
  sn  = sin(ot);

  tmp = cs*ccr+sn*cci;
  u   = pgamp*tmp/(den*omega);

%-----
% done
%-----

return
