      program chan_2d_wom_dr

c========================================
c FDLIB, FDLIBM
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c========================================

c========================================
c Flow in a 2D channel due to an oscillatory
c pressure gradient:
c
c pg = pgamp * sin(Omega*time)
c
c The channel walls are located at y = 0, h
c========================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

      pi = 3.14159 265358 D0

c---
c Input
c---

      open (1,file="chan_2d_wom.dat",status="unknown")

        read (1,*) nprof     ! number of profile points
        read (1,*) h         ! channel width
        read (1,*) visc      ! viscosity
        read (1,*) den       ! density
        read (1,*) Omega     ! angular frequency
        read (1,*) pgamp     ! amplitude of pressure gradient

      close (1)

c---
c prepare 
c---

      open (2,file='chan_2d_wom.out')

c-----
c menu
c-----

 93   Continue

      write (6,*)
      write (6,*)  " Enter 0 to quit"
      write (6,*)  "       1 for one profile"
      write (6,*)  "------------------------"
      read  (5,*) menu

      if(menu.eq.0) Go to 99

c------------
c One profile
c------------

      if(menu.eq.1) then

c---
c print data for plotting channel walls
c---

      write (2,900) Ntwo
      write (2,900) None,-1.2*h,zero
      write (2,900) Ntwo, 1.2*h,zero
      write (2,900) Ntwo
      write (2,900) None,-1.2*h,h
      write (2,900) Ntwo, 1.2*h,h

 94   Continue

      write (6,*)
      write (6,*) " Please enter Omega*t in multiples of pi"
      write (6,*) " Omega is the angular frequency"
      write (6,*) " Enter 99 to quit"
      write (6,*) " ----------------"

      read (5,*) ot

      If(ot.eq.99) Go to 99

c---
c profiles
c---

      ot = ot*pi

      write (6,*)
      write (6,*) "y, u"
      write (6,*)

      nprof1 = nprof+1
      step   = h/(nprof1-1.0)
      y      = 0.0

      write (2,900) nprof1

      Do i=1,nprof1

        call chan_2d_wom
     +
     +      (h
     +      ,visc
     +      ,den
     +      ,pgamp
     +      ,Omega
     +      ,ot
     +      ,y
     +      ,u
     +      )

        write (2,900) i,u,y
        write (6,900) i,y,u

        y = y+step

      End Do

c---
c done with the profile
c---

      Go to 94
      End If

c---
c done
c---

  99  Continue

      write (2,900) null
      close (2)

 900  Format (1x,i3,4(1x,f10.5))

      Stop
      End
