      program film_dr

c======================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c======================================

c======================================
c Flow of a liquid film of thickness h
c down an inclined plane
c
c SYMBOLS:
c -------
c
c nprof: number of points for a velocity profile
c h:     film thickness
c visc:  fluid viscosity
c den:   fluid density
c gac:   acceleration of gravity
c beta: channel inclination angle
c======================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358D0

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

c------
c input
c------

      open (1,file="film.dat",status="unknown")
        read (1,*) nprof
        read (1,*) h
        read (1,*) visc
        read (1,*) den
        read (1,*) gac
        read (1,*) beta
      close (1)

c--------
c prepare
c---------

      open (2,file="film.out",status="unknown")

      nprof1 = nprof+1

      beta = pi*beta

      cth0  = cos(beta)
      sth0  = sin(beta)

c------------------------------------
c Draw the plane and the free surface
c so that they appear inclined
c-------------------------------------

      write (2,900) Ntwo
      xx = -h
      yy =  0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  h
      yy =  0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      write (2,900) Ntwo
      xx = -h
      yy =  h
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  h
      yy =  h
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---------------------
c Draw a vertical line
c through the film
c---------------------

      write (2,900) Ntwo
      xx = zero
      yy = 0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = zero
      yy = h
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c-----------------
c Draw the profile
c-----------------

      write (6,*) 
      write (6,*) "Velocity Profile"
      write (6,*) 

      write (2,900) nprof1

      step = h/(nprof1-1.0)

      y = 0.0;

      Do i=1,nprof1

       call film
     +
     +    (h
     +    ,visc
     +    ,den
     +    ,gac
     +    ,beta
     +    ,y
     +    ,flow_rate
     +    ,umean
     +    ,u
     +    )

        write (6,900) i,y,u

        xx = u
        yy = y

        xprint = xx*cth0+yy*sth0
        yprint =-xx*sth0+yy*cth0

        write (2,900) i,xprint,yprint
c       write (6,900) i,xprint,yprint

        y = y+step

      End Do

      write (2,900) null
      close (2)

c----------------------------
c flow rate and mean velocity
c----------------------------

      write (6,*) 
      write (6,901) flow_rate,umean
      write (6,*) 

c-----
c Done
c-----

 900  Format (1x,i3,4(1x,f10.5))
 901  Format ("Flow rate     = ",f15.10
     +      ,/,"Mean velocity = ",f15.10
     +       )

      Stop 
      End
