      program streamlines

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c--------------------------------------------
c Generation of streamline patterns for a
c variety of flows offered in a menu
c
c SYMBOLS:
c -------
c
c  nc: 	    number of divisions to draw a circle
c
c  rcross:  streamline will end if a point lies within
c           a disk of radius "rcross"
c           centered at the origin
c
c Nstr:     number of streamlines to be drawn
c
c (x0,y0):  coordinates of starting points to draw a 
c           streamline
c
c (xstr,ystr): coordinates of points along a streamline
c
c
c CAPACITY:
c --------
c
c 300 streamlines
c 900 points on each streamline
c
c---------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x0(300),y0(300),xstr(900),ystr(900)

      Dimension xblas(800),fblas(800),fpblas(800)
      Dimension xspf(800),fspf(800),gspf(800)
      Dimension xhom(800),fhom(800)

      Parameter (nc=86)
      
c--------------
c common blocks
c--------------

      common/atlantic1/strength,RL,sig0,rlam,Vx,Vy,circ,Re,chi,cc
      common/atlantic2/wall,wall1,wall2,wall3,width
      common/atlantic3/xc,yc,zc,a,b,aob,boa,xs,ys,zs,R,xsi,ysi,zsi
      common/atlantic4/alpha,beta,gama,rm,g0,g1,blob_rad,Dcn
      common/atlantic5/xpv,ypv,xr,sr,ypv1,ypv2
      common/atlantic6/xblas,fblas,fpblas,xspf,fspf,gspf,xhom,fhom
      common/atlantic7/Nblas,Nspf,Nhom
      common/atlantic8/vsr
      common/atlantic9/rlr,rli,sc
      common/pacific1/Max1,Max2,Max3
      common/pacific2/a11,a12,a13,a21,a22,a23,a31,a32,a33
      common/velvet/Iframe,Iflow,Isym,Ieddies
      common/numdiff/eps,eps2

c----------
c constants
c----------

      pi   = 3.14159 265358
      piq  = 0.25*pi
      pih  = 0.50*pi
      pi2  = 2.00*pi
      pi4  = 4.00*pi
      pi6  = 6.00*pi
      pi8  = 8.00*pi

      eps  = 0.0001     ! for numerical differentiation

      zero =  0.0
      one  =  1.0
      onem = -1.0
      five =  5.0

      null = 0
      none = 1
      ntwo = 2
      nthr = 3
      nfou = 4

c-----------------------
c prepare and initialize
c-----------------------

      eps2 = 2.0*eps

      nc1 = nc+1     ! number of points to draw circles

      xc = 0.0      ! circle center coordinates
      yc = 0.0

      xcross = 0.0
      ycross = 0.0

      Iflow = 0

      Icount = 1     ! graphics

c---
 98   Continue
c---

      write (6,*)
      write (6,*) " Enter: "
      write (6,*)
      write (6,*) " 1  for potential flow I "
      write (6,*) " 10 for potential flow II"
      write (6,*) " 11 for potential flow III"
      write (6,*) " 2  for vortex flow "
      write (6,*) " 3  for Stokes flow I"
      write (6,*) " 31 for Stokes flow II"
      write (6,*) " 32 for Stokes flow III"
      write (6,*) " 33 for Stokes flow IV"
      write (6,*) " 4  for miscellaneous "
      write (6,*)
      write (6,*) " 0  to quit "
      write (6,*) " ----------"
      read  (5,*)  Ichoose

      if(Ichoose.eq.0) Go to 99

      if(     Ichoose.ne.1
     +   .and.Ichoose.ne.10
     +   .and.Ichoose.ne.11
     +   .and.Ichoose.ne.2
     +   .and.Ichoose.ne.3
     +   .and.Ichoose.ne.31
     +   .and.Ichoose.ne.32
     +   .and.Ichoose.ne.33
     +   .and.Ichoose.ne.4
     +  ) then

       write (6,*) " unwise choice; Please try again"
       Go to 98

      end if

c---
 97   Continue
c---

c=======================
      if(Ichoose.eq.1) then
c=======================

 399  Continue

      write (6,*) 
      write (6,*) "  POTENTIAL FLOW I"
      write (6,*) "  ----------------"
      write (6,*) 
      write (6,*) " Enter: "
      write (6,*) 
      write (6,*) " 1 for a 2D point source"
      write (6,*) " 2 for a periodic array of 2D point sources"
      write (6,*) " 3 for a 2D point source above a plane wall"
      write (6,*) " 4 for a 2D point source in a channel"
      write (6,*) " 5 for a 2D point source in a semi-infinite strip"
      write (6,*) " 6 for a 2D point source near a circular cylinder"
      write (6,*) " 7 for an array of 2D point sources above a wall"
      write (6,*) " 8 for a 2D point-source dipole"
      write (6,*) " 9 for a 2D point source in uniform flow"
      write (6,*) "10 for a 2D point source panel"
      write (6,*)
      write (6,*) "99 to return to the main menu"
      write (6,*) " 0 to quit"
      write (6,*) "----------"

      read (5,*) Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c------------------
c a 2D point source
c------------------

      if(Iselect.eq.1) then

       menu = 101

       open (2,file="DATA_IF/ps_2d.dat")
       write (6,*) "data read from file: ps_2d.dat"

       xs = 0.0; ! point source at the origin
       ys = 0.0; ! point source at the origin

       read (2,*) 
       read (2,*) 
       read (2,*) Nstr,rstr

c----------------------------------
c a periodic array of point sources
c----------------------------------

      elseif(Iselect.eq.2) then

       menu = 102

       open (2,file="DATA_IF/ps_2d_1p.dat")
       write (6,*) "data read from file: ps_2d_1p.dat"
 
       read (2,*) 
       read (2,*) 
       read (2,*) RL     ! point source separation
       read (2,*) Nstr,rstr

       strength = 1.0D0 

       xs = 0.0D0
       ys = 0.0D0

       rcross = 0.02

c-----------------------------------------------
c flow due to a 2D point source in above a plane
c-----------------------------------------------

      elseif(Iselect.eq.3) then

       menu = 103

       open (2,file="DATA_IF/ps_2d_w.dat")
       write (6,*) "data read from file: ps_2d_w.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) wall      ! wall is located at y = wall
       read (2,*) xs,ys     ! point source coordinates
       read (2,*) Nstr,rstr

       xsi =  xs            ! location of the image
       ysi = -ys+2.0*wall   ! location of the image

c-------------------------------------------
c flow due to a 2D point source in a channel
c-------------------------------------------

      elseif(Iselect.eq.4) then

       menu = 104

       open (2,file="DATA_IF/ps_2d_ww.dat")
       write (6,*) "data read from file: ps_2d_ww.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) wall1     ! one wall is located at y = wall1
       read (2,*) wall2     ! another wall is located at y = wall2
       read (2,*) xs,ys     ! point source coordinates
       read (2,*) Nstr,rstr

c--------------------------------------------------------
c flow due to a 2D point source in a semi-infinite strip
c--------------------------------------------------------

      else if(Iselect.eq.5) then

       menu = 105

       open (2,file="DATA_IF/ps_2d_www.dat")
       write (6,*) "data read from file: ps_2d_www.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) wall1     ! one wall is located at x = wall1
       read (2,*) wall2     ! another wall is located at x = wall2
       read (2,*) wall3     ! another wall is located at y = wall3
       read (2,*) xs,ys     ! point source coordinates
       read (2,*) Nstr,rstr

c---------------------------------------
c 2D point source in front of a cylinder
c---------------------------------------

      else if(Iselect.eq.6) then

       menu = 106
 
       open (2,file="DATA_IF/ps_2d_cl.dat")
       write (6,*) "data read from file: ps_2d_cl.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) xc,yc     ! cylinder center
       read (2,*) a         ! cylinder radius
       read (2,*) xs,ys     ! point source coordinates
       read (2,*) Nstr,rstr

       as    = a*a
       Dists = (xs-xc)**2+(ys-yc)**2     ! distace of point source
                                         ! from cylinder center
       xsi   = xc + as*(xs-xc)/Dists     ! inverse point
       ysi   = yc + as*(ys-yc)/Dists     ! inverse point
       R     = sqrt(Dists)

       rcross = a

c----------------------------------
c a periodic array of point sources
c above a wall
c----------------------------------

      else if(Iselect.eq.7) then

       menu = 107

       open (2,file="DATA_IF/ps_2d_1p_w.dat")
       write (6,*) "data read from file: ps_2d_1p_w.dat"
 
       read (2,*) 
       read (2,*) 
       read (2,*) RL    ! point source separation
       read (2,*) ys    ! wall at y = wall
       read (2,*) Nstr,rstr

       strength = 1.0   ! point sinks
       xs   = 0.0
       wall = 0.0

       xcross = xs
       ycross = ys

       rcross = 0.02

c--------------------
c 2D potential dipole
c--------------------

      elseif(Iselect.eq.8) then

       menu = 108

       open (2,file="DATA_IF/pd_2d.dat")
       write (6,*) "data read from file: pd_2d.dat"

       read (2,*) 

c--------------------------------
c 2D point source in uniform flow
c--------------------------------

      elseif(Iselect.eq.9) then

       menu = 109

       open (2,file="DATA_IF/ps_2d_uf.dat")
       write (6,*) "data read from file: pd_2d_uf.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) xs,ys
       read (2,*) beta
       read (2,*) Nstr,rstr

c---------------------------------------
c 2D source panel with constant strength
c---------------------------------------

      else if(Iselect.eq.10) then

       menu = 110

       open (2,file="DATA_IF/csp.dat")
       write (6,*) "data read from file: csp.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) b      ! panel semi-width
       read (2,*) sig0   ! panel strength

c---------
      else
c---------

       write (6,*) " Unfortunate choice; please try again"
       Go to 399

c------------
      end if
c------------

c===============================
      elseif(Ichoose.eq.10) then
c===============================

      write (6,*) 
      write (6,*) "  POTENTIAL FLOW II"
      write (6,*) "  -----------------"
      write (6,*) 
      write (6,*) " Enter: "
      write (6,*) 
      write (6,*) "21 for a ring of point sources"
      write (6,*) "22 for a ring of point sources before a wall"
      write (6,*) "29 for flow near a corner"
      write (6,*) "30 for uniform flow past a stationary cylinder "
      write (6,*) "31 for uniform flow due to a moving cylinder "
      write (6,*) "32 for flow past a flat plate"
      write (6,*) "33 for flow past an ellipse "
      write (6,*) "34 for flow past a Joukowski airfoil "
      write (6,*) "35 for flow exiting a channel"
      write (6,*) "36 for flow over a circular bump"
      write (6,*) "37 for flow down and up a strip"
      write (6,*) "38 for uniform flow past a stationary sphere"
      write (6,*) "39 for flow due to a moving sphere"
      write (6,*) "40 for orthogonal stag-point flow past a sphere"
      write (6,*) "51 for axial flow past a prolate spheroid"
      write (6,*) "52 for transverse flow past a prolate spheroid"
      write (6,*) "53 for axial flow past an oblate spheroid"
      write (6,*) "54 for transverse flow past an oblate spheroid"
      write (6,*) 
      write (6,*) "99 to return to the main menu"
      write (6,*) "0 to quit"
      write (6,*) "---------"

      read (5,*) Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c------------------------
c a ring of point sources
c------------------------

      if(Iselect.eq.21) then
 
       menu = 121

       open (2,file="DATA_IF/ps_ring.dat")
       write (6,*) "data read from file: ps_ring.dat"
 
       read (2,*) 
       read (2,*) 
       read (2,*) xs,ys
       read (2,*) Nstr,rstr

c--------------------------------------
c a ring of point sources before a wall
c--------------------------------------

      elseif(Iselect.eq.22) then

       menu = 122

       open (2,file="DATA_IF/ps_ring_w.dat")
       write (6,*) "data read from file: ps_ring_w.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) xs,ys
       read (2,*) wall
       read (2,*) Nstr,rstr

c-------------------
c flow near a corner
c-------------------

      elseif(Iselect.eq.29) then

       menu = 129
 
       write (6,*)
       write (6,*) " The corner angle is pi/m"
       write (6,*) " Please enter m;"
       write (6,*) "  m = 2   will give a right-angle corner  "
       write (6,*) "  m = 0.5 will give a semi-infinite plate "
       write (6,*) " Please choose m = 4 or lower"
       write (6,*) " I recommend 4 and 0.5"
       write (6,*) " ---------------------"
       read  (5,*) rm

       tol = 0.000001

       if(abs(rm-4.00).lt.tol) then
        open (2,file="DATA_IF/crn_040.dat")
        write (6,*) "data read from file: crn_040.dat"
       elseif(abs(rm-0.50).lt.tol) then
        open (2,file="DATA_IF/crn_005.dat")
        write (6,*) "data read from file: crn_005.dat"
       else
        open (2,file="DATA_IF/crn.dat")
        write (6,*) "data read from file: crn.dat"
       end if

       read (2,*) 

c-------------------------------
c flow past a stationary cylinder
c--------------------------------

      elseif(Iselect.eq.30) then

       menu = 130

  339  Continue

       write (6,*)
       write (6,*) " Enter the circulation parameter:"
       write (6,*) " lambda = -k/(4*pi*a*U)"
       write (6,*) " Choose from 0, 0.50, 1.0, 1.2"
       write (6,*) " -----------------------------"
       read  (5,*) circ

       tol = 0.000001

       if(abs(circ-0.0).le.tol) then
        open (2,file="DATA_IF/cl_s_000.dat")

       elseif(abs(circ-0.5).le.tol) then
        open (2,file="DATA_IF/cl_s_050.dat")

       elseif(abs(circ-1.0).le.tol) then
        open (2,file="DATA_IF/cl_s_100.dat")

       elseif(abs(circ-1.2).le.tol) then
        open (2,file="DATA_IF/cl_s_120.dat")

       else

        write (6,*) " Unfortunate choice"
        write (6,*) " Please try again"
        Go to 339

       end if

       xc = 0.0   ! cylinder center
       yc = 0.0   ! cylinder center

       a  = 1.0   ! cylinder radius

       rcross = a

c------------------------------
c flow due to a moving cylinder
c------------------------------

      elseif(Iselect.eq.31) then

       menu = 131

 334   continue

       write (6,*)
       write (6,*) " Enter the circulation parameter:"
       write (6,*) " lamda = k/(4*pi*a*U)"
       write (6,*) " choose from 0, 0.25, 0.50, 1.0"
       write (6,*) " ------------------------------"
       read  (5,*) circ

       tol = 0.000001

       if(abs(circ-0.0).le.tol) then
        open (2,file="DATA_IF/cl_m_000.dat")

       elseif(abs(circ-0.25).le.tol) then
        open (2,file="DATA_IF/cl_m_025.dat")

       elseif(abs(circ-0.5).le.tol) then
        open (2,file="DATA_IF/cl_m_050.dat")

       elseif(abs(circ-1.0).le.tol) then
        open (2,file="DATA_IF/cl_m_100.dat")

       else
        write (6,*) " Unfortunate choice"
        write (6,*) " Please try again"
        Go to 334

       end if

       a = 1.0   ! cylinder radius

       rcross = a

c-----------------------
c flow past a flat plate
c-----------------------

      elseif(Iselect.eq.32) then

       menu = 132

       write (6,*)
       write (6,*) " Select the triplet velocity/circulation:"
       write (6,*) " Vx, Vy, lamda= k/(2*pi*a*Vy)"
       write (6,*) " Kutta-Joukowski value is lamda=1"
       write (6,*)
  335  write (6,*) " To choose 1.0 1.0 0.0 enter 1"
       write (6,*) "           1.0 1.0 1.0       2"
       write (6,*) "           0.0 1.0 0.0       3"
       write (6,*) "           0.0 1.0 1.0       4"
       write (6,*) " -----------------------------"
      read  (5,*) Ind

      if(Ind.eq.1) then
        Vx   = 1.0
        Vy   = 1.0
        circ = 0.0
        open (2,file="DATA_IF/fp_110.dat")

      elseif(Ind.eq.2) then
        Vx   = 1.0
        Vy   = 1.0
        circ = 1.0
        open (2,file="DATA_IF/fp_111.dat")

      elseif(Ind.eq.3) then
        Vx   = 0.0
        Vy   = 1.0
        circ = 0.0
        open (2,file="DATA_IF/fp_010.dat")

      elseif(Ind.eq.4) then
        Vx   = 0.0
        Vy   = 1.0
        circ = 1.0
        open (2,file="DATA_IF/fp_011.dat")

      else
        write (6,*) " Sorry this selection is not valid"
        write (6,*) " Please try again"
        Go to 335

      end if

c---------------------
c flow past an ellipse
c---------------------

      elseif(Iselect.eq.33) then

       menu = 133

       write (6,*)
       write (6,*) " Enter the velocities Vx and Vy "
       write (6,*) " I recommend 1 and 1"
       write (6,*) " -------------------"
       read  (5,*) Vx,Vy

       write (6,*)
       write (6,*) " Enter the major and minor axis a, b"
       write (6,*) " I recommend 2 and 1"
       write (6,*) " -------------------"
       read  (5,*) a,b

       write (6,*)
       write (6,*) " Enter the circulation parameter:"
       write (6,*) " lambda = - k/(4*pi*a*Vx)"
       write (6,*) " I recommend 0 and 4;"
       write (6,*) " --------------------"
       read  (5,*) circ

       circ = circ/(a*Vx)

       tol = 0.000001

       if(abs(circ-0.0).le.tol) then
        open (2,file="DATA_IF/el_s_000.dat")
       else
        open (2,file="DATA_IF/el_s.dat")
       end if

c------------------------------
c flow past a Joukowski airfoil
c------------------------------

      elseif(Iselect.eq.34) then

       menu = 134

       write (6,*)
       write (6,*) " Enter the velocities Vx and Vy "
       write (6,*)
       write (6,*) " I recommend 1.0 and 0.1"
       write (6,*) " -----------------------"
       read  (5,*) Vx,Vy

       write (6,*)
       write (6,*) " Enter the x and y coordinates of the center"
       write (6,*) " of the Joukowski circle "
       write (6,*)
       write (6,*) " I recommend 0.1 and 0.1"
       write (6,*) " ------------------------"
       read  (5,*) xc,yc

       write (6,*)
       write (6,*) " Enter the circulation parameter:"
       write (6,*) " lamda = - k/(4*pi*a*Vx)"
       write (6,*)
       write (6,*) " I recommend 0 and 0.2"
       write (6,*) " ---------------------"
       read  (5,*) circ

       circ = circ/Vx

       tol = 0.000001

       if(abs(circ-0.0).le.tol) then
        open (2,file="DATA_IF/ja_s_000.dat")

       else
        open (2,file="DATA_IF/ja_s.dat")

       end if

c-----------------------------
c flow through an open channel
c-----------------------------

      elseif(Iselect.eq.35) then

       menu = 135

       open (2,file="DATA_IF/opchan.dat")

       shift = (1.0+log(pi))/pi 

c-----------------
c flow over a bump 
c-----------------

      elseif(Iselect.eq.36) then

       menu = 136

       open (2,file="DATA_IF/bump.dat")

c------------------
c flow over a strip
c------------------

      elseif(Iselect.eq.37) then

       menu = 137

       open (2,file="DATA_IF/strip.dat")

c------------------------------
c flow past a stationary sphere 
c------------------------------

      elseif(Iselect.eq.38) then

       menu = 138

       open (2,file="DATA_IF/sp_s.dat")
       read (2,*)
       read (2,*)
       read (2,*) a    ! sphere radius

       rcross = a

c----------------------------
c flow due to a moving sphere
c----------------------------

      elseif(Iselect.eq.39) then

       menu = 139

       open (2,file="DATA_IF/sp_m.dat")
       read (2,*)
       read (2,*)
       read (2,*) a     ! sphere radius

       rcross = a

c-----------------------------------------------
c orthogonal stagnation-point flow past a sphere
c-----------------------------------------------

      elseif(Iselect.eq.40) then

       menu = 140

       open (2,file="DATA_IF/spf_sp.dat")
 
       read (2,*) 
       read (2,*) Nstr
  
       a = 1.0    ! sphere radius

c-----------------------------
c axial flow past a prolate spheroid
c-----------------------------

      elseif(Iselect.eq.51) then

       menu = 151

       write (6,*)
       write (6,*) " Enter aspect ratio b/a (< 1)"
       write (6,*) " ----------------------------"
       read  (5,*) boa

       a = 1.0;
       b = a*boa;

       open (2,file="DATA_IF/sphrd_pr.dat")

c-----------------------------
c transverse flow past a prolate spheroid
c-----------------------------

      elseif(Iselect.eq.52) then

       menu = 152

       write (6,*)
       write (6,*) " Enter aspect ratio b/a (< 1)"
       write (6,*) " ----------------------------"
       read  (5,*) boa

       a = 1.0;
       b = a*boa;

       open (2,file="DATA_IF/sphrd_pr.dat")

c-----------------------------
c axial flow past an oblate spheroid
c-----------------------------

      elseif(Iselect.eq.53) then

       menu = 153

       write (6,*)
       write (6,*) " Enter aspect ratio a/b (< 1)"
       write (6,*) " ---------------------------"
       read  (5,*) aob

       b = 1.0;
       a = b*aob;

       open (2,file="DATA_IF/sphrd_ob.dat")

c-----------------------------
c axial flow past an oblate spheroid
c-----------------------------

      elseif(Iselect.eq.54) then

       menu = 154

       write (6,*)
       write (6,*) " Enter aspect ratio a/b (< 1)"
       write (6,*) " ---------------------------"
       read  (5,*) aob

       b = 1.0;
       a = b*aob;

       open (2,file="DATA_IF/sphrd_ob.dat")

c-------------------------
c END OF IRROTATIONAL FLOW
c-------------------------

c---------
      else
c---------

       write (6,*) " Unfortunate choice; please try again"
       Go to 98

c-----------
      end if
c-----------

c===============================
      elseif(Ichoose.eq.11) then
c===============================

      write (6,*) 
      write (6,*) " POTENTIAL FLOW III"
      write (6,*) " ------------------"
      write (6,*) 
      write (6,*) " Enter: "
      write (6,*) 
      write (6,*) "70 for a 3D point source above a plane"
      write (6,*) "71 for a 3D point source above a plane"
      write (6,*) "72 for a 3D point source in front of a sphere"
      write (6,*) "73 for a planar array of 3D point sources"
      write (6,*) "74 for a 3D point source in uniform flow"
      write (6,*) "75 for axisymmetric flow past a Rankine ovoid"
      write (6,*) "76 for a 3D Potential Dipole"
      write (6,*) 
      write (6,*) "98 to return to the main menu"
      write (6,*) " 0 to quit"
      write (6,*) "----------"

      read (5,*)  Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c------------------
c a 3D point source
c------------------

      if(Iselect.eq.70) then

       menu = 170

       open (2,file="DATA_IF/ps_3d.dat")

       xs = 0.0; ! point source at the origin
       ys = 0.0; ! point source at the origin

       read (2,*) 
       read (2,*) 
       read (2,*) Nstr,rstr

c-----------------------------------------------
c flow due to a 3D point source in above a plane
c wall located at y = wall
c-----------------------------------------------

      else if(Iselect.eq.71) then

       menu = 171

       open (2,file="DATA_IF/ps_3d_w.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) wall      ! wall is located at y = wall
       read (2,*) xs,ys     ! point source coordinates
       read (2,*) Nstr,rstr

       xsi =  xs            ! location of image
       ysi = -ys+2.0*wall 

c---------------------------------------------------
c flow due to a 3D point source in front of a sphere
c---------------------------------------------------

      elseif(Iselect.eq.72) then

       menu = 172

       open (2,file="DATA_IF/ps_3d_sp.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) xc,yc,zc    ! sphere center
       read (2,*) a           ! sphere radius
       read (2,*) xs,ys,zs    ! coordinates of the point source
       read (2,*) Nstr,rstr

       Dists = (xs-xc)**2+(ys-yc)**2+(zs-zc)**2
       as  = a*a
       xsi = xc + as * (xs-xc)/Dists
       ysi = yc + as * (ys-yc)/Dists
       zsi = zc + as * (zs-zc)/Dists
       R   = sqrt(Dists)

       rcross = a

c-----------------------------------------------
c flow due to a planar array of 3D point sources
c-----------------------------------------------

      elseif(Iselect.eq.73) then

       menu = 173

       open (2,file="DATA_IF/ps_3d_2p.dat")

       read (2,*)
       read (2,*)
       read (2,*) a11,a12        ! coordinates of first base vector
       read (2,*) a21,a22        ! coordinates of second base vector
       read (2,*) Max1,Max2,Max3 ! truncation limits
       read (2,*) xs,ys,zs       ! coordinates of the point source
       read (2,*) Nstr,rstr

c--------------------------------
c 3D point source in uniform flow
c--------------------------------

      elseif(Iselect.eq.74) then

       menu = 174

       open (2,file="DATA_IF/ps_3d_uf.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) xs,ys
       read (2,*) beta
       read (2,*) Nstr,rstr

c--------------------------
c Flow past a Rankine ovoid
c--------------------------

      elseif(Iselect.eq.75) then

       menu = 175

       open (2,file="DATA_IF/ps_3d_rank.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) xs
       read (2,*) rlam
       read (2,*) Nstr,rstr

c--------------------
c 3D potential dipole
c--------------------

      elseif(Iselect.eq.76) then

       menu = 176

       open (2,file="DATA_IF/pd_3d.dat")

       read (2,*) 

c-----------
      end if
c-----------

c==============================
      elseif(Ichoose.eq.2) then
c==============================

  449 write (6,*) 
      write (6,*) "  VORTEX FLOWS"
      write (6,*) "  ------------"
      write (6,*) 
      write (6,*) " Enter "
      write (6,*) 
      write (6,*) " 1 for a Stuart periodic array of point vortices"
      write (6,*) " 2 for a pair of point vortices "
      write (6,*) " 3 for a point vortex outside a circular cylinder"
      write (6,*) " 4 for a point vortex inside  a circular cylinder"
      write (6,*) " 5 for a point vortex between two walls"
      write (6,*) " 6 for a point vortex in a semi-infinite strip"
      write (6,*) " 8 for periodic array of vortex blobs"
      write (6,*) " 9 for periodic vortex street"
      write (6,*) "10 line vortex ring"
      write (6,*) "11 line vortex ring in front of a plane"
      write (6,*) "12 line vortex ring in front of a sphere"
      write (6,*) "13 line vortex ring inside a sphere"
      write (6,*) "20 Hill's vortex"
      write (6,*) "30 for a vortex panel with constant strength"
      write (6,*) "31 for a vortex panel with linear strength"
      write (6,*) 
      write (6,*) "98 to return to the main menu"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"

      read  (5,*)  Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c---------------------------------
c Periodic array of point vortices
c in a shear layer
c---------------------------------

      if(Iselect.eq.1) then

       menu = 201

       open (2,file="DATA_VF/pv_pr.dat")
       write (6,*) "data read from file: pv_pr.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) RL    ! period

       write (6,*) 
       write (6,*) " Enter the parameter beta"
       write (6,*) 
       write (6,*) " beta = 0 gives a shear layer"
       write (6,*) " beta = 1 fives a periodic array of pv" 
       write (6,*) " -------------------------------------"
       read (5,*) beta

c-----------------------
c pair of point vortices
c-----------------------

      elseif(Iselect.eq.2) then

       menu = 202

       open (2,file="DATA_VF/pv_pair.dat")
       write (6,*) "data read from file: pv_pair.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) ypv1
       read (2,*) ypv2

c---------------------------------------------
c point vortex in front of a circular cylinder
c---------------------------------------------

      elseif(Iselect.eq.3) then

       menu = 203

       open (2,file="DATA_VF/pv_cl.dat")
       write (6,*) "data read from file: pv_cl.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) xc,yc
       read (2,*) a
       read (2,*) xpv,ypv
       read (2,*) Nstr

       rcross = a

c----------------------------------------
c point vortex inside a circular cylinder
c----------------------------------------

      elseif(Iselect.eq.4) then

       menu = 204

       open (2,file="DATA_VF/pv_incl.dat")
       write (6,*) "data read from file: pv_incl.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) xc,yc
       read (2,*) a
       read (2,*) xpv,ypv
       read (2,*) Nstr

c----------------------------------------
c point vortex between two parallel walls 
c walls are located at y = 0 and y = y_wall sep
c----------------------------------------

      elseif(Iselect.eq.5) then

       menu = 205

       open (2,file="DATA_VF/pv_ww.dat")
       write (6,*) "data read from file: pv_ww.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) wall1,wall2
       read (2,*) xpv,ypv
       read (2,*) Nstr

c--------------------------------------
c point vortex in a semi-infinite strip
c--------------------------------------

      elseif(Iselect.eq.6) then

       menu = 206

       open (2,file="DATA_VF/pv_www.dat")
       write (6,*) "data read from file: pv_www.dat"

       read (2,*) 
       read (2,*) 
       read (2,*) wall1,wall2
       read (2,*) wall3
       read (2,*) xpv,ypv
       read (2,*) Nstr

c-------------------------------
c periodic array of vortex blobs 
c-------------------------------

      elseif(Iselect.eq.8) then

       menu = 208

       open (2,file="DATA_VF/vb_pr.dat")
       write (6,*) "data read from file: vb_pr.dat"

       read (2,*) 
       read (2,*) 
       read (2,*)  RL        ! period

       write (6,*) 
       write (6,*) " Enter the influence radius sigma"
       write (6,*) " sigma = 0 yields a point vortex"
       write (6,*) " -------------------------------"
       read (5,*) blob_rad

c-----------------------
c periodic vortex street
c-----------------------

      elseif(Iselect.eq.9) then

       menu = 209

       write (6,*) 
       write (6,*) " Enter:"
       write (6,*) 
       write (6,*) " 0 for symmetric"
       write (6,*) " 1 for antisymmetric"
       write (6,*) " -------------------"
       write (6,*) 
       read  (5,*) Isym

       write (6,*) " Enter the horizontal and vertical separation"
       write (6,*) " I recommend 2 and 2"
       write (6,*) " -------------------"
       write (6,*) 
       read  (5,*) a,b

       if(Isym==0) then
        open (2,file="DATA_VF/vs_sym.dat")
       end if
       if(Isym==1) then
        open (2,file="DATA_VF/vs_ant.dat")
       end if

       read (2,*) 

c-----------------
c line vortex ring
c-----------------

      elseif(Iselect.eq.10) then

       menu = 210

       open (2,file="DATA_VF/lvr.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) xr   ! coordinates of the ring
       read (2,*) sr
       read (2,*) Nstr

c------------------------------------
c line vortex ring in front of a wall
c------------------------------------

      elseif(Iselect.eq.11) then

      menu = 211

      open (2,file="DATA_VF/lvr_w.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) xr    ! coordinates of the ring
       read (2,*) sr
       read (2,*) wall
       read (2,*) Nstr

       xri = 2*wall-xr  ! coordinates of the image ring
       sri = sr

c--------------------------------------
c line vortex ring in front of a sphere
c--------------------------------------

      elseif(Iselect.eq.12) then

       menu = 212

       open (2,file="DATA_VF/lvr_sp.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) xc         ! sphere center
       read (2,*) a          ! sphere radius
       read (2,*) xr,sr
       read (2,*) Nstr,rstr

       yc = 0.0

       rcross = a

c---------------------------------
c line vortex ring inside a sphere
c---------------------------------

      elseif(Iselect.eq.13) then

       menu = 213

       open (2,file="DATA_VF/lvr_insp.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) xc
       read (2,*) a
       read (2,*) xr,sr
       read (2,*) Nstr,rstr

       yc = 0.0

c---------------
c  Hill's vortex
c---------------

      elseif(Iselect.eq.20) then

       menu = 220

  444  Continue
       write (6,*) 
       write (6,*) " Enter: "
       write (6,*) 
       write (6,*) " 0 for a frame moving with the vortex"
       write (6,*) " 1 for a stationary frame"
       write (6,*) " ------------------------"
       read (5,*) Iframe

       if(Iframe.eq.0) then
          open (2,file="DATA_VF/hill_m.dat")

       elseif(Iframe.eq.1) then
         open (2,file="DATA_VF/hill_s.dat")

       else
        write (6,*) " Unfortunate choice"
        write (6,*) " Please try again"
        Go to 444

       end if

       a = 1.0          ! vortex radius

c------------------------------------
c vortex panel with constant strength
c------------------------------------

      elseif(Iselect.eq.30) then

       menu = 230

       open (2,file="DATA_VF/cvp.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) b    ! panel semi-width
       read (2,*) g0   ! panel strength

c------------------------------------
c vortex panel with constant strength
c------------------------------------

      elseif(Iselect.eq.31) then

       menu = 231

       open (2,file="DATA_VF/lvp.dat")

       read (2,*) 
       read (2,*) 
       read (2,*) b    ! panel semi-width
       read (2,*) g0   ! panel strength
       read (2,*) g1   ! panel strength

c---------------------------------
c No more options for vortex flows
c---------------------------------

      else

        write (6,*) " Unfortunate choice; please try again"
        Go to 449

      End If

c==============================
      elseif(Ichoose.eq.3) then
c==============================

 569  Continue

      write (6,*) 
      write (6,*) "  STOKES FLOW I"
      write (6,*) "  -------------"
      write (6,*) 
      write (6,*) " Enter: "
      write (6,*) 
      write (6,*) " 1 for stagnation point flow on a wall"
      write (6,*) " 2 for flow near the Taylor scraper"
      write (6,*) " 3 for flow due to a moving belt"
      write (6,*) " 4 for flow due to a plunging tape"
      write (6,*) " 5 for flow in a corner"
      write (6,*) " 6 for flow due to shear stress at a corner"
      write (6,*) " 7 for flow due to a moving sphere"
      write (6,*) " 8 for flow due to a moving drop"
      write (6,*) " 9 for a 3D point force"
      write (6,*) "10 for a 2D point force"
      write (6,*)
      write (6,*) "99 to return to the main menu"
      write (6,*) " 0 to quit"
      write (6,*) "----------"

      read  (5,*)  Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c----------------------
c stagnation-point flow
c----------------------

      if(Iselect.eq.1) then

       menu = 301

 555   write (6,*) 
       write (6,*) " Please enter the angle alpha "
       write (6,*) " in multiples of pi"
       write (6,*) 
       write (6,*) " Choose from 0.10, 0.25, 0.50"
       write (6,*) " ------------------------------"
       read  (5,*) alpha

       tol = 0.0000001

       if(abs(alpha-0.10).le.tol) then 
         open (2,file="DATA_SF/spf_010.dat")
       elseif(abs(alpha-0.25).le.tol) then 
          open (2,file="DATA_SF/spf_025.dat")
       elseif(abs(alpha-0.50).le.tol) then
          open (2,file="DATA_SF/spf_050.dat")
       else
        write (6,*) " Invalid choice"
        Go to 555

      end if

      alpha = pi*alpha

c--------
c Scraper
c--------

      elseif(Iselect.eq.2) then

       menu = 302
 
 556   write (6,*) 
       write (6,*) " Please enter the angle alpha "
       write (6,*) " in multiples of pi"
       write (6,*) " choose from 0.25, 0.50, 0.75 1.00"
       write (6,*) " ---------------------------------"
       read  (5,*) alpha

       tol = 0.0000001

       if(abs(alpha-0.25).le.tol) then
          open (2,file="DATA_SF/scr_025.dat")
          write (6,*) "data read from file: scr_025.dat"

       else If(abs(alpha-0.50).le.tol) then
          open (2,file="DATA_SF/scr_050.dat")
          write (6,*) "data read from file: scr_050.dat"

       else if(abs(alpha-0.75).le.tol) then
          open (2,file="DATA_SF/scr_075.dat")
          write (6,*) "data read from file: scr_075.dat"

       else if(abs(alpha-1.00).le.tol) then
          open (2,file="DATA_SF/scr_100.dat")
          write (6,*) "data read from file: scr_100.dat"

       else
        write (6,*) " Unfortunate choice"
        write (6,*) " Please try again"
        Go to 556

       end if

       alpha = pi*alpha

c-------
c Finite belt on a wall
c-------

      elseif(Iselect.eq.3) then

       menu = 303

       open (2,file="DATA_SF/belt.dat")

       write (6,*) 
       write (6,*) " Please enter the length of the belt"
       write (6,*) " I suggest 0.1"
       write (6,*) " -------------"
       read  (5,*) width

c-----
c plunging tape
c-----

      elseif(Iselect.eq.4) then

       menu = 304

       write (6,*) 
       write (6,*) " Please enter the angle alpha"
       write (6,*) " in multiples of pi"
       write (6,*) " I suggest 0.25"
       write (6,*) " --------------"
       read  (5,*) alpha

       open (2,file="DATA_SF/tape.dat")

       alpha = pi*alpha

c-----------------
c Flow in a corner
c-----------------

      elseif(Iselect.eq.5) then

       menu = 305

  557  write (6,*) 
       write (6,*) " Choose the flow"
       write (6,*) 
       write (6,*) " 1 for antisymmetric flow with a=135 deg" 
       write (6,*) " 2 for antisymmetric flow with a= 45 deg" 
       write (6,*) " 3 for antisymmetric flow with a= 30 deg" 
       write (6,*) " 4 for antisymmetric flow with a= 10 deg" 
       write (6,*) "10 for     symmetric flow with a=135 deg" 
       write (6,*) "11 for     symmetric flow with a= 10 deg" 
       write (6,*) " ---------------------------------------"
       read  (5,*) Iflow

       if(Iflow.eq.1) then
         alpha   = 0.75*pi
         rlr     = 1.5444
         rli     = 0.0
         Isym    = 0
         Ieddies = 0              ! no eddies
         open (2,file="DATA_SF/crn_a_135.dat")

        elseif(Iflow.eq.2) then
         alpha   = 0.25*pi
         rlr     = 3.739593
         rli     = 1.119025
         Isym    = 0
         Ieddies = 1              ! eddies present
         open (2,file="DATA_SF/crn_a_045.dat")

        elseif(Iflow.eq.3) then
         alpha   = pi/6.0
         rlr     = 5.059329
         rli     = 1.952050
         Isym    = 0
         Ieddies = 1              ! eddies present
         open (2,file="DATA_SF/crn_a_030.dat")

        elseif(Iflow.eq.4) then
         alpha   = pi/18.0
         rlr     = 13.079480
         rli     =  6.384388
         Isym    = 0.0
         Ieddies = 1              ! eddies present
         open (2,file="DATA_SF/crn_a_010.dat")

       elseif(Iflow.eq.10) then
         alpha   = 0.75*pi
         rlr     = 1.908529
         rli     = 0.0
         Isym    = 1
         Ieddies = 0              ! no eddies
         open (2,file="DATA_SF/crn_s_135.dat")

       elseif(Iflow.eq.11) then
         alpha   = pi/18.0
         rlr     = 22.486415
         rli     =  7.871079
         Isym    = 1
         Ieddies = 1              ! eddies present
         open (2,file="DATA_SF/crn_s_010.dat")

       Else
        write (6,*) " Unfortunate choice"
        write (6,*) " Please try again"
        Go to 557

       End If

c------------------------------------
c flow due to a constant shear stress
c at a corner
c------------------------------------

      elseif(Iselect.eq.6) then

       menu = 306

 558   continue

       write (6,*) 
       write (6,*) " Please enter the angle alpha "
       write (6,*) " in multiples of pi"
       write (6,*) 
       write (6,*) " Choose from 0.25, 0.50, 0.75, 0.90"
       write (6,*) " Enter 0 to quit"
       write (6,*) " ---------------"
       read  (5,*) alpha

       tol = 0.0000001

       if(abs(alpha).lt.tol) then
         stop

       elseif(abs(alpha-0.25).lt.tol) then
           open (2,file="DATA_SF/crn_sh_025.dat")

       elseif(abs(alpha-0.50).lt.tol) then
           open (2,file="DATA_SF/crn_sh_050.dat")

       elseif(abs(alpha-0.75).lt.tol) then
           open (2,file="DATA_SF/crn_sh_075.dat")

       elseif(abs(alpha-0.90).lt.tol) then
           open (2,file="DATA_SF/crn_sh_090.dat")

       else
        write (6,*) " Unfortunate choice"
        write (6,*) " Please try again"
        Go to 558

      end if

      alpha = pi*alpha

c--------------------
c translating sphere
c--------------------

      elseif(Iselect.eq.7)  then

       menu = 307

 559   write (6,*)
       write (6,*) " Enter:"
       write (6,*) 
       write (6,*) " 0 for a frame moving with the sphere"
       write (6,*) " 1 for a stationary frame of reference"
       write (6,*) " -------------------------------------"
       read (5,*) Iframe

       if(Iframe.eq.0) then
          open (2,file="DATA_SF/sp_m.dat")

       elseif(Iframe.eq.1) then
         open (2,file="DATA_SF/sp_s.dat")

       else
        write (6,*) " Unfortunate choice"
        write (6,*) " Please try again"
        Go to 559

       end if

      read (2,*)
      read (2,*)
      read (2,*) a    ! sphere radius

      rcross = a

c------------------
c translating drop
c-----------------

      else if(Iselect.eq.8) then

       menu = 308

 560   write (6,*)
       write (6,*) " Enter 0 for a frame moving with the drop"
       write (6,*) "       1 for a stationary frame"
       write (6,*) " ------------------------------"
       read (5,*) Iframe

       if(Iframe.eq.0) then
         open (2,file="DATA_SF/dr_m.dat")
       else if(Iframe.eq.1) then
         open (2,file="DATA_SF/dr_s.dat")
       else
        write (6,*) " unwise choice; please try again"
        Go to 560
       end if

       read (2,*)
       read (2,*)
       read (2,*) a    ! drop radius

       write (6,*) " Please enter the viscosity ratio"
       write (6,*) " --------------------------------"
       read  (5,*) vsr

       rcross = a

c--------------
c  3D Stokeslet
c--------------

      elseif(Iselect.eq.9)  then

       menu = 309

       open (2,file="DATA_SF/sgf_3d_fs.dat")
       write (6,*) "data read from file: sgf_3d_fs.dat"

c--------------
c 2D Stokeslet
c--------------

      elseif(Iselect.eq.10)  then

       menu = 310

       open (2,file="DATA_SF/sgf_2d_fs.dat")
       write (6,*) "data read from file: sgf_2d_fs.dat"

      end if

c===============================
      elseif(Ichoose.eq.31) then
c===============================

 383  Continue

      write (6,*) 
      write (6,*) "  STOKES FLOW II"
      write (6,*) "  --------------"
      write (6,*) 
      write (6,*) " Enter: "
      write (6,*) 
      write (6,*) "11 for a doubly periodic lattice of 2D point forces"
      write (6,*) "12 for a triply periodic lattice of 3D point forces"
      write (6,*) "13 for a 3D point force near a sphere"
      write (6,*) "14 for a 3D point force near a plane wall"
      write (6,*) "15 for a doubly periodic array of 3D point forces"
      write (6,*) "16 for a periodic array of 2D point forces"
      write (6,*) "17 for a per array of 2D point forces in a channel"
      write (6,*) "18 for a 2D point force near a plane wall"
      write (6,*) "19 for an array of 2D point forces near a plane wall"
      write (6,*) 
      write (6,*) "99 to return to the main menu"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"

      read  (5,*)  Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c--------------------------------
c doubly periodic array of 2D point forces
c--------------------------------

      if(Iselect.eq.11)  then

       menu = 311

       open (2,file="DATA_SF/sgf_2d_2p.dat")
       write (6,*) "data read from file: sgf_2d_2p.dat"

       read (2,*)
       read (2,*)
       read (2,*) a11,a12
       read (2,*) a21,a22
       read (2,*) Max1,Max2

c--------------------------------
c triply periodic array of 3D point forces
c--------------------------------

      elseif(Iselect.eq.12)  then

       menu = 312

       open (2,file="DATA_SF/sgf_3d_3p.dat")

       write (6,*) "data read from file: sgf_3d_3p.dat"

       read (2,*)
       read (2,*)
       read (2,*) a11,a12,a13
       read (2,*) a21,a22,a23
       read (2,*) a31,a32,a33
       read (2,*) Max1,Max2

c-------------------------------------
c  3D point force in front of a sphere
c-------------------------------------

      elseif(Iselect.eq.13) then

       menu = 313

 561   Continue
       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for axial orientation"
       write (6,*) " 2 for azimuthal orientation"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
          Go to 98
       elseif(Iflow.eq.1) then
          open (2,file="DATA_SF/sgf_3d_sph_rd.dat")
          write (6,*) "data read from file: sgf_3d_sph_rd.dat"
       elseif(Iflow.eq.2) then
          open (2,file="DATA_SF/sgf_3d_sph_az.dat")
          write (6,*) "data read from file: sgf_3d_sph_az.dat"
       else
          write (6,*) " Unfortunate choice"
          write (6,*) " Please try again"
          Go to 561
       end if

      read (2,*)
      read (2,*) xc,yc,zc        ! sphere center
      read (2,*) a               ! sphere radius
      read (2,*) xs,ys,zs        ! position of point force
      read (2,*) Nstr

      rcross = a

c------------------------------------
c  3D point force above a plane wall
c------------------------------------

      elseif(Iselect.eq.14) then

      menu = 314

 562   write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for normal orientation"
       write (6,*) " 2 for parallel orientation"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
          Go to 98
       elseif(Iflow.eq.1) then
          open (2,file="DATA_SF/sgf_3d_w_n.dat")
          write (6,*) "data read from file: sgf_3d_w_n.dat"
       elseif(Iflow.eq.2) then
          open (2,file="DATA_SF/sgf_3d_w_p.dat")
          write (6,*) "data read from file: sgf_3d_w_p.dat"
       else
          write (6,*) " Unfortunate choice"
          write (6,*) " Please try again"
          Go to 562
       end if

      wall = 0.0

c----------------------------
c 2p array of 3d point forces
c----------------------------

      elseif(Iselect.eq.15)  then

       menu = 315

 315   Continue

       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for symmetric flow in the xz plane"
       write (6,*) " 2 for the xz plane"
       write (6,*) " 3 for the xy plane"
       write (6,*) " 0 to quit"
       write (6,*) " ----------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
         Go to 98
       elseif(Iflow.eq.1) then
         open (2,file="DATA_SF/sgf_3d_2p_xz.dat")
         write (6,*) "data read from file: sgf_3d_2p_xz.dat"
       elseif(Iflow.eq.2) then
         open (2,file="DATA_SF/sgf_3d_2p_xy.dat")
         write (6,*) "data read from file: sgf_3d_2p_xy.dat"
       elseif(Iflow.eq.3) then
         open (2,file="DATA_SF/sgf_3d_2p_xzs.dat")
         write (6,*) "data read from file: sgf_3d_2p_xzs.dat"
       else
         write (6,*) " unwise choice, please try again"
         Go to 315
       end if

       read (2,*)
       read (2,*)
       read (2,*) a11,a12
       read (2,*) a21,a22
       read (2,*) Max1,Max2

c----------------------------
c 1P array of 2D point forces
c----------------------------

      elseif(Iselect.eq.16)  then

       menu = 316

 316   Continue
       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for streamwise orientation"
       write (6,*) " 2 for transverse orientation"
       write (6,*) " 3 for streamwise symmetric"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
         Go to 98
       elseif(Iflow.eq.1) then
         open (2,file="DATA_SF/sgf_2d_1p_p.dat")
         write (6,*) "data read from file: sgf_2d_1p_p.dat"
       elseif(Iflow.eq.2) then
         open (2,file="DATA_SF/sgf_2d_1p_n.dat")
         write (6,*) "data read from file: sgf_2d_1p_n.dat"
       elseif(Iflow.eq.3) then
         open (2,file="DATA_SF/sgf_2d_1p_ps.dat")
         write (6,*) "data read from file: sgf_2d_1p_ps.dat"
       else
        write (6,*) " unwise choice, please try again"
        Go to 316
       end if

       read (2,*)
       read (2,*)
       read (2,*) RL

c--------------------------------------------
c An 1P array of 2D point forces in a channel
c--------------------------------------------

      elseif(Iselect.eq.17)  then

       menu = 317

 534   Continue

       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for normal orientation"
       write (6,*) " 2 for parallel orientation"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
          Go to 98
       elseif(Iflow.eq.1) then
          open (2,file="DATA_SF/sgf_2d_1p_ww_n.dat")
          write (6,*) "data read from file: sgf_2d_1p_ww_n.dat"
       elseif(Iflow.eq.2) then
          open (2,file="DATA_SF/sgf_2d_1p_ww_p.dat")
          write (6,*) "data read from file: sgf_2d_1p_ww_p.dat"
       else
          write (6,*) " Unfortunate choice"
          write (6,*) " Please try again"
          Go to 534
       end if

       read (2,*)
       read (2,*)
       read (2,*) RL

       xs = 0.0    ! coordinates of the point force

       if(Iflow.eq.1) then
        ys = 0.25
       else
        ys = 0.5
       end If

       h = 1.0

c------------------------------------
c 2D point force above a plane wall
c------------------------------------

      elseif(Iselect.eq.18) then

      menu = 318

 564  write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 for normal orientation"
      write (6,*) " 2 for parallel orientation"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Iflow

       if(Iflow.eq.0) then
          Go to 98
       elseif(Iflow.eq.1) then
          open (2,file="DATA_SF/sgf_2d_w_n.dat")
          write (6,*) "data read from file: sgf_2d_w_n.dat"
       elseif(Iflow.eq.2) then
          open (2,file="DATA_SF/sgf_2d_w_p.dat")
          write (6,*) "data read from file: sgf_2d_w_p.dat"
       else
          write (6,*) " Unfortunate choice"
          write (6,*) " Please try again"
          Go to 564
       End If

      wall = 0.0

c-----------------------------------------------
c an array of 2D point forces above a plane wall
c-----------------------------------------------

      elseif(Iselect.eq.19) then

      menu = 319

 565  Continue

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 for normal orientation"
      write (6,*) " 2 for parallel orientation"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Iflow

      if(Iflow.eq.0) then
        Go to 98
      elseif(Iflow.eq.1) then
        open (2,file="DATA_SF/sgf_2d_1p_w_n.dat")
        write (6,*) "data read from file: sgf_2d_1p_w_n.dat"
      elseif(Iflow.eq.2) then
        open (2,file="DATA_SF/sgf_2d_1p_w_p.dat")
        write (6,*) "data read from file: sgf_2d_1p_w_p.dat"
      else
        write (6,*) " unwise choice; please try again"
        Go to 565
      end if

      xs = 0.0  ! coordinates of the point force
      ys = 0.5
      RL = 1.0
      wall = 0.0

c------------
      else
c------------

      write (6,*) "unwise choice; please try again"
      Go to 383

c------------
      end if
c------------

c===============================
      elseif(Ichoose.eq.32) then
c===============================

 888  Continue

      write (6,*) 
      write (6,*) "  STOKES FLOW"
      write (6,*) "  ------------"
      write (6,*) 
      write (6,*) " Enter: "
      write (6,*)
      write (6,*) "20 for linear flow past a drop"
      write (6,*) "30 for quadratic stagnation-point flow"
      write (6,*) "40 for flow past a slender cylinder "
      write (6,*) "50 for shear flow over a plane wall"
      write (6,*) "   with a circular orifice"
      write (6,*) "51 for shear flow over a plane wall with"
      write (6,*) "   a discoidal zero-shear-stress patch"
      write (6,*) "52 for shear flow over a plane wall with a slit"
      write (6,*) "53 for shear flow over a plane wall with"
      write (6,*) "   a slit-like zero-shear-stress patch"
      write (6,*)
      write (6,*) "99 to return to the main menu"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"

      read (5,*) Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c-----------------------------------
c  linear flow past a spherical drop 
c-----------------------------------

      if(Iselect.eq.20) then

       menu = 320

       write (6,*)
       write (6,*) " Please enter the parameter chi in (0,1)"
       write (6,*) " 0 for simple shear flow"
       write (6,*) " 1 for orthogonal stagnation-point flow"
       write (6,*) " I recommend 0.5"
       write (6,*) " ---------------"
       read  (5,*) chi

      If(chi.eq.0) then
        open (2,file="DATA_SF/fpd_000.dat")
      Else
        open (2,file="DATA_SF/fpd.dat")
      End If

      read (2,*)
      read (2,*)
      read (2,*) a    ! drop radius

c----------------------------------
c  stagnation-point flow in a fluid
c----------------------------------

      elseif(Iselect.eq.30) then

       menu = 330

       open (2,file="DATA_SF/spff.dat")

c------------------------------
c  flow past a slender cylinder
c------------------------------

      elseif(Iselect.eq.40) then

       menu = 340

       write (6,*)
       write (6,*) " Please enter the aspect ratio a/b > 1" 
       write (6,*) " -------------------------------------"
       read  (5,*) aob

 340   Continue

       write (6,*)
       write (6,*) " Enter 1 for axial flow"
       write (6,*) "       2 for transverse flow"
       write (6,*) "       0 to quit"
       write (6,*) " ---------------"
       read  (5,*) Iflow

       if(Iflow.eq.1) then

        open (2,file="DATA_SF/slcl_ax.dat")

        a  = 1.0
        b  = a/aob
        RL = 2.0*a
        alpha = pi4/(2.0*log(RL/b)-1.0)

       elseif(Iflow.eq.2) then

        open (2,file="DATA_SF/slcl_tr.dat")

        b  = 1.0
        a  = b*aob
        RL = 2.0*a
        beta = pi8/(2.0*log(RL/b)+1.0)
        gama = - beta*b*b/4.0
        rcross = b

      else

        write (6,*) " unwise choice; please try again"
        Go to 340

      end if

c-----------------------------
c shear flow over a plane wall
c with a circular orifice
c-----------------------------

      elseif(Iselect.eq.50) then
  
       menu = 350
 
       open (2,file="DATA_SF/orifice_3D.dat")

       read (2,*)
       read (2,*)
       read (2,*) a   ! radius of the orifice

       a_open = a

c---------------------------------------------
c Shear flow over a plane wall with a discoidal 
c zero shear-stress patch
c---------------------------------------------

      elseif(Iselect.eq.51) then
  
       menu = 351

       open (2,file="DATA_SF/patch_3D.dat")

       read (2,*)
       read (2,*)
       read (2,*) a
 
       a_open = a

c-----------------------------------------
c shear flow over a plane wall with a slit
c-----------------------------------------

      elseif(Iselect.eq.52) then
  
       menu = 352

       write (6,*)
       write (6,*) " Enter the slit semi-width a" 
       write (6,*) " I recommend: 0.5"
       write (6,*) " ----------------"
       read  (5,*) a

       open (2,file="DATA_SF/orifice_2D.dat")

       a_open = a

c---------------------------------------------
c Shear flow over a plane wall with a slit-like 
c zero shear-stress patch
c---------------------------------------------

      elseif(Iselect.eq.53) then
  
       menu = 353

       write (6,*)
       write (6,*) " Enter the patch length" 
       write (6,*) " I recommend: 0.5"
       write (6,*) " ----------------"
       read  (5,*) a
 
       open (2,file="DATA_SF/patch_2D.dat")

       a_open = a

c---------
      else
c---------

       write (6,*) " Unfortunate choice; please try again"
       Go to 98

c-----------
      end if
c-----------

c===============================
      elseif(Ichoose.eq.33) then
c===============================

 531  Continue

      write (6,*) 
      write (6,*) "  STOKES FLOW"
      write (6,*) "  -----------"
      write (6,*) 
      write (6,*) " Enter: "
      write (6,*) 
      write (6,*) "60 for a ring of point forces"
      write (6,*) "61 for a ring of point forces near a plane"
      write (6,*) "62 for an array of rings of point forces"
      write (6,*) "63 for an array of rings of point forces in a tube"
      write (6,*)
      write (6,*) "70 for a 3D point source in front of a wall"
      write (6,*) "71 for a 2D point source in front of a wall"
      write (6,*) "72 for a 2D periodic point source"
      write (6,*) "73 for a 2D periodic point source above a wall"
      write (6,*) 
      write (6,*) "99 to return to the main menu"
      write (6,*) 
      write (6,*) " 0 to quit"
      write (6,*) " ---------"

      read  (5,*)  Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c---------------------
c ring of point forces
c---------------------

      if(Iselect.eq.60) then

       menu = 360

 579   Continue

       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for axial orientation"
       write (6,*) " 2 for radial orientation"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
         Go to 98
       elseif(Iflow.eq.1) then
         open (2,file="DATA_SF/sgf_ax_fs_ax.dat")
         write (6,*) "data read from file: sgf_ax_fs_ax.dat"
       elseif(Iflow.eq.2) then
         open (2,file="DATA_SF/sgf_ax_fs_rd.dat")
         write (6,*) "data read from file: sgf_ax_fs_rd.dat"
       else
         write (6,*) " invalid choice; Please try again"
         Go to 579
       end if

       xr = 0.0  ! ring location
       sr = 1.0

c----------------------------------------
c ring of point forces in front of a wall
c----------------------------------------

      elseif(Iselect.eq.61) then

       menu = 361

 519   Continue

       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for axial orientation"
       write (6,*) " 2 for radial orientation"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
         Go to 98
       elseif(Iflow.eq.1) then
         open (2,file="DATA_SF/sgf_ax_w_ax.dat")
         write (6,*) "data read from file: sgf_ax_w_ax.dat"
       elseif(Iflow.eq.2) then
         open (2,file="DATA_SF/sgf_ax_w_rd.dat")
         write (6,*) "data read from file: sgf_ax_w_rd.dat"
       else
          write (6,*) " invalid choice; Please try again"
          Go to 519
       end if

       xr = 0.5  ! ring location
       sr = 0.5
       wall = 0.0

c-------------------------------
c array of rings of point forces
c-------------------------------

      elseif(Iselect.eq.62) then

       menu = 362

 510   Continue

       write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for axial orientation"
       write (6,*) " 2 for radial orientation"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
         Go to 98
       elseif(Iflow.eq.1) then
         open (2,file="DATA_SF/sgf_ax_1p_ax.dat")
         write (6,*) "data read from file: sgf_ax_1p_ax.dat"
         read (2,*)
         read (2,*) cc
       elseif(Iflow.eq.2) then
         open (2,file="DATA_SF/sgf_ax_1p_rd.dat")
         write (6,*) "data read from file: sgf_ax_1p_rd.dat"
       else
         write (6,*) " invalid choice; Please try again"
         Go to 510
       end if

       xr = 0.0  ! ring location
       sr = 0.5

c-----------------------------------------
c array of rings of point forces in a tube
c-----------------------------------------

      elseif(Iselect.eq.63) then

       menu = 363

 511   write (6,*)
       write (6,*) " Enter:"
       write (6,*)
       write (6,*) " 1 for axial orientation"
       write (6,*) " 2 for radial orientation"
       write (6,*) " 0 to quit"
       write (6,*) " ---------"
       read  (5,*) Iflow

       if(Iflow.eq.0) then
         Go to 98
       elseif(Iflow.eq.1) then
         open (2,file="DATA_SF/sgf_ax_1p_ct_ax.dat")
         write (6,*) "data read from file: sgf_ax_1p_ct_ax.dat"
       elseif(Iflow.eq.2) then
         open (2,file="DATA_SF/sgf_ax_1p_ct_rd.dat")
         write (6,*) "data read from file: sgf_ax_1p_ct_rd.dat"
       else
         write (6,*) " invalid choice; Please try again"
         Go to 511
       end if

       xr = 0.0  ! ring location
       sr = 0.5
       sc = 1.0
       RL = 1.0  ! period

c---------------------------------
c point source in front of a plane
c---------------------------------

      elseif(Iselect.eq.70) then

       menu = 370

       open (2,file="DATA_SF/sgf_3d_w_source.dat")
       read (2,*) 
       read (2,*) 
       read (2,*) Nstr,rstr

       wall = 0.0
       xs = 1.0;
       ys = 0.0;

c------------------------------------
c 2D point source in front of a plane
c------------------------------------

      elseif(Iselect.eq.71) then

       menu = 371

       open (2,file="DATA_SF/sgf_2d_w_source.dat")
       read (2,*) 
       read (2,*) 
       read (2,*) Nstr,rstr

       wall = 0.0
       xs = 0.0;
       ys = 1.0;

c-------------------------
c 2D periodic point source
c-------------------------

      elseif(Iselect.eq.72) then

       menu = 372

       open (2,file="DATA_SF/sgf_2d_p_source.dat")
       read (2,*) 
       read (2,*) 
       read (2,*) Nstr,rstr
       read (2,*) RL

       xs = 0.0;
       ys = 0.0;

c-------------------------------------
c 2D periodic point source above a wall
c-------------------------------------

      elseif(Iselect.eq.73) then

       menu = 373

       open (2,file="DATA_SF/sgf_2d_pw_source.dat")
       read (2,*) 
       read (2,*) 
       read (2,*) Nstr,rstr
       read (2,*) RL

       wall = 0.0;
       xs = 0.0;
       ys = 0.1;

c------------
       end if
c------------

c==============================
      elseif(Ichoose.eq.4) then
c==============================

      write (6,*) 
      write (6,*) " MISCELLANEOUS "
      write (6,*) " --------------"
      write (6,*) 
      write (6,*) " Enter "
      write (6,*) 
      write (6,*) " 1 Kovasznay flow "
      write (6,*) " 2 Taylor cellular flow "
      write (6,*) " 3 Oseen flow due to a moving sphere"
      write (6,*) " 4 flow due to a point source of momentum"
      write (6,*) " 5 Blasius boundary layer on a flat plate"
      write (6,*) " 6 2D oblique stagnation point flow against a plate"
      write (6,*) " 7 Axis ortho stagn-point flow against a plate"
      write (6,*) " 8 Two-dimensional linear flow"
      write (6,*)
      write (6,*) "99 to return to the main menu"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"

      read  (5,*)  Iselect

      if(Iselect.eq.0) Go to 99
      if(Iselect.eq.99) Go to 98

c---------------
c Kovasznay flow
c---------------

      if(Iselect.eq.1) then

      menu = 401

 401  Continue

      write (6,*) " Enter Reynolds number (10 or 50)"
      write (6,*) " --------------------------------"
      read (5,*) Re

      tol = 0.00001

      if(abs(Re-10).le.tol) then
       open (2,file="DATA_VAR/kov_10.dat")
       write (6,*) "data read from file: kov_10.dat"
      elseif(abs(Re-50).le.tol) then
       open (2,file="DATA_VAR/kov_50.dat")
       write (6,*) "data read from file: kov_50.dat"
      else
       write (6,*) "unwise choice"
       Go to 401
      end if

c----------------------
c  Taylor cellular flow
c----------------------

      elseif(Iselect.eq.2) then

      menu = 402

      open (2,file="DATA_VAR/taylor_cel.dat")

c-------------------------
c  Oseen flow past a sphere
c-------------------------

      elseif(Iselect.eq.3) then

      menu = 403

      open (2,file="DATA_VAR/oseen_sp.dat")
       write (6,*) "data read from file: oseen_sp.dat"

      write (6,*) " Enter the Reynolds number"
      write (6,*) " I recommend 2"
      write (6,*) " ------------"
      read  (5,*) Re

      a = 1.0    ! sphere radius
      rcross = a

c--------------------------
c  point source of momentum
c--------------------------

      elseif(Iselect.eq.4) then

      menu = 404

      open (2,file="DATA_VAR/ps_mom.dat")
       write (6,*) "data read from file: ps_mom.dat"

      write (6,*) " Enter the constant D"
      write (6,*) " I recommend 1.1"
      write (6,*) " ---------------"
      read  (5,*) Dcn

c---------------------------------------
c Blasius boundary layer on a flat plate
c---------------------------------------

      elseif(Iselect.eq.5) then

      menu = 405

      open (2,file="DATA_VAR/blasius.dat")
       write (6,*) "data read from file: blasius.dat"

      open (10,file="DATA_VAR/blasius_sim.dat") ! read similarity solution
        read (10,*) Nblas
        Do i=1,Nblas
         read (10,*) idle,xblas(i),fblas(i),fpblas(i)
c        write (6,*) idle,xblas(i),fblas(i),fpblas(i)
        End Do
      close (10)

c-----------------------------------------------------
c 2D oblique stagnation-point flow toward a flat plate
c-----------------------------------------------------

      elseif(Iselect.eq.6) then

      menu = 406

      open (10,file="DATA_VAR/spf_sim.dat")       ! read similarity solution
        read (10,*) Nspf
        Do i=1,Nspf
         read (10,*) idle,xspf(i),fspf(i),gspf(i)
        End Do
      close (10)

      write (6,*) " Please enter the angle beta"
      write (6,*) "          in multiples of pi"
      write (6,*) " Choose 0.5 (orthogonal flow) or 0.25"
      write (6,*) " ------------------------------------"
      read  (5,*) beta
 
      tol = 0.0000001

      if(abs(beta-0.50).lt.tol) then
        open (2,file="DATA_VAR/spf_ortho.dat")
        write (6,*) "data read from file: spf_ortho.dat"
        beta = beta*pi
        alpha = 0.5*pi
      else
        open (2,file="DATA_VAR/spf.dat")
        write (6,*) "data read from file: spf.dat"
        beta = beta*pi
        if(beta.lt.pih) alpha = atan(-0.50*tan(beta))+pi
        if(beta.gt.pih) alpha = atan(-0.50*tan(beta))
      end if

c-----------------------------------------
c axisymmetric orthogonal stagnation-point 
c flow toward a flat plate
c-----------------------------------------

      elseif(Iselect.eq.7) then

      menu = 407

      open (10,file="DATA_VAR/homann_sim.dat")  ! similarity solution
        read (10,*) Nhom
        Do i=1,Nhom
         read (10,*) idle,xhom(i),fhom(i)
        End Do
      close (10)

      open (2,file="DATA_VAR/homann.dat")
      write (6,*) "data read from file: homman.dat"

c-----------------------------------------------------
c Linear flow described by u = y, v = chi * x
c-----------------------------------------------------

      elseif(Iselect.eq.8) then

      menu = 408

      write (6,*) " Enter the parameter chi in the range: (0,1)"
      write (6,*) " 0 gives simple shear flow"
      write (6,*) " ------------------------"
      read  (5,*) chi

      open (2,file="DATA_VAR/lin_flow.dat")
      write (6,*) "data read from file: lin_flow.dat"

c---------
      else
c---------

       write (6,*) " unwise choice; please try again"
       stop

c----
      end if     ! End of local menu
c----

c----
      end if     ! End of main menu
c----

       write (6,*) "flow code: ",menu

c===================================
c READ PARAMETERS
c ALL INPUT FILES MUST CONFORM WITH
c THIS FORMAT
c===================================

c-----
c
c Icrossx = 1 will stop when the streamline has crossed the x axis
c Icrossy = 1                                           the y axis
c Icrossr = 1                                 a circle
c                                             of radius rcross
c
c Isymo = 1 the streamlines will be reflected with respect 
c           to the origin
c Isymx = 1 the streamlines will be reflected with respect 
c           to the y axis
c Isymy = 1 the streamlines will be reflected with respect 
c           to the x axis
c-----

      read (2,*) 
      read (2,*) Ds,Nmax
      read (2,*) xmin,xmax,ymin,ymax
      read (2,*) nticsx,nticsy
      read (2,*) Icrossx,Icrossy,Icrossr
      read (2,*) Isymo,Isymx,Isymy
      read (2,*) Iperx,perx
      read (2,*) Ipery,pery

c===================================
c RECORD IN THE OUTPUT FILE
c===================================

      open (1,file="strml.out")

      write (1,202) menu,Iflow
      write (1,200) xmin,xmax,ymin,ymax
      write (1,201) nticsx,nticsy
      write (1,202) Isymo,Isymx,Isymy
      write (1,203) Iperx,perx
      write (1,204) Ipery,pery

c=================================
c  READ OR COMPUTE STARTING POINTS
c  FOR THE STREAMLINES 
c=================================

c--------------------------
      if(       menu.eq.104
     +      .or.menu.eq.103
     +      .or.menu.eq.102
     +      .or.menu.eq.370
     +      .or.menu.eq.371
     +      .or.menu.eq.372
     +      .or.menu.eq.373
     +  ) then
c--------------------------

         Dth = pi2/Nstr

         Do i=1,Nstr
          theta = Dth*(i-0.5)
          x0(i) = xs + rstr*cos(theta)
          y0(i) = ys + rstr*sin(theta)
         End Do

c--------------------------
      elseif(   menu.eq.101
     +      .or.menu.eq.105
     +      .or.menu.eq.106
     +      .or.menu.eq.107
     +      .or.menu.eq.170
     +      .or.menu.eq.171
     +      .or.menu.eq.172
     +      .or.menu.eq.173
     +      .or.menu.eq.122
     +  ) then
c--------------------------

         Dth = pi2/Nstr

         Do i=1,Nstr
          theta = Dth*(i-1.0)
          x0(i) = xs + rstr*cos(theta)
          y0(i) = ys + rstr*sin(theta)
         End Do

c--------------------------
      elseif(   menu.eq.109
     +      .or.menu.eq.174
     +      .or.menu.eq.175) then
c--------------------------

         Dth = pi/(Nstr-1.0)

         Do i=1,Nstr          ! around the singularity
          theta = Dth*(i-1.0)
          x0(i) = xs + rstr*cos(theta)
          y0(i) = ys + rstr*sin(theta)
         End Do

         Dy = ymax/Nstr

         Do i=1,Nstr          ! on x = xmin
          j = Nstr+i
          x0(j) = xmin
          y0(j) = (i-1.0)*Dy
         End Do

         Nstr = 2*Nstr

c--------------------------
      elseif(menu.eq.121) then
c--------------------------

         Dth = pi/(Nstr-1.0)
         Do i = 1,Nstr
          theta = -pih+0.01+Dth*(i-1.0)
          x0(i) = xs + rstr*cos(theta)
          y0(i) = ys + rstr*sin(theta)
         End Do

c--------------------------
      elseif(menu.eq.140) then
c--------------------------

         Dx = xmax/(Nstr)

         Do i = 1,Nstr
          x0(i) = 0.001+Dx*(i-1.0)
          y0(i) = ymax
         End Do

c--------------------------
      elseif(menu.eq.203) then
c--------------------------

        Dx = (xpv-a-xc)/(Nstr+1.0)

        Do i=1,Nstr
          x0(i) = a+xc+Dx*i
          y0(i) = 0.0
        End Do

c--------------------------
      elseif(menu.eq.204) then
c--------------------------

        Dx = (a-xpv)/(Nstr+1.0)
        Do i = 1,Nstr
          x0(i) = xpv + Dx*i
          y0(i) = 0.
        End Do

c--------------------------
      elseif(menu.eq.205) then
c--------------------------

        Dy = (wall2-ypv)/(Nstr+1.0)

        Do i=1,Nstr
          x0(i) = xpv
          y0(i) = ypv + Dy*i
        End Do

c--------------------------
      Else If(menu.eq.206) then
c--------------------------

        Dy = (ypv-wall3)/(Nstr+1.0)
        Do i=1,Nstr
          x0(i) = xpv
          y0(i) = wall3 + Dy*i
        End Do

c--------------------------
      elseif(menu.eq.210) then
c--------------------------

        Dy = sr/(Nstr+1.0)
        Do i=1,Nstr
          x0(i) = xr
          y0(i) = Dy*i
        End Do

c--------------------------
      elseif(menu.eq.211) then
c--------------------------

        Dy = sr/(Nstr+1.0)
        Do i = 1,Nstr
          x0(i) = wall+0.01
          y0(i) = Dy*i
        End Do
        Dy = 0.5*sr/(Nstr+1.0)
        Do i = 1,Nstr
          x0(Nstr+i) = xr
          y0(Nstr+i) = 0.5*sr+Dy*i
        End Do
       Nstr = 2*Nstr

c--------------------------
      elseif(menu.eq.212) then
c--------------------------

        Dx = (xr-xmin)/Nstr

        Do i=1,Nstr
         x0(i) = xmin+Dx*(i-2)
         y0(i) = ymax
        End Do

        Dy = 2.0*(ymax-0.02)/Nstr

        Do i=1,Nstr/2
         k=Nstr+i
         x0(k) = xmin
         y0(k) = 0.01+Dy*(i-1)
        End Do
        Nstr = Nstr+Nstr/2

        Dy = (ymax-sr)/8

        Do i=1,Nstr/2
         k=Nstr+i
         x0(k) = xr
         y0(k) = sr+Dy*i
        End Do
        Nstr = Nstr+Nstr/2

c--------------------------
      elseif(menu.eq.213) then
c--------------------------

        Dy = sr/(Nstr+1.0)

        Do i=1,Nstr
          x0(i) = xr
          y0(i) = Dy*i
        End Do
           
c---------
      else                    ! Read starting points
c---------

        Read (2,*) 
        Nstr = 0

        Do i=1,200
          read (2,*) x0(i),y0(i)
          If(x0(i).gt.998) Go to 2
          Nstr = Nstr + 1
        End Do

 2      Continue

c-----------
      End If
c-----------

      close (2)

      if(Nstr.eq.0) then
        write (6,*) 
        write (6,*) " strml: could not find data for streamlines"
        write (6,*) "         Please check the main program"
        stop
      else if(Nstr.eq.1) then
c       write (6,*) 
c       write (6,*) " Will plot ",Nstr," streamline"
c       write (6,*) 
c       write (6,*) " Starting point:"
c       write (6,*) 
      else
c       write (6,*) 
c       write (6,*) " Will plot ",Nstr," streamlines"
c       write (6,*) 
c       write (6,*) " Starting points:"
c       write (6,*) 
      end if

c     Do i=1,Nstr
c       write (6,100) i,x0(i),y0(i)
c     End Do

c=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

c-------------------------------
c  READY TO PLOT FLOW BOUNDARIES
c-------------------------------

      if(  menu.eq.102
     +  ) then
       write (1,100) Ntwo
       write (1,100) None,0.5*RL,zero
       write (1,100) Ntwo,0.5*RL,ymax
       write (1,100) Ntwo
       write (1,100) None,zero,zero
       write (1,100) Ntwo,0.5*RL,zero
      end if

c---
c DRAW HORIZONTAL LINES ALONG THE x-AXIS
c---

      if(  menu.eq.103
     +   .or.menu.eq.107
     +   .or.menu.eq.121
     +   .or.menu.eq.171
     +   .or.menu.eq.371
     +   .or.menu.eq.372
     +   .or.menu.eq.373
     +  ) then
        write (1,100) Ntwo
        write (1,100) None,xmin,wall
        write (1,100) Ntwo,xmax,wall
      end if

      if(  menu.eq.104
     + .or.menu.eq.205) then
        write (1,100) Ntwo
        write (1,100) None,xmin,wall1
        write (1,100) Ntwo,xmax,wall1
        write (1,100) Ntwo
        write (1,100) None,xmin,wall2
        write (1,100) Ntwo,xmax,wall2
        if(menu.eq.104) then
         write (1,100) Ntwo
         write (1,100) None,zero,wall1
         write (1,100) None,zero,wall2
        end if
      end if

      if(  menu.eq.151
     +  ) then
         write (1,100) Ntwo
         write (1,100) None,a,zero
         write (1,100) Ntwo,xmax,zero
      end if

      if(  menu.eq.152
     +  ) then
         write (1,100) Ntwo
         write (1,100) None,b,zero
         write (1,100) Ntwo,xmax,zero
      end if

      if(  menu.eq.153
     +  ) then
         write (1,100) Ntwo
         write (1,100) None,a,zero
         write (1,100) Ntwo,xmax,zero
      end if

      if(  menu.eq.154
     +  ) then
         write (1,100) Ntwo
         write (1,100) None,b,zero
         write (1,100) Ntwo,xmax,zero
      end if

      if(  menu.eq.105
     + .or.menu.eq.206) then
        write (1,100) nfou
        write (1,100) none,wall1,ymax
        write (1,100) ntwo,wall1,wall3
        write (1,100) nthr,wall2,wall3
        write (1,100) nfou,wall2,ymax
      end if

      if(  menu.eq.406
     +    .and.abs(beta-0.5*pi)<0.00001
     +  ) then
         write (1,100) Ntwo
         write (1,100) None,zero,zero
         write (1,100) Ntwo,zero,ymax
      end if

      if(   menu.eq.202
     +  .or.menu.eq.209
     +  .or.menu.eq.210
     +  .or.menu.eq.220
     +  .or.menu.eq.301
     +  .or.menu.eq.303
     +  .or.menu.eq.308
     +  .or.menu.eq.309
     +  .or.menu.eq.310
     +  .or.menu.eq.312
     +  .or.menu.eq.315
     +  .or.(menu.eq.316.and.Iflow.eq.1)
     +  .or.(menu.eq.316.and.Iflow.eq.3)
     +  .or.menu.eq.330
     +  .or.menu.eq.360
     +  .or.menu.eq.318
     +  .or.menu.eq.319
     +  .or.menu.eq.362
     +  .or.menu.eq.401
     +  .or.menu.eq.404
     +  .or.menu.eq.406
     +  ) then
        write (1,100) Ntwo
        write (1,100) None,xmin,zero
        write (1,100) Ntwo,xmax,zero
      end if

      if(   menu.eq.138
     +  .or.menu.eq.139
     +  .or.menu.eq.307
     +  ) then
        write (1,100) Ntwo
        write (1,100) None,a,zero
        write (1,100) Ntwo,xmax,zero
      end if

      if(  menu.eq.403
     +   .or.(menu.eq.130).and.abs(circ).lt.0.00001
     +   .or.(menu.eq.131).and.abs(circ).lt.0.00001
     +   .or.menu.eq.140
     +  ) then
        write (1,100) Ntwo
        write (1,100) None,xmin,zero
        write (1,100) Ntwo,-a,zero
        write (1,100) Ntwo
        write (1,100) None,a,zero
        write (1,100) Ntwo,xmax,zero
      end if

      if(  menu.eq.213
     +  ) then
         write (1,100) Ntwo
         write (1,100) None,-a,zero
         write (1,100) Ntwo,a,zeto
      end if

      If(menu.eq.317) then
        write (1,100) Ntwo
        write (1,100) None,xmin,-h
        write (1,100) Ntwo,xmax,-h
        write (1,100) Ntwo
        write (1,100) None,xmin,h
        write (1,100) Ntwo,xmax,h
      End If

c---
c DRAW a semi-infinite line along the x-axis
c---

      if(menu.eq.135) then
        write (1,100) Ntwo
        write (1,100) None,xmin,one
        write (1,100) Ntwo,zero,one
      end If

      if(menu.eq.211) then
        write (1,100) Ntwo
        write (1,100) None,wall,zero
        write (1,100) Ntwo,wall,ymax
      end if

c---
c DRAW TWO PARALLEL LINES ALONG THE X AXIS
c---

      if(menu.eq.205) then
        write (1,100) Ntwo
        write (1,100) None,xmin,wall1
        write (1,100) Ntwo,xmax,wall1
        write (1,100) Ntwo
        write (1,100) None,xmin,wall2
        write (1,100) Ntwo,xmax,wall2
      end if

      if(menu.eq.363) then
        write (1,100) Ntwo
        write (1,100) None,xmin,-one
        write (1,100) Ntwo,xmax,-one
        write (1,100) Ntwo
        write (1,100) None,xmin,one
        write (1,100) Ntwo,xmax,one
        write (1,100) Ntwo
        write (1,100) None,xmin,zero
        write (1,100) Ntwo,xmax,zero
      end if

c--------------------------------------
c DRAW A VERTICAL LINE ALONG THE y-AXIS
c--------------------------------------

      if(    menu.eq.314
     +   .or.menu.eq.330
     +   .or.menu.eq.361
     +   .or.menu.eq.370
     +  ) then
        write (1,100) Ntwo
        write (1,100) None,zero,ymin
        write (1,100) Ntwo,zero,ymax
      end if

      if(menu.eq.316.and.Iflow.eq.2) then
        write (1,100) Ntwo
        write (1,100) None,zero,ymin
        write (1,100) Ntwo,zero,ymax
        write (1,100) Ntwo
        write (1,100) None,0.5*RL,ymin
        write (1,100) Ntwo,0.5*RL,ymax
      end if

      if(menu.eq.407) then
        write (1,100) Ntwo
        write (1,100) None,zero,ymin
        write (1,100) Ntwo,zero,ymax
      end if

      if(menu.eq.122) then
        Write (1,100) Ntwo
        Write (1,100) None,wall,zero
        Write (1,100) Ntwo,wall,ymax
        Write (1,100) Ntwo
        Write (1,100) None,wall,zero
        Write (1,100) Ntwo,xmax,zero
      end if

c---
c  DRAW A CIRCLE centered at (xc,yc)
c---

      if(menu.eq.106.or.menu.eq.172
     +   .or.menu.eq.130
     +   .or.menu.eq.131
     +   .or.menu.eq.138
     +   .or.menu.eq.139
     +   .or.menu.eq.140
     +   .or.menu.eq.203
     +   .or.menu.eq.204
     +   .or.menu.eq.220
     +   .or.menu.eq.212
     +   .or.menu.eq.213
     +   .or.menu.eq.307
     +   .or.menu.eq.308
     +   .or.menu.eq.313
     +   .or.menu.eq.320
     +   .or.(menu.eq.340.and.Iflow.eq.2)
     +   .or.menu.eq.360
     +   .or.menu.eq.361
     +   .or.menu.eq.362
     +   .or.menu.eq.363
     +   .or.menu.eq.403
     +   ) then

      crad = a
      if(menu.eq.340) crad = b

      if(menu.eq.360.or.menu.eq.361
     +  .or.menu.eq.362
     +  .or.menu.eq.363) then 
       xc=xr
       yc=sr
       crad = 0.05*sr;
      end if

      dth = pi2 /(nc1-1.0)
      write (1,100) nc1

       Do j = 1,nc1
        th = (j-1.0)*dth
        If(menu.eq.104) th = th-pih
        xstr(j) = xc + crad*cos(th)
        ystr(j) = yc + crad*sin(th)
        write (1,100)  j,xstr(j),ystr(j)
       End Do

      End If

c---
c small circle
c---

      if(   menu.eq.313
     +  .or.menu.eq.317
     +  .or.menu.eq.319
     +   ) then

      dth = pi2 /(nc1-1.0)
      write (1,100) nc1

       Do j = 1,nc1
        th = (j-1.0)*dth
        If(menu.eq.104) th = th-pih
        xstr(j) = xs + 0.02*cos(th)
        ystr(j) = ys + 0.02*sin(th)
        write (1,100)  j,xstr(j),ystr(j)
       End Do

      end if
c---

c----------------
c DRAW AN ELLIPSE
c----------------

      if(
     +      menu.eq.133
     +  .or.menu.eq.151
     +  .or.menu.eq.152
     +  .or.menu.eq.153
     +  .or.menu.eq.154
     +  .or.menu.eq.142
     +  ) then

       axis1 = a
       axis2 = b

      if(menu.eq.152) axis1 = b 
      if(menu.eq.154) axis1 = b
      if(menu.eq.154) axis2 = a

      if(menu.eq.141.and.Iflow.eq.2) axis1 = b 

      if(menu.eq.142.and.Iflow.eq.2) then
         ap = b
         bp = a
      end if

      dth = pi2 /(nc1-1.0)
      write (1,100) nc1

      Do j=1,nc1
        th = (j-1.0)*dth
        xstr(j) = axis1 * cos(th)
        ystr(j) = axis2 * sin(th)
        write (1,100) j,xstr(j),ystr(j)
      end do

      end if

c----------------------------
c  draw the Joukowski airfoil
c----------------------------

      if(menu.eq.134) then

       rads = (xc+1.0)**2+yc*yc
       rad  = sqrt(rads)

       dth = pi2 /(nc1-1.0)
       write (1,100) nc1

       Do j=1,nc1
        th = (j-1.0)*dth
        x  = xc + rad*cos(th)
        y  = yc + rad*sin(th)
        r2 = x*x+y*y
        xx = x + x/r2
        yy = y - y/r2
        xstr(j) = xx
        ystr(j) = yy
        write (1,100) j,xstr(j),ystr(j)
       End Do

      end if

c-------------------
c  DRAW CORNER WALLS
c-------------------

      if(   menu.eq.129
     +  .or.menu.eq.302.or.menu.eq.304
     +  .or.menu.eq.306) then

       if(menu.eq.129) alpha = pi/RM

       r = 100.0
       a = r*dcos(alpha)
       b = r*dsin(alpha)

       if(  menu.eq.304
     +  .or.menu.eq.306) b = -b

       if(a.lt.xmin) then
        b = b*xmin/a
        a = xmin
       end if

       if(a.gt.xmax) then
         b = b*xmax/a
         a = xmax
       end if

       if(b.lt.ymin)  then
         a = a*ymin/b
         b = ymin
       end if

       if(b.gt.ymax) then
         a = a*ymax/b
         b = ymax
       end if

      i = 2

      write (1,100) i
      write (1,100) i,zero,zero
      write (1,100) i,xmax,zero
      write (1,100) i
      write (1,100) i,zero,zero
      write (1,100) i,a,b

      end if

c-------------------
c  DRAW CORNER WALLS
c-------------------

      if(menu.eq.137) then

      i = 4

      write (1,100) i
      write (1,100) i,zero,ymax
      write (1,100) i,zero,zero
      write (1,100) i,one,zero
      write (1,100) i,one,ymax

      end if

c-----------------------------
c  DRAW SYMMETRIC CORNER WALLS
c-----------------------------

      if(menu.eq.305) then

       r = 10.0
       a = r*cos(alpha)
       b = r*sin(alpha)

       If(a.lt.xmin) then
        b = b*xmin/a
        a = xmin
       End If

       If(a.gt.xmax) then
        b = b*xmax/a
        a = xmax
        Istop = 1
       End If

       If(b.lt.ymin) then
        a = a*ymin/b
        b = ymin
        Istop = 1
       End If

       If(b.gt.ymax) then
        a = a*ymax/b
        b = ymax
       End If
 
       i = 2

       write (1,100) i
       write (1,100) i,zero,zero
       write (1,100) i,a,b

      End If

c-------------------
c  DRAW A FLAT PLATE
c-------------------

      if(   menu.eq.132
     +  .or.(menu.eq.340.and.Iflow.eq.1)
     +  .or.menu.eq.405
     +   ) then

      thick  = 0.01
      omax   = one
      omin   = onem

      if(menu.eq.405) then
        omax = five
        omin = zero
      elseif(menu.eq.340) then
        omax =  a
        omin = -a
        thick = b
      end if

      thickn = - thick

      i = 5
      write (1,100) i
      i = 1
      write (1,100) i,omax,thick
      i = 2
      write (1,100) i,omin,thick
      i = 3
      write (1,100) i,omin,thickn
      i = 4
      write (1,100) i,omax,thickn
      i = 5
      write (1,100) i,omax,thick

      End If

c-----------------
c  DRAW AN ORIFICE
c-----------------

      if(    menu.eq.350
     +   .or.menu.eq.351
     +   .or.menu.eq.352
     +   .or.menu.eq.353
     +  ) then

      thick  = 0.001
      thick  = 0.000
      thickn = - thick

      i = 5
      write (1,100) i
      i = 1
      write (1,100) i,xmax,thick
      i = 2
      write (1,100) i,a_open,thick
      i = 3
      write (1,100) i,a_open,thickn
      i = 4
      write (1,100) i,xmax,thickn
      i = 5
      write (1,100) i,xmax,thick

      End If

c================================

c-------------------------
c GENERATE THE STREAMLINES
c-------------------------

      Do 700 l=1,Nstr       ! loop over streamlines

      xn = x0(l)
      yn = y0(l)

      xstr(1) = xn    ! new point
      ystr(1) = yn    ! new point

c----
c integrate by the second-order Runge-Kutta
c----

      Do i=2,Nmax         ! step in time

      call velocity (menu,xn,yn,u,v)

      Umag = sqrt(u*u+v*v)
      Dt   = Ds/Umag      ! adjust the time step 
                          ! for constant travel time
      x1 = xn + Dt*u
      y1 = yn + Dt*v

      call velocity (menu,x1,y1,u1,v1)

      xn = xn + 0.5*Dt*(u+u1)
      yn = yn + 0.5*Dt*(v+v1)

c---------------
c stopping checks
c---------------

      Istop = 0

c---
c crossed the x = 0 plane?
c interpolate to place the point on x=0
c and stop
c---

      if(Icrossx.eq.1) then

       crossx = xn*xstr(i-1)

       if(crossx.lt.0.0) then
        yn = ystr(i-1)+(yn-ystr(i-1))*(0.0-xstr(i-1))
     +                               /(xn-xstr(i-1))
        xn = 0.0
        Istop = 1
       end if

      End If

c---
c crossed the y=0 plane?
c interpolate to place the point on y=0
c and stop
c---

      if(Icrossy.eq.1) then

       crossy = yn*ystr(i-1)

       if(crossy.lt.0.0) then
        xn = xstr(i-1)+(xn-xstr(i-1))*(0.0-ystr(i-1))
     +                               /(yn-ystr(i-1))
        yn = 0.0
        Istop = 1
       end if

      End If

c---
c crossed r = rcross ?
c---

      if(Icrossr.eq.1) then
        rrr = sqrt((xn-xcross)**2+(yn-ycross)**2)
        if(rrr.lt.rcross) Istop = 1
      end if

c---
c crossed plotting boundaries? 
c interpolate to place the point on the boundaries
c and stop
c---

      if(xn.lt.xmin) then
        yn = ystr(i-1)+(yn-ystr(i-1))*(xmin-xstr(i-1))
     +                               /(xn-xstr(i-1))
        xn = xmin
        Istop = 1
      end if

      if(xn.gt.xmax) then
        yn = ystr(i-1)+(yn-ystr(i-1))*(xmax-xstr(i-1))
     +                                 /(xn-xstr(i-1))
        xn = xmax
        Istop = 1
      end if

      if(yn.lt.ymin) then
        xn = xstr(i-1)+(xn-xstr(i-1))*(ymin-ystr(i-1))
     +                                 /(yn-ystr(i-1))
        yn = ymin
        Istop = 1
      end if

      If(yn.gt.ymax) then
        xn = xstr(i-1)+(xn-xstr(i-1))*(ymax-ystr(i-1))
     +                                 /(yn-ystr(i-1))
        yn = ymax
        Istop = 1
      end if

c---------------------
c done with the checks
c---------------------

      xstr(i) = xn
      ystr(i) = yn

c---
c check for a closed streamline after 5 steps
c---

      if(i.gt.5) then
       Dist = sqrt((xn-xstr(1))**2+(yn-ystr(1))**2)
       if(Dist.lt.Ds) Istop = 1
      end if

      if(Istop.eq.1) Go to 702

c---------------
      End Do
c---------------

      i = i-1

  702 Continue

c     if(Istop.eq.0) write (6,*) "More than ",Nmax," steps"

c=====================
c print the streamline
c=====================
 
      !---
      if(i.gt.2) then
      !---

       write (1,100) i

       !---
       Do j=1,i
       !---

         if(menu.eq.134) then                 ! conformal mapping
           r2 = xstr(j)**2+ystr(j)**2
           xx = xstr(j) + xstr(j)/r2
           yy = ystr(j) - ystr(j)/r2
           write (1,100)  j,xx,yy

         elseif(menu.eq.135) then            ! conformal mapping
           tmp = exp(pi*xstr(j))
           xx = xstr(j)+tmp*cos(pi*ystr(j))+shift
           yy = ystr(j)+tmp*sin(pi*ystr(j))
           write (1,100)  j,xx,yy

         elseif(menu.eq.136) then             ! conformal mapping
           xt = xstr(j)
           yt = ystr(j)
           xx = xt
           yy = yt
           rr1  = sqrt((xt-2.0)**2+yt*yt)
           arg1 = acos((xt-2.0)/rr1)
           rr1  = sqrt(rr1)
           arg1 = 0.50*arg1
           rr2  = sqrt((xt+2.0)**2+yt*yt)
           arg2 = acos((xt+2.0)/rr2)
           rr2  = sqrt(rr2)
           arg2 = 0.50*arg2
           rr   = rr1*rr2
           arg  = arg1+arg2
           xx   = 0.5*(xx + rr*cos(arg))
           yy   = 0.5*(yy + rr*sin(arg))
           write (1,100)  j,xx,yy

         else
           write (1,100)  j,xstr(j),ystr(j)
         end if

       !---
       End Do
       !---

      !---
      end if
      !---

c     write (6,*) "streamline ",l," computed"

c-------------
 700  Continue
c-------------

      write (1,100) null
      close (1)

      write (6,*) "Done"

c     call strml_gr              ! graphics
c     Icount = Icount + 1          ! graphics

      Go to 97  ! Return to the main menu

c---
c Done
c---

  99  Continue 

      write (6,*)
      write (6,*) " Thank you for your business"
      write (6,*)

c--------

 100  Format (1x,i3,6(1x,f10.5))
 101  Format (1x,"Spatial step = ",f10.5," Max No of points :",I3)

c200  Format (4(1x,F12.6),"  xmin, xmax, ymin, ymax")
 200  Format (4(1x,F12.6))
c201  Format (2(1x,I4),"  Number of ticks")
 201  Format (2(1x,I4))
c202  Format (3(1x,I4),"  Indices Isymo, Isymx, Isymy")
 202  Format (3(1x,I4))
c203  Format (1x,I1,1x,F12.6,"  Iperx, perx")
 203  Format (1x,I1,1x,F12.6)
c204  Format (1x,I1,1x,F12.6,"  Ipery, pery")
 204  Format (1x,I1,1x,F12.6)

      Stop
      End
