      program tube_ann_sw_dr

c========================================
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement
c========================================

c========================================
c Steady swirling flow in an annular tube
c confined between two concentric cylinders
c========================================

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      pi = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Null = 0
      None = 1
      Ntwo = 2

      zero = 0.0

c---
c input
c---

      open  (1,file="tube_ann_sw.dat",status="unknown")

       read (1,*) nprof
       read (1,*) a1
       read (1,*) a2
       read (1,*) visc
       read (1,*) Omega1
       read (1,*) Omega2

      close (1)

c---
c prepare
c---

      open (2,file="tube_ann_sw.out",status="unknown")

c---
c Draw the cylinders
c---

      Kpts = 64

      Dth = pi2/(Kpts+1.0D0-1.0D0)

      write (2,900) Kpts+1

      Do i=1,Kpts+1
       thet = (i-1.0)*Dth
       write (2,900) i,a1*cos(thet),a1*sin(thet)
      End Do

      write (2,900) Kpts+1

      Do i=1,Kpts+1
       thet = (i-1.0)*Dth
       write (2,900) i,a2*cos(thet),a2*sin(thet)
      End Do

c---
c Draw a vertical line
c---

      write (2,900) Ntwo
      write (2,900) None,zero,a1
      write (2,900) Ntwo,zero,a2

c---
c profile
c---

      write (6,*)
      write (6,*)  " velocity profile:"
      write (6,*)

      write (2,900) nprof+3

      write (2,900) i,zero,a1
      write (6,900) i,zero,a1

      step = (a2-a1)/(nprof+1.0D0-1.0D0)

      sigma = a1

      Do i=1,nprof+1

      call tube_ann_sw
     +
     +   (a1
     +   ,a2
     +   ,visc
     +   ,Omega1
     +   ,Omega2
     +   ,sigma
     +   ,Q
     +   ,torque
     +   ,u
     +   )

       write (2,900) i,u,sigma
       write (6,900) i,u,sigma

       sigma = sigma + step

      End Do

      write (2,900) i+1,zero,a2
      write (6,900) i+1,zero,a2

      write (2,900) null
      close (2)

c---
c Done with a profile
c---

      write (6,*)
      write (6,901) Q,torque
      write (6,*)

 900  Format (1x,i3,4(1x,f10.5))

 901  Format (
     +       "Flow rate = ",f15.10
     +    ,/,"Torque    = ",f15.10
     +       )

      Stop
      End
