function u = tube_crc_trans ...
 ...
  (a ...
  ,visc ...
  ,den ...
  ,pg ...
  ,time ...
  ,s ...
  )

%---------------------------------
% Transient flow in a circular tube
% due to the sudden application of 
% a constant pressure gradient
%
% SYMBOLS:
% -------
%
% a:    tube radius
% den:  fluid density
% visc: fluid viscosity
% s:    distance from axis
%
% aj0:  zeros of Bessel funciton J0
%---------------------------------

%--------------------------------
% zeros of the Bessel function J0
% From Abramowitz & Stegun, p.409
%--------------------------------

aj0(1) =  2.4048255577;
aj0(2) =  5.5200781103;
aj0(3) =  8.6537279129;
aj0(4) = 11.7915344391;
aj0(5) = 14.9309177086;
aj0(6) = 18.0710639679;
aj0(7) = 21.2116339679;
aj0(8) = 24.3524715308;
aj0(9) = 27.4934791320;
aj0(10)= 30.6346064684;

%--------
% prepare
%--------

  rnu = visc/den;         % kinematic viscosity
  as  = a*a;
  sr  = s/a;           % reduced sig
  tr  = time*rnu/as;    % reduced time

%---------------------
% compute the velocity
%---------------------

  u = 0.0;

  for j=1:5
    tmp0 =   aj0(j);
    tmp1 =   aj0(j)*sr;
    tmp2 = - aj0(j)^2*tr;
    u = u + bessel(0,tmp1)/bessel(1,tmp0)*exp(tmp2)/tmp0^3;
  end

  u = 0.25*pg*(as - s^2 - 8.0*as*u)/visc;

%---
% done
%---

return
