      subroutine tube_crc_wom
     + 
     +  (a
     +  ,visc
     +  ,den
     +  ,pgamp
     +  ,Omega
     +  ,ot
     +  ,s
     +  ,u
     +  )

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c=========================================

c=========================================
c Unsteady unidirectional flow in a circular tube 
c due to an oscillating pressure gradient.
c
c The pressure gradient is:
c
c dP/dx = - pgamp * sin(Omega*time)
c
c SYMBOLS:
c -------
c
c a:     tube radius
c Omega: angular frequency
c ot:    Omega*time
c visc:  fluid viscosity
c den:   fluid density
c s:     sigma, distance from centerline
c u:     axial velocity
c=========================================

      Implicit Double Precision (a-h,o-z)

c--------
c prepare
c--------

      cs = Dcos(ot)
      sn = Dsin(ot)

      rnu   = visc/den               ! kinematic viscosity
      delta = Dsqrt(2.0D0*rnu/Omega)  ! boundary layer thickness

      ar = a/delta    ! reduced radius
      sr = s/delta    ! reduced radial position

c---
c compute the velocity
c---

      Iopt = 1       ! do not need BERp and BEIp

      call ber_bei_0
     +
     +   (Iopt
     +   ,ar
     +   ,BER,BEI
     +   ,BERp,BEIp
     +   )

      denr = BER
      deni = BEI
      dens = denr*denr+deni*deni
      aar  = denr/dens
      aai  =-deni/dens

      call ber_bei_0
     +
     +   (Iopt
     +   ,sr
     +   ,BER,BEI
     +   ,BERp,BEIp
     +   )

      bbr = BER
      bbi = BEI

c----
c pick up the imaginary part
c---

      Ur = 1.0D0-aar*bbr+aai*bbi
      Ui =      -aar*bbi-aai*bbr

      u = -pgamp*(cs*Ur+Ui*sn)/(den*Omega)

c-----
c Done
c-----

      Return
      End
