function u = tube_crc_wom ...
  ...
  (a ...
  ,visc ...
  ,den ...
  ,pgamp ...
  ,omega ...
  ,ot ...
  ,s ...
  )

%-------------------------------------------
% Unsteady unidirectional flow in a circular tube 
% due to an oscillating pressure gradient.
%
% The pressure gradient is:
%
% dp/dx = - pgamp * sin(omega*time)
%
% SYMBOLS:
% -------
%
% a:     tube radius
% omega: angular frequency
% ot:    omega*time
% visc:  fluid viscosity
% den:   fluid density
% s:     sigma, distance from centerline
% u:     axial velocity
%
%-------------------------------------------

%--------
% prepare
%--------

  rnu  = visc/den;  %   kinematic viscosity

  delta = sqrt(2.0*rnu/omega);
  ar    = a/delta;
  sr    = s/delta;

  Iopt = 1;     % need neither BERp nor BEIp

%---
% compute the velocity
%---

 [BER,BEI,BERp,BEIp] ...
  ...
  = ber_bei_0 (Iopt,ar);

  denr =  BER;
  deni = -BEI;
  dens = denr*denr+deni*deni;
  aar  = denr/dens;
  aai  =-deni/dens;

 [BER,BEI,BERp,BEIp] ...
  ...
  = ber_bei_0 (Iopt,sr);

 bbr =  BER;
 bbi = -BEI;

%----
% pick up the real part
%---

 Ur = aar*bbr-aai*bbi-1.0;
 Ui = aar*bbi+aai*bbr;

 cs = cos(ot);
 sn = sin(ot);

 u = pgamp*(cs*Ur+Ui*sn)/(den*omega);

%-----
% done
%-----

return
