      program tube_ell_dr

c=========================================
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement
c=========================================

c=========================================
c Steady flow through an elliptical tube
c due to a pressure gradient or gravity
c
c SYMBOLS:
c --------
c
c beta:  tube inclination angle
c gac:   magnitude of the acceleration of gravity
c=========================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358 D0

c---
c input
c---

      open  (1,file="tube_ell.dat",status="unknown")
       read (1,*) nprof
       read (1,*) a       ! y semi-axis
       read (1,*) b       ! z semi-axis
       read (1,*) visc
       read (1,*) den
       read (1,*) gac
       read (1,*) chi
       read (1,*) beta
      close (1)

c---
c prepare to draw a profile
c---

      open (2,file="tube_ell.out",status="unknown")

c---

  98  Continue

      write (6,*)
      write (6,*) "Please enter the polar angle of the profile"
      write (6,*) "in multiples of pi"
      write (6,*)
      write (6,*) "99 to stop"
      write (6,*) "----------"
      write (6,*)
      read  (5,*) phi

      if(phi.eq.99) Go to 99

c---
c prepare
c---

      beta = beta*pi
      phi = phi*pi

      csp = cos(phi)
      snp = sin(phi)

      ytube = a*csp
      ztube = b*snp
      ystep = ytube/(nprof+1.0D0-1.0D0)
      zstep = ztube/(nprof+1.0D0-1.0D0)

c---
c profile
c---

      write (2,900) nprof+1,phi/pi

      write (6,*) 
      write (6,900) nprof+1,phi/pi
      write (6,*) 
      write (6,*) "u, sigma"
      write (6,*) 

      Do i=1,nprof+1

       y = (i-1.0)*ystep
       z = (i-1.0)*zstep

       call tube_ell
     +
     +     (a
     +     ,b
     +     ,visc
     +     ,den
     +     ,gac
     +     ,chi
     +     ,beta
     +     ,y,z
     +     ,flow_rate
     +     ,umean
     +     ,umax
     +     ,u
     +     )
  
       sigma = Dsqrt(y**2+z**2)

       write (2,900) i,u,sigma
       write (6,900) i,u,sigma

      End Do

      Go to 98

c-----
c Done
c-----

  99  Continue

      write (6,*)
      write (6,901) flow_rate,umean,umax
      write (6,*)

      write (2,900) null
      close (2)

 900  Format (1x,i3,4(1x,f10.5))
 901  Format ("Flow rate     = ",f15.10
     +     ,/,"Mean velocity = ",f15.10
     +     ,/,"Max  velocity = ",f15.10
     +       )

      Stop
      End
