      program tube_rec_dr

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c==========================================
c Steady flow through a tube with rectangular
c cross-section and side-lengths 2a and 2b
c
c Flow is due to pressure gradient or gravity
c
c Origin is set at the tube center
c
c SYMBOLS:
c -------
c
c beta:  tube inclination angle
c gac:   magnitude of the acceleration of gravity
c maxsm: truncation limit for infinite sum
c==========================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358 D0

c------
c input
c------

      open  (1,file="tube_rec.dat",status="unknown")
       read (1,*) nprof
       read (1,*) a       ! y semi-side
       read (1,*) b       ! z semi-side
       read (1,*) visc
       read (1,*) den
       read (1,*) gac
       read (1,*) chi
       read (1,*) beta
       read (1,*) maxsm    !  truncation limit
      close (1)

c---
c prepare 
c---

      open (2,file="tube_rec.out",status="unknown")

      beta = pi*beta

      nprof1 = nprof+1

      area   = 4.0*a*b      ! cross-sectional area

c---
c prepare to draw a profile
c---

  98  Continue

      write (6,*)
      write (6,*) "Please enter the y coordinate for a y-profile"
      write (6,*) "in (",-a,", ",a,")"
      write (6,*)
      write (6,*) "Enter 99 to quit"
      write (6,*) "-----------------"
      write (6,*)
      read  (5,*) y

      if(y.eq.99) then 
        Go to 99
      elseif(y.lt.-a.or.y.ge.1) then
        write (6,*) " Sorry; limits exceeded"
        Go to 98
      end if
      
c---
c profile
c---

      zstep  = b/(nprof1-1.0)

      write (2,900) nprof1,y

      write (6,*) 
      write (6,*) "u, z"
      write (6,*) 

      Do i=1,nprof1

       z = (i-1.0)*zstep
c      write (6,*) z

       call tube_rec
     +
     +   (a,b
     +   ,visc
     +   ,den
     +   ,gac
     +   ,chi
     +   ,beta
     +   ,maxsm
     +   ,y,z
     +   ,flow_rate
     +   ,u_mean
     +   ,u_max
     +   ,u
     +   )

       write (2,900) i,u,z
       write (6,900) i,u,z

      End Do

      Go to 98

c--------
c display
c--------

  99  Continue

      write (6,*)
      write (6,901) flow_rate,u_mean,u_max
      write (6,*)

c-----
c Done
c-----

      write (2,900) null
      close (2)

 900  Format (1x,i3,4(1x,f10.5))
 901  Format ("Flow rate     = ",f15.10
     +     ,/,"Mean velocity = ",f15.10
     +     ,/,"Max  velocity = ",f15.10
     +       )

      Stop
      End
