      subroutine tube_trgl_eql
     +
     +  (a
     +  ,visc
     +  ,den
     +  ,gac
     +  ,chi
     +  ,beta
     +  ,y,z
     +  ,flow_rate
     +  ,umean
     +  ,umax
     +  ,u
     +  )

c========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c========================================

c-----------------------------------------------
c Steady flow through a tube whose cross-section
c is an equilateral triangle with side-length 
c equal to "a"
c
c Flow is driven by a pressure gradient or gravity
c
c Origin is set at vertex 1, with the y axis
c passing throught vertices 1 and 2
c
c SYMBOLS:
c -------
c
c beta:  tube inclination angle
c gac:   magnitude of the acceleration of gravity
c-----------------------------------------------

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------

      pi = 3.14159 265358

      three = 3.0
      srt  = sqrt(three)

c--------
c prepare 
c--------

      as    = a**2
      area  = srt*as/4.0   ! triangle cross-sectional area

c---
c modifed pressure gradient
c---

      pg_mod = chi + den*gac*sin(beta)

c---
c compute the velocity
c---

      u = pg_mod*srt/(6.0*visc*a)
     +          *z
     +          *(z-srt*(a-y))
     +          *(z-srt*y)

      ysc = y-0.5*a            ! shift to triangle centroid
      zsc = z-0.5*a/srt        ! shift to triangle centroid

      u = pg_mod/(36.0*visc*a)
     +          *(2.0*srt*zsc+a)
     +          *(srt*zsc+3.0*ysc-a)
     +          *(srt*zsc-3.0*ysc-a)


c     write (6,*) u,u1
c     pause
  
c----------------------------------
c Flow rate, mean, and max velocity
c----------------------------------

      flow_rate = srt*pg_mod*as**2/(320.0*visc)
      umean     = flow_rate/area
      umax      = pg_mod*as/(36.0*visc)

c-----
c Done
c-----

      Return
      End
