      program tube_trgl_eql_dr

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c=========================================
c Steady flow through a tube whose cross-section
c is an equilateral triangle with side length 
c equal to "a",
c due to a pressure gradient or gravity
c
c SYMBOLS:
c --------
c
c gac:    magnitude of the acceleration of gravity
c a:      tube side length
c visc:   fluid viscosity
c chi:    negative of the pressure gradient
c beta:   tube inclination angle
c nprof:  number of points on a profile
c=========================================

      Implicit Double Precision (a-h,o-z)

c----------
c constants
c----------
 
      pi = 3.14159 265358D0

      three = 3.0
      srth  = Dsqrt(three)

c---
c input
c---

      open  (1,file="tube_trgl_eql.dat",status="unknown")

       read (1,*) nprof
       read (1,*) a       ! tube side-length
       read (1,*) visc
       read (1,*) den
       read (1,*) gac
       read (1,*) chi
       read (1,*) beta

      close (1)

c---
c prepare
c---

      beta = beta*pi

      open (2,file="tube_trgl_eql.out",status="unknown")

c---
c prepare to draw a profile
c---

  98  Continue

      write (6,*)
      write (6,*) "Enter the polar angle of the profile"
      write (6,*) "emanating from the lower-left vertex"
      write (6,*) "in multiples of pi/3"
      write (6,*)
      write (6,*) "It must lie between 0 and 1"
      write (6,*)
      write (6,*) "Enter 99 to quit"
      write (6,*) "----------------"
      write (6,*)
      read  (5,*) psi

      if(psi.eq.99) Go to 99
       
      if(psi.gt.1.0) then
        write (6,*) " Sorry, psi must be less then 1.0"
        Go to 98
      elseif(psi.lt.0) then
        write (6,*) " Sorry, psi must be higher than  0.0"
        Go to 98
      end if

c--------
c prepare
c--------

      psi = psi*pi/3.0
      csp = cos(psi)
      snp = sin(psi)

      rtube = a/(csp+snp/srth)    ! radial position of the third side
      rstep = rtube/(nprof+1.0D0-1.0D0)

c---
c profile
c---

      write (2,900) nprof+1,psi/(pi/3.0)

      write (6,*) 
      write (6,900) nprof+1,psi/(pi/3.0)
      write (6,*) 
      write (6,*) "u, radial position"
      write (6,*) 

      Do i=1,nprof+1

       r = (i-1.0)*rstep
       y = r*csp
       z = r*snp

       call tube_trgl_eql
     + 
     +    (a
     +    ,visc
     +    ,den
     +    ,gac
     +    ,chi
     +    ,beta
     +    ,y,z
     +    ,flow_rate
     +    ,umean
     +    ,umax
     +    ,u
     +    )

       write (6,900) i,u,r
       write (2,900) i,u,r
                        
      End Do

      Go to 98

c---
c wrap up
c---

  99  Continue

      write (6,*)
      write (6,901) flow_rate,umean,umax
      write (6,*)

      write (2,900) null
      close (2)

c-----
c Done
c-----

 900  Format (1x,i3,4(1x,f10.5))
 901  Format ("Flow rate     = ",f15.10
     +     ,/,"Mean velocity = ",f15.10
     +     ,/,"Max  velocity = ",f15.10
     +       )

      Stop
      End
