      program joukowski

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c----------------------------------------

c------------------------------------------
c Generate Joukowski airfoils
c according to equation (7.10.2)
c of Pozrikidis (1997, p.372)
c
c lamda will be set to 1.0
c-------------------------------------

      Implicit Double Precision (a-h,o-z)

c---
c constants
c---

      pi = 3.14159 265358 D0
      null= 0

c---
c parameters
c---

      write (6,*) 
      write (6,*) "  Enter "
      write (6,*) 
      write (6,*) "  0 to quit "
      write (6,*) "  1 for a Joukowski airfoil"
      write (6,*) "  -------------------------"
      read  (5,*) menu

      If(menu.eq.0) Go to 99

      open (2,file="joukowski.out")

c----------------
      If(menu.eq.1) then
c----------------

  98  Continue

      write (6,*)
      write (6,*) "  Please enter the number of points"
      write (6,*) "  0 to quit"
      write (6,*) "  ---------"
      read  (5,*) n

      If(n.eq.0) Go to 99

      write (6,*)
      write (6,*) " Enter x and y coordinates of circle center"
      write (6,*) " 99 to quit"
      write (6,*) " ----------"
      read (5,*) xc,yc

      If(xc.eq.99.or.yc.eq.99) Go to 99

c---
c compute the airfoil
c---

      n1 = n+1
      rad  = Dsqrt((xc+1.0)*(xc+1.0)+yc*yc)
      thstart = -pi+acos((xc+1)/rad)

      write (2,900) n1,xc,yc
      write (6,900) n1,xc,yc

      Do i=1,n+1
        th = thstart+(i-1)*2*pi/n
        xi = xc+rad*Dcos(th)
        et = yc+rad*Dsin(th)
        r2 = xi*xi+et*et
         x = xi*(1.0D0+1.0D0/r2)
         y = et*(1.0D0-1.0D0/r2)
        write (2,900) i,x,y
        write (6,900) i,x,y
      End Do

      Go to 98

c------------
      End If
c------------

  99  Continue

      write (2,900) null
      close (2)

c-----
c done
c-----

 900  Format (1x,i3,4(1x,f10.5))

      Stop
      End
