      subroutine velocity
     +
     +    (x0,y0
     +    ,uvel,vvel
     +    )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c---------------------------------------
c Compute the velocity at point: (x0, y0)
c using the vortex sheet representation
c---------------------------------------

      Implicit double precision (a-h,o-z)

      Dimension pt1(500,2),pt2(500,2)
      Dimension tnx(500),tny(500)
      Dimension gamma(500)

c---
c common blocks
c---

      common/intgr1/Npl
      common/points/pt1,pt2
      common/vortex/gamma
      common/pannels/tnx,tny
      common/incident/Ux,Uy

      common/piii/pi,pi2

c--------
c prepare
c--------

      uvel = 0.0D0
      vvel = 0.0D0

c------------
c over panels
c------------

      Do j=1,Npl     ! LOOP OVER PANELS - ENDS AT THE DOG

c---
c Compute local panel coordinates.
c The first panel point becomes the shifted origin;
c the panel becomes horizontal
c---

      xt = x0-pt1(j,1)               ! shift colocation point
      yt = y0-pt1(j,2)

      x =  xt*tnx(j) + yt*tny(j)    ! and rotate
      y = -xt*tny(j) + yt*tnx(j)

      x1 = 0.0D0
      y1 = 0.0D0

      x2t = pt2(j,1)-pt1(j,1)         ! shift second-end point
      y2t = pt2(j,2)-pt1(j,2)

      x2  = x2t*tnx(j) + y2t*tny(j)   ! and rotate
      y2  = 0.0

c----------------------------------
c Compute radial distances:  r1, r2
c         subtended angles: th1, th2
c----------------------------------

      r1 = Dsqrt( x**2    +y**2)
      r2 = Dsqrt((x-x2)**2+y**2)

      th1 = atan2(y,x)
      th2 = atan2(y,x-x2)

c--------------------------------------------
c  Compute influence coefficients 
c  associated with the velocity components (u, w)
c  corresponding to gamma1 and gamma, so that:
c
c  u = u1l*gamma1 + u2l*gamma2     (l stands for "linear")
c  w = w1l*gamma1 + w2l*gamma2
c
c  These velocities are in
c  the jth-panel reference frame.
c--------------------------------------------

      dth  = th2-th1
      rrt  = r2/r1
      rrtl = log(rrt)
      fcc  = 1.0/(pi2*x2)

      ax1 = fcc*(     y*rrtl + (x-x2)*dth)
      ay1 = fcc*((x-x2)*rrtl -      y*dth+x2)

      ax2 =-fcc*(y*rrtl + x*dth)
      ay2 =-fcc*(x*rrtl - y*dth+ x2)

c---
c Transform the local velocities into the
c global reference frame by rotation
c---

      ux1 = ax1*tnx(j) - ay1*tny(j)
      uy1 = ax1*tny(j) + ay1*tnx(j)

      ux2 = ax2*tnx(j) - ay2*tny(j)
      uy2 = ax2*tny(j) + ay2*tnx(j)
                                           
c---
c Panel induced velocity
c---

      uvel = uvel + ux1*gamma(j)+ux2*gamma(j+1)
      vvel = vvel + uy1*gamma(j)+uy2*gamma(j+1)

c-----------
      End Do                 ! End of loop over panels  -  DOG
c-----------

c-----------------------
c Add the incident flow
c----------------------

      uvel = uvel + Ux
      vvel = vvel + Uy

c-----
c Done
c-----

      Return
      End
