      program film0_dr

c----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-------------------------------------------
c Linear stability of a viscous film 
c resting on a horizontal wall or flowing down
c an inclined plane in the presence of gravity
c and surface tension
c
c Inertial effects are neglected
c and the fluid motion occurs under conditions
c of Stokes flow
c
c SYMBOLS:
c -------
c
c Method = 1: Determinant method
c        = 2: Yih's analytical solution
c------------------------------------------

      Implicit Double Precision (A-H,O-Z)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Null = 0

c------
c input
c------
      
      open (1,file="film0.dat")

        read (1,*) Method
        read (1,*) h         ! film thickness
        read (1,*) RL        ! wave length
        read (1,*)
        read (1,*) visc
        read (1,*) rho
        read (1,*) gamma
        read (1,*)
        read (1,*) gac        ! acceleration of gravity
        read (1,*) th0
        read (1,*)
        read (1,*) Nstp
        read (1,*) wnmax

      close (1)

c------------
c preferences
c------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 for one evaluation "
      write (6,*) " 2 for a graph with respect to the"
      write (6,*) "   wave number"
      write (6,*) "--------------"
      read  (5,*) menu

      If(menu.eq.0) Go to 99

c--------
c prepare
c--------

      th0 = th0*pi

c----------------------
      If(menu.eq.1) then
c----------------------

 97   Continue

      call film0
     +
     +  (Method
     +  ,h,RL
     +  ,visc
     +  ,rho
     +  ,gamma
     +  ,gac
     +  ,th0
     +  ,eta
     +  ,cr,ci
     +  ,si
     +  )

      write (6,101) cr,ci,si

      write (6,*)
      write (6,*) ' Please enter new value of L'
      write (6,*) ' 0 to quit '
      write (6,*) '----------------------------'
      read  (5,*) RL

      If(RL.eq.0) Go to 99
      Go to 97

c----------------------
      Else If(menu.eq.2) then
c----------------------

      open (9,file="film0.out")

      Nstp1 = Nstp+1
      wnmin = 0.0001
      step = (wnmax-wnmin)/Nstp

      wn = wnmin

      write (6,*) Nstp1

      Do i=1,Nstp1

       RL = pi2/wn

       call film0
     +
     +  (Method
     +  ,h
     +  ,RL
     +  ,visc
     +  ,rho
     +  ,gamma
     +  ,gac
     +  ,th0
     +  ,eta
     +  ,cr,ci
     +  ,si
     +  )

c---
c asymptotic
c---

      si_as = -2.0D0/3.0D0*eta*(h*wn)*(h*wn)

c     write (6,101) cr,ci,si
      write (6,100) i,wn,cr,ci,si,si_as
      write (9,100) i,wn,cr,ci,si,si_as

      wn = wn+step

      End Do

      write (9,*) Null

      close (9)

c-----------
      End If
c-----------

c-----
c Done
c-----

  99  Continue

 100  Format (1x,I3,10(1x,f12.8))
 101  Format (' cr          = ',f12.8
     +       ,' ci          = ',f12.8
     +       ,' Growth rate = ',f12.8
     +       )

      Stop
      End
