      program film_s

c-------------------------------------------
c linear stability analysis of a film flowing
c down an inclined plane wall in Stokes flow,
c in the presence of an insoluble surfactant
c
c LEGEND:
c ------
c
c ASR: coefficient matrix for the cubic
c      secular equation
c
c E:  surface elasticity
c
c Ma: Marangoni number 
c
c--------------------------------

      Implicit Double Precision (a-h,o-z)

      Double Precision Ma

      Dimension AR(4,4),AI(4,4)

      Dimension ASR(3,3),ASI(3,3)
      Dimension BSR(3),BSI(3)
      Dimension  XR(3), XI(3)

      Dimension AR3(3,3),AI3(3,3)
      Dimension BR3(3),BI3(3)
      Dimension XR3(3),XI3(3)

      common/pii/pi,pih,pi2,pis

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pih = 0.5D0*pi
      pi2 = 2.0D0*pi
      pis = pi*pi

      Null = 0

c------
c input
c------

c----------------------------------
c     write (6,*) " MENU"
c     write (6,*) 
c     write (6,*) " Enter:"
c     write (6,*)  
c     write (6,*) " 1 for a graph with respect to kh" 
c     write (6,*) " 2 for one value"
c     write (6,*) " -------------------------------"
c     read  (5,*) menu
c----------------------------------

c---------
c     Data
c---------

      open (1,file="film_s.dat")

       read (1,*) menu
       read (1,*)
       read (1,*) RL
       read (1,*) H
       read (1,*) th0
       read (1,*) visc
       read (1,*) dens
       read (1,*) G0    ! unperturbed Gamma
       read (1,*) T0    ! unperturbed surface tension
       read (1,*) Ds
       read (1,*) beta
       read (1,*) gac
       read (1,*)
       read (1,*) cc1R,cc1I     ! test values of c
       read (1,*) cc2R,cc2I
       read (1,*) cc3R,cc3I
       read (1,*)
       read (1,*) Isee
       read (1,*) Iverbose
       read (1,*)
       read (1,*) Inm
       read (1,*)
       read (1,*) Hmin    ! for menu = 1
       read (1,*) Hmax
       read (1,*) Nloop

c--------
c prepare
c--------

      th0  = th0*pi
      sth0 = Dsin(th0)
      tth0 = Dtan(th0)

      open (2,file="PLOTDAT")

c-----------
      If(menu.eq.1) then   ! a graph
c-----------

       step = (Hmax-Hmin)/Nloop
       Nloop1 = Nloop+1

       read (1,*)
       read (1,*) Idefine
       read (1,*)
       read (1,*) Ireduce
       read (1,*)

c-----------------------
      Else If(menu.eq.2) then   ! one value
c-----------------------

       Us = 0.50D0*dens*sth0*gac*H**2/visc
       Ca = visc*Us*RL**2/(T0*H**2)
       Pe = Us*RL/Ds
       Ma = beta/(1.0D0-beta)
       alpha = T0*RL/(visc*Ds)
       Nloop1 = 1

       write (6,*) "Us    = ",Us
       write (6,*) "Ca    = ",Ca
       write (6,*) "beta  = ",beta
       write (6,*) "Pe    = ",Pe
       write (6,*) "alpha = ",alpha

c-----------
      End If
c-----------

c-------
c repeat
c-------

   97 Continue

      H = Hmin

c------
c graph
c------

      If(menu.eq.1) then

c-------------------------------
c     write (6,*) " Please enter beta"
c     write (6,*) " -----------------"
c     read  (5,*) beta
c
c     write (6,*) " Please enter:"
c     write (6,*)
c     write (6,*) " 1 to define Ca and alpha wrt h (prime)"
c     write (6,*) " 2 to define Ca and alpha wrt L"
c     write (6,*) " --------------------------------------"
c     read  (5,*) Idefine
c
c     write (6,*) " Please enter:"
c     write (6,*)
c     write (6,*) " 1 to compute s      "
c     write (6,*) " 1 to compute s prime"
c     write (6,*) " --------------------"
c     read  (5,*) Ireduce
c
c     write (6,*) " Please enter Ca, beta, alpha"
c     write (6,*) " ----------------------------"
c     read  (5,*) Cain,beta,alin
c-------------------------------

      read (1,*) Cain,beta,alin
      If(Cain.eq.99) Go to 99

      write (2,100) Nloop1,Cain,beta,alin
      write (6,100) Nloop1,Cain,beta,alin

      End If

c---------------------------
c LOOP OVER FILM THICKNESSES
c---------------------------

      Do 31 Ic=1,Nloop1

c--------
c prepare
c--------

c-----------------------
      If(menu.eq.1) then    ! graph
c-----------------------

       If(Idefine.eq.1) then    ! Cain = Ca', alin = alpha'

         Cap    = Cain
         alphap = alin

         Ca     = Cap    * (RL/h)**2
         alpha  = alphap * (RL/h)

       Else If(Idefine.eq.2) then

         Ca    = Cain
         alpha = alin

         Cap    = Ca    * (h/RL)**2
         alphap = alpha * (h/RL)

       End If

       Pe = alpha * Ca * (h/RL)**2
       Ma = beta/(1.0D0-beta)

c-----------
      End If
c-----------

      w  = pi2*H/RL
      ws = w**2
      ev = Dexp(-w)
      v  = ev**2
      vc = 1.0D0-v

      tau = 1.0D0/tth0 + 2.0D0*pis/Ca

c-------------
c Yih solution
c-------------

      w2 = 2.0D0*w

      par = 1.0D0+Dcosh(w2)+2.0D0*ws

      CR_Yih = 1.0D0 + 2.0D0/par
      GR_Yih = - tau * (Dsinh(w2)-w2)/(w*par)

c----------
c impulse 1
c----------

      zR = cc1R ! Real zeta
      zI = cc1I ! Imag zeta

      call matrix
     +
     +   (w,ws,ev,v,vc
     +   ,Ma,tau,Ca,Pe
     +   ,zR,zI
     +   ,AR,AI
     +   ,DTR,DTI
     +   )

      Den = ZR**2+ZI**2
      BSR(1) = ( DTR*ZR+DTI*ZI)/Den
      BSI(1) = (-DTR*ZI+DTI*ZR)/Den

      ASR(1,1) = zR**2-zI**2
      ASI(1,1) = 2.0D0*zR*zI
      ASR(1,2) = zR
      ASI(1,2) = zI
      ASR(1,3) = 1.0D0
      ASI(1,3) = 0.0D0

c-------
c impulse 2
c-------

      zR = cc2R ! Real zeta
      zI = cc2I ! Imag zeta

      call matrix
     +
     +   (w,ws,ev,v,vc
     +   ,Ma,tau,Ca,Pe
     +   ,zR,zI
     +   ,AR,AI
     +   ,DTR,DTI
     +   )

      Den = ZR**2+ZI**2
      BSR(2) = ( DTR*ZR+DTI*ZI)/Den
      BSI(2) = (-DTR*ZI+DTI*ZR)/Den

      ASR(2,1) = zR**2-zI**2
      ASI(2,1) = 2.0D0*zR*zI
      ASR(2,2) = zR
      ASI(2,2) = zI
      ASR(2,3) = 1.0D0
      ASI(2,3) = 0.0D0

c-------
c impulse 3
c-------

      zR = cc3R  ! Real zeta
      zI = cc3I   ! Imag zeta

      call matrix
     +
     +   (w,ws,ev,v,vc
     +   ,Ma,tau,Ca,Pe
     +   ,zR,zI
     +   ,AR,AI
     +   ,DTR,DTI
     +   )

      Den = ZR**2+ZI**2
      BSR(3) = ( DTR*ZR+DTI*ZI)/Den
      BSI(3) = (-DTR*ZI+DTI*ZR)/Den

      ASR(3,1) = zR**2-zI**2
      ASI(3,1) = 2.0D0*zR*zI
      ASR(3,2) = zR
      ASI(3,2) = zI
      ASR(3,3) = 1.0D0
      ASI(3,3) = 0.0D0

c------------------------
c Solve the linear system
c------------------------

      call cramer_33c
     +
     +   (ASR,ASI
     +   ,BSR,BSI
     +   ,XR,XI
     +   )

c-------------------
c display the system
c-------------------

      If (Isee.eq.1) then

      write (6,101)
      Do i=1,3
       write (6,101) (ASR(i,j),j=1,3),BSR(i),XR(i)
      End Do
      write (6,101)
      Do i=1,3
       write (6,101) (ASI(i,j),j=1,3),BSI(i),XI(i)
      End Do

      End If

c-----------------------------
c solve the quadratic equation
c-----------------------------

      AqR = XR(1)
      AqI = XI(1)

      BqR = XR(2)
      BqI = XI(2)

      CqR = XR(3)
      CqI = XI(3)

      DR = BqR**2 - BqI**2 - 4.0D0*(AqR*CqR-AqI*CqI)
      DI = 2.0D0*BqR*BqI   - 4.0D0*(AqR*CqI+AqI*CqR)
      DM = Dsqrt(DR**2+DI**2)
      DA = Dacos(DR/DM)
      If(DI.lt.0.0D0) DA = -DA

      DMH = Dsqrt(DM)
      DAH = 0.50D0*DA

c-----------
c first root
c-----------

      R1R = 0.50D0*(- BqR - DMH * Dcos(DAH) )
      R1I = 0.50D0*(- BqI - DMH * Dsin(DAH) )
      Den = AqR**2+AqI**2
      tmR = R1R
      tmI = R1I
      R1R = ( tmR*AqR+tmI*AqI)/Den
      R1I = (-tmR*AqI+tmI*AqR)/Den

c------------
c second root
c-----------

      R2R = 0.50D0*(- BqR + DMH * Dcos(DAH) )
      R2I = 0.50D0*(- BqI + DMH * Dsin(DAH) )
      Den = AqR**2+AqI**2
      tmR = R2R
      tmI = R2I
      R2R = ( tmR*AqR+tmI*AqI)/Den
      R2I = (-tmR*AqI+tmI*AqR)/Den

c----------------
c print the roots
c of the binomial
c----------------

      If(Iverbose.eq.1) then
       write (6,114)  R1R,R1I,R2R,R2I
      End If
      
c-----------------
c verify the roots
c-----------------

      Ipass = 1

      zR = R1R
      zI = R1I

 66   Continue

c-----

      call matrix
     +
     +   (w,ws,ev,v,vc
     +   ,Ma,tau,Ca,Pe
     +   ,zR,zI
     +   ,AR,AI
     +   ,DTR,DTI
     +   )

c-----

      zRs  = zR**2-zI**2
      zIs  = 2.0D0*zR*zI

      pR =  AqR*zRs - AqI*zIs 
     +    + BqR*zR  - BqI*zI
     +    + CqR

      pI =  AqR*zIs + AqI*zRs 
     +    + BqR*zI  + BqI*zR
     +    + CqI

c---
c should be zero
c---

      If(Iverbose.eq.1) then
        write (6,115) DTR,DTI,pR,pI
      End If

c-----------------
c If Inm=1 
c compute the properties of the normal modes
c-----------------

      If(Inm.eq.0) Go to 81

c----------------------
c compute the eigenmode
c----------------------

      Do i=1,3
       Do j=1,3
        AR3(i,j) = AR(i,j)
        AI3(i,j) = AI(i,j)
       End Do
       BR3(i) = AR(i,4) 
       BI3(i) = AI(i,4) 
      End Do

      call cramer_33c
     +
     +   (AR3,AI3
     +   ,BR3,BI3
     +   ,XR3,XI3
     +   )

      a1R = XR3(1)
      a1I = XI3(1)

      a2R = XR3(2)
      a2I = XI3(2)

      a3R = -a1R
      a3I = -a1I

      a4R = XR3(3)
      a4I = XI3(3)

c----------
c arbitrary value for the complex
c constant D
c----------

      DR = -1.0D0
      DI =  0.0D0

c-----------------------------------
c compute the film surface amplitude
c-----------------------------------

      EX1 = Dexp(w)
      EX2 = Dexp(-w)

      F1R = a1R*EX1 + a2R*EX1 + a3R*EX2 + a4R*EX2
      F1I = a1I*EX1 + a2I*EX1 + a3I*EX2 + a4I*EX2
      Den = ZR**2+ZI**2
      RR = ( F1R*ZR+F1I*ZI)/Den
      RI = (-F1R*ZI+F1I*ZR)/Den

      SR = -RR
      SI = -RI
      SM = Dsqrt(SR**2+SI**2)
      SP = SR/SM
      SP = Dacos(SP)
      If(SI.lt.0.0) SP = -SP
      SP = -SP/pih    ! normalize

c---------------------------------
c compute the surfactant amplitude
c---------------------------------

      If(Ma.le.0.000001) then

       CM = 0.0D0
       CP = 0.0D0

      Else

       CR = - DR/Ma
       CI = - DI/Ma
       CM = Dsqrt(CR**2+CI**2)
       CP = CR/CM
       CP = Dacos(CP)
       If(CI.lt.0.0) CP = -CP
       CP = - CP/pih   ! normalize

      End If

      write (6,100) Ic,H/RL,SM,SP,CM,CP

  81  Continue

c-------
c repeat for the second mode
c-------

      If(Ipass.eq.1) then
       Ipass = 2
       zR = R2R
       zI = R2I
       Go to 66
      End If

c------------------
c Recover cr and ci
c reduced by U_s
c------------------

      c1R = 1.D0-R1R
      c1I =     -R1I

      c2R = 1.D0-R2R
      c2I =     -R2I

c-------------------------
c Compute the growth rates
c reduced by Us/H
c-------------------------

      GR1 = c1I*w
      GR2 = c2I*w

c-----
c swap
c-----

      If(GR2.lt.GR1) then
        tmp = GR1
        GR1 = GR2
        GR2 = tmp
        tmp = c1R
        c1R = c2R
        c2R = tmp
      End If

c------------------------
c Reduce phase velocities
c by the Yih values
c------------------------

      c1R = c1R/CR_Yih
      c2R = c2R/CR_Yih

c------------------------
c Reduce growth rates
c by the Yih values
c------------------------

      GR1Y = GR1/GR_Yih
      GR2Y = GR2/GR_Yih

c------------------------
c Reduce and asymptotics
c------------------------

      If(Ireduce.eq.1) then
        GR2_inf = - w*(w/alphap+Ma/2.0)/Cap
      Else If(Ireduce.eq.2) then
        GR1    = Cap*GR1
        GR2    = Cap*GR2
        GR_Yih = Cap*GR_Yih
        GR2_inf = - w*(w/alphap+Ma/2)
c       test = 2.0*(GR1 + w**2/alphap)/w
c       write (6,*) test,Ma
      End If

      write (6,102) Ic,w, GR1,GR2, GR1Y,GR2Y, GR_Yih, GR2_inf
      write (2,102) Ic,w, GR1,GR2, GR1Y,GR2Y, GR_Yih, GR2_inf

      Us = 0.50D0*dens*sth0*gac*H**2/visc

c     write (6,102) Ic,pi2*H/RL,c1R,GR1,c2R,GR2,CR_Yih,GR_Yih
c     write (2,102) Ic,pi2*H/RL,c1R,GR1,c2R,GR2,CR_Yih,GR_Yih


      H = H+step

c-------------
  31  Continue
c-------------

      If(Menu.eq.1) Go to 97

c-----
c Done
c-----

   99 Continue

      write (2,100) Null
      close (1)
      close (2)

  100 Format (1x,i4,100(1x,F12.6))
  101 Format (100(1x,F15.8))
  102 Format (1x,i4,1x,F8.5,1x,2(1x,F12.6)
     +                     ,1x,2(1x,F12.6)
     +                     ,1x,2(1x,F12.6))
  114 Format ("Roots:          ",10(1x,F12.8))
  115 Format ("Verified roots: ",6(1x,F9.8))

      Return
      End
