clear all
close all
%===

global NGL

%======================
% stability of Kirchhoff's
% elliptical patch
% a:  major axis
% b:  minor axis
%===========================

a=1.0;

b=0.25;
b=0.40;
b=0.80;
b=0.20;
b=0.90;
b=1.00;
b=0.50;

Omega = 1.0;

NGL = 12;
NGL = 6;

N = 128;
N = 4;
N = 8;
N = 16;
N = 32;
N = 64;

epsilon = 0.002;

%---
% shape
%---

deta = 2*pi/N;

for i=1:N+1
 eta(i) = (i-1.0)*deta+0.00*pi;
 rad1 = a;
 rad2 = b;
 x(i) = rad1*cos(eta(i));
 y(i) = rad2*sin(eta(i));
end

%---
% prepare
%---

Omega_z = 0;
Omega_z = Omega*a*b/(a+b)^2;

%---
figure(1)
hold on
plot(x,y,'ro-')
xlabel('x')
ylabel('y')
axis equal
axis([-1.2 1.2 -1.2 1.2])
%---

%---
% cubic spline interpolation
%---

%---
[vnx_el,vny_el ...
,crv,s ...
,Xint ...
,Axint,Bxint,Cxint ...
,Ayint,Byint,Cyint ...
,area,xcenter,ycenter ...
,aspect,angle1,angle2] = splc_geo (N,x,y);
%---

%---
% perform impulses
%
% ir=0 no impulse
%---

%---
  for ir=0:N
%---

  if(ir>0)
    x(ir) = x(ir)+epsilon*vnx_el(ir);
    y(ir) = y(ir)+epsilon*vny_el(ir);
  end
  x(N+1) = x(1);
  y(N+1) = y(1);

  [vnx,vny,crv,s ...
  ,Xint ...
  ,Axint,Bxint,Cxint ...
  ,Ayint,Byint,Cyint ...
  ,area,xcenter,ycenter ...
  ,aspect,angle1,angle2] = splc_geo (N,x,y);

%---
% contour integral
%---

%---
  for i=1:N
%---

    velx(i) = 0.0;
    vely(i) = 0.0;

    for j=1:N
    j1=j+1;

    Ising = 0;
    if(i==j) Ising = 1; end          % first endpoint
    if(i==j+1) Ising = 2; end        % second endpoint
    if(i==1 & j==N) Ising = 2; end   % second endpoint

    [SLPx, SLPy] = vp_slp_spline  ...
    ...
    (x(i),y(i) ...
    ,x(j),y(j)...
    ,x(j1),y(j1)...
    ,Xint(j),Xint(j1) ...
    ,Axint(j),Bxint(j),Cxint(j) ...
    ,Ayint(j),Byint(j),Cyint(j) ...
    ,Ising);

     velx(i) = velx(i)+SLPx;
     vely(i) = vely(i)+SLPy;
    end
  end

  velx = Omega*velx;
  vely = Omega*vely;

%---
% rotating frame
%---

  for i=1:N
   velx(i) = velx(i) + Omega_z*y(i);
   vely(i) = vely(i) - Omega_z*x(i);
  end

  velx(N+1) = velx(1);
  vely(N+1) = vely(1);

% [velx', vely']
% pause

%---
% normal velocity
%---

   for i=1:N
    un(i) = velx(i)*vnx_el(i)+vely(i)*vny_el(i);
   end

%---
% tangential velocity and Jacobian on the horizontal ellipse
%---

  if(ir==0)
   un_el = un;
    for i=1:N
     tnx = -vny(i);
     tny =  vnx(i);
     ut(i) = velx(i)*tnx+vely(i)*tny;
     jac(i) = Omega_z*sqrt(a^2/b^2*y(i)^2+b^2/a^2*x(i)^2);
    end
%   [un' ut' jac']
%   pause
  end

%---
% compose the matrix
%---

  if(ir>0)
    for i=1:N
     MAT(i,ir) = un(i)/epsilon;
     MAT(i,ir) = (un(i)-un_el(i))/epsilon;
    end

    %---
    % convective term
    %---

    impulse(1:N+1) = 0.0;
    delta = 0.345;
    impulse(ir)=delta;
    impulse(N+1)=impulse(1);
    [a_es,b_es,c_es] = splc_pr (N,eta,impulse);
%   ir
%   c_es'
%   pause
    for i=1:N
      MAT(i,ir) = MAT(i,ir) - Omega_z*c_es(i)/delta;
    end

    %---
    % reset
    %---

    x(ir) = x(ir)-epsilon*vnx_el(ir);
    y(ir) = y(ir)-epsilon*vny_el(ir);
    x(N+1) = x(1);
    y(N+1) = y(1);

  end

%---
end % of repeat
%---

%=========
% compute the eigenvalues
%=========

%---
% power method
%---

Ido = 0;

if(Ido==1)

 for i=1:N
   eigv(i)=rand;
 end
 eigv = eigv/norm(eigv);
 eigv = eigv'

 for i=1:500
  eigv = MAT*eigv;
  lambda = norm(eigv)
  eigv = eigv/lambda;
  pause
 end

 figure(10)
 plot(eigv,'o-')
 pause

end

%---
% matlab
%---

im = sqrt(-1);
cgm = im*eig(MAT)

for m=1:N
 love(m) = 0.5*Omega ...
  *sqrt( (2*m*Omega_z/Omega-1.0)^2 -((a-b)/(a+b))^(2*m) );
 rlove(m)=real(love(m));
 ilove(m)=imag(love(m));
end

for m=1:N
 rcgm(m)=real(cgm(m));
 icgm(m)=imag(cgm(m));
end

figure(20)
hold on
M=10;
plot(rlove(1:M),ilove(1:M),'o')
plot(rcgm(1:N),icgm(1:N),'r+')

%---
% matlab
%---

[V,D]=eig(MAT);

for i=1:N+1
 J = sqrt(a^2*sin(eta(i))^2+b^2*cos(eta(i))^2);
 eigf1(i) = cos(1.0*eta(i))/J^1;
 eigf2(i) = cos(2.0*eta(i))/J^1;
 eigf3(i) = cos(3.0*eta(i))/J^1;
 eigf4(i) = cos(4.0*eta(i))/J;
 eigf10(i) = sin(1.0*eta(i))/J^1;
 eigf20(i) = sin(2.0*eta(i))/J^1;
 eigf30(i) = sin(3.0*eta(i))/J^1;
 eigf40(i) = sin(4.0*eta(i))/J;
end
eigf1= eigf1/norm(eigf1,inf);
eigf2= eigf2/norm(eigf2,inf);
eigf3= eigf3/norm(eigf3,inf);
eigf4= eigf4/norm(eigf4,inf);
eigf10= eigf10/norm(eigf1,inf);
eigf20= eigf20/norm(eigf2,inf);
eigf30= eigf30/norm(eigf3,inf);
eigf40= eigf40/norm(eigf4,inf);

for k=N:-1:1
  V(N+1,k) = V(1,k);
  k
  cgm(k)
%if(imag(D(k,k))<0)
 figure(2)
 rreal = real(V(1:N+1,k));
 rreal = rreal/norm(rreal,inf);
 iimag = imag(V(1:N+1,k));
 iimag = iimag/norm(iimag,inf);
 plot(eta(1:N+1),rreal)
 hold on
 plot(eta(1:N+1),iimag,'r')
 plot(eta(1:N+1),eigf1(1:N+1),':')
 plot(eta(1:N+1),eigf2(1:N+1),':')
 plot(eta(1:N+1),eigf3(1:N+1),':')
 plot(eta(1:N+1),eigf4(1:N+1),':')
 plot(eta(1:N+1),eigf10(1:N+1),':')
 plot(eta(1:N+1),eigf20(1:N+1),':')
 plot(eta(1:N+1),eigf30(1:N+1),':')
 plot(eta(1:N+1),eigf40(1:N+1),':')
 hold off
 pause
%end
end
