function [SLPx, SLPy] = vp_slp_spline  ... 
...
  (X0,Y0 ...
  ,X1,Y1...
  ,X2,Y2...
  ,Xint1,Xint2 ...
  ,Axint,Bxint,Cxint ...
  ,Ayint,Byint,Cyint ...
  ,Ising)

%--------------------------------------
% Integrate the Green's function over
% a spline segment
% to compute the single-layer potential
%--------------------------------------

global NGL

%---
% prepare
%---

pi2 = 2.0*pi;
pi4 = 4.0*pi;

Iopt=1;  % only the GF needed

%---
% Gauss-Legendre quadrature
%---

[ZZ,WW] = gauss_leg(NGL);

%---
% singular element
%---

  if(Ising>0) 

   if(Ising==1)     % singular point at the first point
    DX = 0.0;
   elseif(Ising==2) % singular point at the last point
    DX = Xint2-Xint1;
   end

   Dxp0 = (3.0D0*Axint*DX + 2.0D0*Bxint )*DX + Cxint;
   Dyp0 = (3.0D0*Ayint*DX + 2.0D0*Byint )*DX + Cyint;
   h0 = sqrt(Dxp0*Dxp0+Dyp0*Dyp0);

  end

%-----------
% initialize
%-----------

 SLPx = 0.0;
 SLPy = 0.0;

%---
% prepare for the quadrature
%---

 XintM = 0.5D0*(Xint2+Xint1);
 XintD = 0.5D0*(Xint2-Xint1);

%--------------------------
% loop over Gaussian points
%--------------------------

for i=1:NGL

  Xint = XintM + XintD*ZZ(i);

  DX = Xint-Xint1;
  X = ((Axint*DX + Bxint )*DX + Cxint)*DX+X1;
  Y = ((Ayint*DX + Byint )*DX + Cyint)*DX+Y1;
  Dxp = (3.0*Axint*DX + 2.0*Bxint )*DX + Cxint;
  Dyp = (3.0*Ayint*DX + 2.0*Byint )*DX + Cyint;

  h = sqrt(Dxp*Dxp+Dyp*Dyp);

 [G,Gx,Gy] = lgf_2d_fs ...
...
  (X,Y ...
  ,X0,Y0 ...
  ,Iopt);

%---
% carry out the quadrature
%---

  WI = G*h*WW(i)*XintD;

  SLPx = SLPx + Dxp*WI;
  SLPy = SLPy + Dyp*WI;

%---
  if(Ising > 0)

    if(Ising == 1) DelX = Xint-Xint1; end
    if(Ising == 2) DelX = Xint2-Xint; end
    tmp = log(DelX)*h0*WW(i)*XintD/pi4;
    SLPx = SLPx + tmp*Dxp0;
    SLPy = SLPy + tmp*Dyp0;

  end
%---

%---
end % over segments
%---

  if(Ising > 0)
    DelX = Xint2-Xint1;
    tmp = h0*DelX*(log(DelX)-1.0)/pi4;
    SLPx = SLPx - tmp*Dxp0;
    SLPy = SLPy - tmp*Dyp0;
  end

%-----
% done
%-----

return
