      subroutine layer0
     +
     +  (b           ! layer thickness
     +  ,L           ! wave length
     +  ,vs1,vs2     ! viscosities
     +  ,rho1,rho2
     +  ,gamma
     +  ,gac
     +  ,method
     +  ,Phi
     +  ,si
     +  )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c--------------------------------------
c Instability of a liquid layer (1)
c resting on a plane wall
c underneath a semi-infinite fluid (2)
c under conditions of Stokes flow.
c
c SYMBOLS:
c -------
c
c b: layer thickness
c L: wave length
c
c method = 1 for analytical solution
c          2 for the matrix method
c---------------------------------------

      Implicit Double Precision (a-h,o-z)

      Double precision L

      Dimension a(6,6)

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Nsix = 6

c--------
c prepare
c--------

      Drho = rho1-rho2

      wn  = pi2/L      ! wave number
      bk  = wn*b       ! reduced wave number

      vsr = vs2/vs1    ! viscosity ratio

c--------------------
c Analytical solution
c--------------------

c-------------------------
c Method of Newhouse & Pozrikidis (1990)
c----
      If(method.eq.1) then
c-------------------------

      bks = bk*bk
      arithm = 0.50D0*Dsinh(2.0D0*bk)-bk+vsr*(Dsinh(bk)**2-bks)
      parono = (1.0D0-vsr**2)*bks +(Dcosh(bk)+vsr*Dsinh(bk) )**2

      W = arithm/parono

      Phi = (1.0D0+vsr)*W

c------------------------------
      Else If(method.eq.2) then
c------------------------------

c-------------------------------------------
c The determinant of the six-by-six matrix a(i,j)
c is a linear function of the reduced 
c growth rate group q that is placed in the 
c a(6,5) entry
c
c Secular equation is: F(q) = c*q + d
c
c First compute d, and then compute c
c
c-------------------------------------------

      p = Dexp(-2.0D0*bk)

c continuity of psi at the interface:

      a(1,1) = 1.0D0
      a(1,2) = 0.0D0
      a(1,3) = 1.0D0
      a(1,4) = 0.0D0
      a(1,5) =-1.0D0
      a(1,6) = 0.0D0

c continuity of d psi/d y at the interface

      a(2,1) = 1.0D0
      a(2,2) = 1.0D0
      a(2,3) =-1.0D0
      a(2,4) = 1.0D0
      a(2,5) = 1.0D0
      a(2,6) =-1.0D0

c d psi^1/d y = 0 at y=-b

      a(3,1) = p
      a(3,2) = p*(1.0D0-bk)
      a(3,3) =-1.0D0
      a(3,4) = 1.0D0+bk
      a(3,5) = 0.0D0
      a(3,6) = 0.0D0

c psi^1 = 0 at y=-b

      a(4,1) = p
      a(4,2) =-p*bk
      a(4,3) = 1.0D0
      a(4,4) =-bk
      a(4,5) = 0.0D0
      a(4,6) = 0.0D0

c continuity of shear stress

      a(5,1) = 1.0D0
      a(5,2) = 1.0D0
      a(5,3) = 1.0D0
      a(5,4) =-1.0D0
      a(5,5) =-vsr
      a(5,6) = vsr

c continuity of normal stress

      a(6,1) = 0.0D0
      a(6,2) = 1.0D0
      a(6,3) = 0.0D0
      a(6,4) = 1.0D0
      a(6,5) = 0.0D0   ! impulse
      a(6,6) =-1.0D0

c----------------
c  By definition,
c  cpd = c+d
c----------------

      call crout (Nsix,a,d)

      a(6,5) = 1.00D0

      call crout (Nsix,a,cpd)

      c = cpd-d

c--------------------------------
c compute the growth rate group q
c and the modulation function W
c--------------------------------

      q = -d/c

      W = 1.0D0/(q-1.0D0+vsr)

c--------------------------------
      End If    ! end of option 2
c--------------------------------

c------------------------
c recover the growth rate
c------------------------

      si = - 0.5D0*W*(Drho*gac/wn + wn*gamma)/vs1

c-----
c Done
c-----

      Return
      End
