      program layersf0_s_dr

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c----------------------------------------
c  Complex phase velocity of shear flow
c  over a layer resting on wall
c  in Stokes flow
c
c  SYMBOLS:
c  -------
c 
c  h:  layer thickness
c-----------------------------------------

      Implicit Double Precision (a-h,o-z)
      Double Precision Ma

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi2 = 2.0D0*pi

      Null = 0

c----------------
c read parameters
c----------------

      open (1,file="layersf0_s.dat")

        read (1,*) h
        read (1,*)
        read (1,*) RL
        read (1,*)
        read (1,*) vs1 
        read (1,*) vs2
        read (1,*)
        read (1,*) rho1 
        read (1,*) rho2
        read (1,*)
        read (1,*) gamma0
        read (1,*)
        read (1,*) Ma
        read (1,*) Ds
        read (1,*)
        read (1,*) gac
        read (1,*) 
        read (1,*) xi
        read (1,*)
        read (1,*) Nstp
        read (1,*) wnmax

      close (1)

c------------
c preferences
c------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 to quit"
      write (6,*) " 1 for one evaluation "
      write (6,*) " 2 for a graph with respect to the"
      write (6,*) "   wave number"
      write (6,*) " -------------"
      read  (5,*) menu

      If(menu.eq.0) Go to 99

c---
c prepare
c---

      rlam = vs2/vs1

c----------------------
      If(menu.eq.1) then
c----------------------

 97   Continue

       call layersf0_s
     +
     +  (RL
     +  ,h
     +  ,xi
     +  ,vs1,vs2
     +  ,rho1,rho2
     +  ,gamma0
     +  ,Ma
     +  ,Ds
     +  ,gac
     +  ,cr1,ci1
     +  ,sr1,si1
     +  ,cr2,ci2
     +  ,sr2,si2
     +  )

      write (6,101) cr1,sr1
      write (6,102) ci1,si1
      write (6,101) cr2,sr2
      write (6,102) ci2,si2

      write (6,*) 
      write (6,*) ' Current value of L is: ',RL
      write (6,*) ' Please enter a new value'
      write (6,*) ' 0 to quit'
      write (6,*) ' ----------'
      read  (5,*) RL

      If(RL.eq.0) Go to 99

      Go to 97

c----------------------
      Else If(menu.eq.2) then
c----------------------

      open (9,file="layersf0_s.out")

      Nstp1 = Nstp+1
      wnmin = 0.001D0
      step = (wnmax-wnmin)/Nstp

 98   Continue

      Do iloop=1,8
c     Do iloop=1,1
    
      if(iloop.eq.1) then
        Ma = 0.01
        gamma0 = 1/5.0
        vs2= 0.01
      elseif(iloop.eq.2) then
        Ma = 0.05
        gamma0 = 1/3.0
        vs2= 0.5
      elseif(iloop.eq.3) then
        Ma = 0.1
        gamma0 = 1/2.0
        vs2= 1.0
      elseif(iloop.eq.4) then
        Ma = 0.2
        gamma0 = 1/1.0
        vs2= 2.0
      elseif(iloop.eq.5) then
        Ma = 0.5
        gamma0 = 1/0.5
        vs2= 5.0
      elseif(iloop.eq.6) then
        Ma = 1.0
        gamma0 = 1/0.2
        vs2= 10.0
      elseif(iloop.eq.7) then
        Ma = 2.0
        gamma0 = 1/0.1
        vs2= 20.0
      elseif(iloop.eq.8) then
        Ma = 5.0
        gamma0 = 1/0.05
        vs2= 50.0
      end if

      Ma = 0.01
      Ma = 1.0
      Ma = 0.1
      gamma0 = 20

      write (9,100) Nstp1,Ma,vs1,vs2

      Do i=1,Nstp1

      wn = wnmin + (i-1.0D0)*step

       RL = pi2/wn

       call layersf0_s
     +
     +  (RL
     +  ,h
     +  ,xi
     +  ,vs1,vs2
     +  ,rho1,rho2
     +  ,gamma0
     +  ,Ma
     +  ,Ds
     +  ,gac
     +  ,cr1,ci1
     +  ,sr1,si1
     +  ,cr2,ci2
     +  ,sr2,si2
     +  )

      siinf1 = -0.5D0*((rho1-rho2)*gac/wn+gamma0*wn )/(vs1+vs2)
      siinf2 = -wn*(0.5D0*Ma*gamma0/(vs1+vs2) + wn*Ds)

      write (6,100) i,wn,cr1,si1,cr2,si2,siinf1,siinf2
      write (9,100) i,wn,cr1,si1,cr2,si2,siinf1,siinf2

      End Do

c     write (6,*) "Please enter another Ma number"
c     write (6,*) "negative to quit"
c     write (6,*) "----------------"
c     read (5,*) Ma
c     If(Ma>0) then
c       Go to 98
c     End If

      End Do

      write (9,*) Null,Null,Null,Null,Null
      close (9)

c-----------
      End If
c-----------

c-----
c Done
c-----

  99  Continue

 100  Format (1x,I3,10(1x,f10.6))
 101  Format ("cr = ",f16.10," sr = ",f16.10)
 102  Format ("ci = ",f16.10," si = ",f16.10)
 105  Format ("uI = ",f12.8,", xi1= ",f12.8,", xi2= ",f12.8)

      Stop
      End
