function [detr, deti] = rayleigh_sys (N,Dys,U,Upp,k,creal,cimag)

%---------------------------------
% Compile the (N-1)x(N-1) tridiagonal matrix E
% and compute the determinant
%
% matrix has the structure:
%
%       |a b
%       |  c a b
% E =   |    c a b
%       |      c a b
%
% ar is the real part of a
% ai is the imaginary part of a
%---------------------------------

%---
% generate the tridiagonal matrix
%---

      for i=2:N

         den   = (U(i)-creal)^2+cimag^2;
         ar(i-1) = -2.0-Dys*(k^2 + Upp(i)*(U(i)-creal)/den);
         br(i-1) = 1.0;
         cr(i-1) = 1.0;

         ai(i-1) = - Dys*Upp(i)*cimag/den;
         bi(i-1) = 0.0;
         ci(i-1) = 0.0;

      end

%---
% Compute the determinant of the 
% complex matrix 
% using the nifty method
%---

      Pr(1) = ar(1);
      Pi(1) = ai(1);
      tmpr  = br(1)*cr(2)-bi(1)*ci(2);
      tmpi  = br(1)*ci(2)+bi(1)*cr(2);
      Pr(2) = ar(2)*Pr(1)-ai(2)*Pi(1)-tmpr;
      Pi(2) = ar(2)*Pi(1)+ai(2)*Pr(1)-tmpi;

      for i=3:N-1
          ia    = i-1;
          tmpr  = br(ia)*cr(i)-bi(ia)*ci(i);
          tmpi  = br(ia)*ci(i)+bi(ia)*cr(i);
          ib    = i-2;
          auxr  = tmpr*Pr(ib)-tmpi*Pi(ib);
          auxi  = tmpr*Pi(ib)+tmpi*Pr(ib);
          Pr(i) = ar(i)*Pr(ia)-ai(i)*Pi(ia)-auxr;
          Pi(i) = ar(i)*Pi(ia)+ai(i)*Pr(ia)-auxi;
      end

  detr = Pr(N-1);
  deti = Pi(N-1);

% [creal,cimag,detr,deti]

%---
% compute the determinant using matlab (optional)
%---

Ido = 0;

if(Ido==1)
  M = zeros(N-1,N-1);
  imag = sqrt(-1);
  for i=1:N-1
    M(i,i) = ar(i)+imag*ai(i);
  end
  for i=1:N-2
    M(i,i+1) = br(i)+imag*bi(i);
    M(i+1,i) = cr(i)+imag*ci(i);
  end
 det(M)
end

%-----
% done
%-----

return
