function [U,Up,Upp] = rayleigh_vel (menu,U0,delta,alpha,y)

%-------------------------------
% Evaluation of the velocity (U)
% first derivative (Up)
% and second derivative (Upp)
% at a point y
%
% menu: choice of velocity profile
% U0:   coefficient of the velocity profile
% delta: length scale of the velocity profile
% alpha: velocity profile parameter
%---------------------------

%---------
% prepare
%---------

  eta  = y/delta;

%---------
% evaluate
%---------

%---
  if(menu==1) % derivatives analytically
%---
      
    alphac = 1.0-alpha;
    tmp = exp(-eta^2);
    U   =  alpha*tanh(eta) - alphac*tmp;
    Up  =  alpha/cosh(eta)^2 + alphac*2.0*eta*tmp;
    Upp = -alpha*2.0*sinh(eta)/cosh(eta)^3  ...
          -alphac*2.0*(1.0-2.0*eta)*tmp;
%---
 elseif(menu==2)  % derivatives by numerical differentiation
%---

    eps = 0.00001;
    U   = 1.0/cosh(eta)^2;
    U1  = 1.0/cosh(eta-eps)^2;
    U2  = 1.0/cosh(eta+eps)^2;
    Up  = (U2-U1)/(2.0*eps);
    Upp = (U2-2.0*U+U1)/eps^2;

%---
  else
%---

    disp('rayleigh_vel: this menu item is not available')
    return

%---
  end 
%---

%-----
% scale
%-----

  Up  =  Up/delta;
  Upp = Upp/delta^2;

  U   =   U*U0;
  Up  =  Up*U0;
  Upp = Upp*U0;

%-----
% Done
%-----

  return
