      program lvr_motion

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c---------------------------
c  Simulation of the interaction of a number of
c  co-axial line vortex rings.
c
c  The self induced axial velocity is
c  obtained from the Helmholtz formula 
c
c  (Pozrikidis 1997, eq. 11.7.3)
c---------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(10),s(10),core(10),circ(10)
      Dimension u(10),v(10)

      Character kbd*1        ! graphics
      Integer checkkey       ! graphics

c---
c common blocks
c---

      common/xxss/x,s,core,circ
      common/uuvv/u,v

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi4 = 4.0D0*pi

c---------------
c Set parameters
c---------------

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) ' 1 to read data from file: lvrm.dat'
      write (6,*) " 2 to type in data"
      write (6,*) " 0 to quit "
      write (6,*) '-----------'
      read  (5,*) Ienrd

      if(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.2) then
c------------------------

      write (6,*) 
      write (6,*) 'How many rings ?'
      write (6,*) '----------------'
      read  (5,*) Nrings

      Do i=1,Nrings

        write (6,*) 'Ring',i
        write (6,*) 
        write (6,*) 'Please enter:' 
        write (6,*) '   the ring x and sigma coordinates'
        write (6,*) '   the core radius '
        write (6,*) '   the circulation around this ring'
        write (6,*) '-----------------------------------'
        read  (5,*) x(i),s(i)
        read  (5,*) core(i)
        read  (5,*) circ(i)

      End Do
 
      write (6,*)
      write (6,*) 'Please enter the time step'
      write (6,*) '--------------------------'
      read  (5,*) Dt

      write (6,*)
      write (6,*) 'After how many steps print the position ?'
      write (6,*) '-----------------------------------------'
      read  (5,*) Nprint

      write (6,*)
      write (6,*) 'Enter number of time steps'
      write (6,*) '--------------------------'
      read  (5,*) Nstep

c---------
      else
c---------

      open (3,file="lvrm.dat")

       read (3,*) Nrings

       Do i=1,Nrings
         read (3,*)
         read (3,*) x(i),s(i)
         read (3,*) core(i)
         read (3,*) circ(i)
       End Do

       read (3,*)
       read (3,*) Dt
       read (3,*) Nprint
       read (3,*) Nstep

      close (3)

c-----------
      End If
c-----------

c--------
c prepare
c--------

      open (4,file="lvrm.xy")

      time  = 0.0D0

      Iprint = Nprint

c---------------
c prepare to run
c---------------

      Kstep  = 0      ! counters
      Istep  = 0

c-.-.-.-.-.-.-.-.-.-.-.-.-.-.
c BEGINNING OF THE SIMULATION
c-.-.-.-.-.-.-.-.-.-.-.-.-.-.

  98  Continue

      Kstep = Kstep+1
      Istep = Istep+1

      if(Istop.eq.1) Go to 99
      Itry = 2

c---
c compute the velocity
c---

      call lvrm_vel (Nrings,Iflow)

c---
c printing session
c---

      if(Iprint.eq.Nprint) then
        write (4,100) Kstep,time,(x(i),s(i),i=1,Nrings)
        write (6,100) Kstep,time,(x(i),s(i),i=1,Nrings)
        Iprint = 0
      end if

c-------------------------------------
c time integration by the Euler method
c
c Rate of the change of the core radius
c by equation (11.7.4) given on page 552
c of Pozrikidis (1997)
c-------------------------------------

      Do i=1,Nrings   
       core(i) = core(i) - 0.5*core(i)/s(i) * v(i) * Dt
       x(i) = x(i) + u(i)*Dt
       s(i) = s(i) + v(i)*Dt
      End Do

      time   = time+Dt
      Iprint = Iprint+1

c-----------------------
c check for continuation
c-----------------------

      if(Istep.eq.Nstep) then

       write (6,*) 
       write (6,*) " Continue the computation ? "
       write (6,*) 
       write (6,*) " Enter 0 for NO"
       write (6,*) " --------------"
       read  (5,*) more

       if(more.eq.1) then
          Go to 98
          Istep = 0
       end if

       Go to 99

      End If

c-------------------
c continue the steps
c-------------------

      Go to 98

c-----
c Done
c-----

  99  Continue

      write (4,100) null
      close (4)

 100  Format (1x,i10,10(1x,f10.5))

      Stop
      End
