      subroutine lvrm_vel (Nrings,Iflow)

c==========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c==========================================

c----------------------------------------
c Velocity induced by a collection of coaxial
c vortex rings
c----------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(10),s(10),core(10),circ(10)
      Dimension u(10),v(10)

      common/xxss/x,s,core,circ
      common/uuvv/u,v

c----------
c constants
c----------

      pi  = 3.14159 265358 D0
      pi4 = 4.0D0*pi

c-----------
c initialize
c-----------

      Iopt = 1    ! stream function is not needed

c---
c compute the induced velocity
c---

c---
      Do i=1,Nrings   ! loop over rings
c---

c---
c First compute velocity at point (x0,s0)
c induced from all other rings
c---

       x0 = x(i)
       s0 = s(i)

       u0 = 0.0D0
       v0 = 0.0D0

       Do j=1,Nrings

       if(i.ne.j) then   ! skip self-induced velocity

        xx = x(j)
        ss = s(j)

c---
c compute mutually-induced velocities
c using equations (11.7.1) on p. 551 of Pozrikidis (1997)
c---

        call lvr_fs 
     +
     +      (Iopt
     +      ,x0,s0
     +      ,xx,ss
     +      ,ur,vr
     +      ,psi
     +      )

         u0 = u0 + circ(j)*ur
         v0 = v0 + circ(j)*vr

       end if

       End Do

c---------------------------------------------------
c self-induced axial velocity according to Helmholtz
c from equation (11.7.3) on p.552
c---------------------------------------------------

       u0 = u0 + circ(i)/(pi4*s0)
     +    * (log(8.0D0*s0/core(i))-0.25D0)

c-----
c Done
c-----

       u(i) = u0
       v(i) = v0

c---
       End Do ! over rings
c---
       
      Return
      End
