%---
close all
clear all
%---

%=========
% self-indiced velocity of a vortex ring with finite core
%=========

rr = 1.0; % ring radius
circ = 1.0; % circulation
cr = 0.1;   % core radius

nrad = 64; % radial divisions
nchi = 64; % azimuthal divisions

nrepeat=32;

%---
% prepare
%---

dchi = 2*pi/nchi;  % angular divisions

%===========
for repeat=1:nrepeat
%===========

cr = 0.01+(repeat-1.0)/nrepeat;
dcr  = cr/nrad;   % radial divisions

%-----------------------------------------
% discretize the vortex core into elements
% and compute the area of each element: ar
%-----------------------------------------

for i=1:nrad
 for j=1:nchi
   ar(i,j) = 0.5*dchi*dcr*dcr*(2.0*i-1);
 end
end

%-----------
% initialize
%-----------

 arintv1 = 0.0; % areal integral of the vorticity
 arintv2 = 0.0; % areal integral of the vorticity

 u1 = 0.0;
 v1 = 0.0;
 u2 = 0.0;
 v2 = 0.0;

%----------------------
% compute the self-induced velocity
% integrate by the trapezoidal rule
%----------------------

 for i=1:nrad   % loop over radial elements

   rad = (i-0.5)*dcr;    % center of the ring elements
   w1 = circ/(pi*cr*cr);    % uniform vorticity
   w2 = circ*pi/(cr*cr) ... % normalized bell shape
         /(pi*pi-4.0)*(1.0+cos(pi*rad/cr));

   for j=1:nchi     %  loop over azimuthal elements

     chi = (j-0.5)*dchi;
     sig = rr+rad*sin(chi);
     Dx  = rad*cos(chi); 
     Dxs = Dx*Dx;
     ks = 4.0*rr*sig/(Dxs+(rr+sig)^2);
     [F, E]= ellipke(ks);
     RJ30 = E/(1.0-ks);
     RJ31 = (-2.0*F + E*(2.0-ks)/(1.0-ks))/ks;
     cf = 4.0/sqrt((Dxs+(rr+sig)^2)^3);
     RI30 = cf*RJ30;
     RI31 = cf*RJ31;
     cf1 = ar(i,j)*sig*w1/(4*pi);
     u1 = u1 + cf1*(-rr*RI31+sig*RI30);
     v1 = v1 + cf1*  Dx*RI31;
     cf2 = ar(i,j)*sig*w2/(4*pi);
     u2 = u2 + cf2*(-rr*RI31+sig*RI30);
     v2 = v2 + cf2*  Dx*RI31;
     arintv1 = arintv1 + w1*ar(i,j);
     arintv2 = arintv2 + w2*ar(i,j);

  end                %  of azimuthal loop

end        % of radial loop

[arintv1, arintv2]

%------------------------
% Helmholtz approximation
% for comparison purposes
%------------------------

 uh = circ/(4*pi*rr)*(log(8.0*rr/cr)-0.25);

 xplot(repeat) = cr/rr;
 velr1(repeat) = u1;
 velr2(repeat) = u2;
 velr3(repeat) = uh;

%===========
end
%===========

figure(1)
hold on
plot(xplot,velr1,'k');
plot(xplot,velr2,'k--');
plot(xplot,velr3,'k:');
xlabel('a/\sigma_r','fontsize',15)
ylabel('\sigma_r U_x/\kappa','fontsize',15)
set(gca,'fontsize',15)
axis([0 1.0 0 0.6])
box on

