clear all
close all
%===

global NGL

%=============================
% simulation of a vortex patch
%=============================

a=1.0;
b=0.40;
b=0.25;
b=0.15;

Dt = 0.02;
Dt = 0.2;
Dt = 0.05;

Nsteps = 1000;

Omega = 1.0;

NGL = 12;
NGL = 6;

N = 32;
N = 64;
N = 128;

%---
% point reditribution
%---

Ich1=1;
Ich2=1;
Ich3=1;
thmax = 0.25*pi;
spmax = 0.20;
spmin = 0.05;

%---
% initial shape
%---

deta = 2*pi/N;

for i=1:N+1
 eta = (i-1.0)*deta+0.00*pi;
 rad1 = a*(1.0+0.1*cos(2*eta));
 rad2 = b*(1.0-0.1*cos(2*eta));
 x(i) = rad1*cos(eta);
 y(i) = rad2*sin(eta);
 P1(i) = 0.0;
 P2(i) = 0.0;
end

%---
% prepare
%---

Omega_z = Omega*a*b/(a+b)^2;
Omega_z = 0;

figure(1)
xlabel('x')
ylabel('y')
hold on
plot(x,y,'r.')
patch(x,y,'y')
axis off

%==========
for Istep=1:Nsteps
%==========

 Isym = 0;
 Irepeat = 1;
 while(Irepeat==1)

  [N,x,y ...
  ,vnx,vny ...
  ,crv,s,Xint ...
  ,Axint,Bxint,Cxint ...
  ,Ayint,Byint,Cyint ...
  ,area,centerx,centery ...
  ,P1,P2 ...
  ,Istop,Irepeat,action] ...
  ...
    = prd_2d (N,x,y,P1,P2 ...
            ,Ich1,thmax ...
            ,Ich2,spmax ...
            ,Ich3,spmin ...
            ,Isym);

  if(Istop==1) break; end
  end
  if(Istop==1) break; end

%---
% contour integral
%---

  for i=1:N
    velx(i) = 0.0;
    vely(i) = 0.0;

    for j=1:N
    j1=j+1;

    Ising = 0;
    if(i==j) Ising = 1; end           % first end-point
    if(i==j+1) Ising = 2; end         % second end-point
    if(i==1 & j==N) Ising = 2; end   % second end-point

    [SLPx, SLPy] = vp_slp_spline  ...
    ...
    (x(i),y(i) ...
    ,x(j),y(j)...
    ,x(j1),y(j1)...
    ,Xint(j),Xint(j1) ...
    ,Axint(j),Bxint(j),Cxint(j) ...
    ,Ayint(j),Byint(j),Cyint(j) ...
    ,Ising);
    velx(i) = velx(i)+SLPx;
    vely(i) = vely(i)+SLPy;
    end
  end

  velx = Omega*velx;
  vely = Omega*vely;
  for i=1:N
   velx(i) = velx(i) + Omega_z*y(i);
   vely(i) = vely(i) - Omega_z*x(i);
  end

  velx(N+1) = velx(1);
  vely(N+1) = vely(1);

% [velx', vely']
% pause

%---
% tangential velocity and Jacobian on the horizontal ellipse
%---
%  for i=1:N
%   tnx = -vny(i);
%   tny =  vnx(i);
%   un(i) = velx(i)*vnx(i)+vely(i)*vny(i)
%   ut(i) = velx(i)*tnx+vely(i)*tny
%   jac(i) = Omega_z*sqrt(a^2/b^2*y(i)^2+b^2/a^2*x(i)^2)
%   pause
%  end
%---

%  [velx; vely]'
%  pause

 figure(1)
 hold off
 plot(x(1:N+1),y(1:N+1),'r.')
 patch(x(1:N+1),y(1:N+1),'y')
 axis equal
 axis([-1.5 1.5 -1.5 1.5])
 axis off
 pause (0.01)

 for i=1:N+1
  x(i) = x(i) + Dt*velx(i);
  y(i) = y(i) + Dt*vely(i);
 end

 velxsave = velx;
 velysave = vely;
 xsave = x;
 ysave = y;

  for i=1:N
    velx(i) = 0.0;
    vely(i) = 0.0;
    for j=1:N
    j1=j+1;
    Ising = 0;
    if(i==j) Ising = 1; end           % first end-point
    if(i==j+1) Ising = 2; end         % second end-point
    if(i==1 & j==N) Ising = 2; end   % second end-point
    [SLPx, SLPy] = vp_slp_spline  ...
    ...
    (x(i),y(i) ...
    ,x(j),y(j)...
    ,x(j1),y(j1)...
    ,Xint(j),Xint(j1) ...
    ,Axint(j),Bxint(j),Cxint(j) ...
    ,Ayint(j),Byint(j),Cyint(j) ...
    ,Ising);
    velx(i) = velx(i)+SLPx;
    vely(i) = vely(i)+SLPy;
    end
  end

  velx = Omega*velx;
  vely = Omega*vely;
  for i=1:N
   velx(i) = velx(i) + Omega_z*y(i);
   vely(i) = vely(i) - Omega_z*x(i);
  end
  velx(N+1) = velx(1);
  vely(N+1) = vely(1);

 for i=1:N+1
  x(i) = xsave(i) + 0.5*Dt*(velxsave(i)+velx(i));
  y(i) = ysave(i) + 0.5*Dt*(velysave(i)+vely(i));
 end

end

