      subroutine sgrad3_3d_interp
     +
     +   (x1,y1,z1
     +   ,x2,y2,z2
     +   ,x3,y3,z3
     +   ,x4,y4,z4
     +   ,x5,y5,z5
     +   ,x6,y6,z6
     +
     +   ,fx1,fy1,fz1
     +   ,fx2,fy2,fz2
     +   ,fx3,fy3,fz3
     +   ,fx4,fy4,fz4
     +   ,fx5,fy5,fz5
     +   ,fx6,fy6,fz6
     +
     +   ,al,be,ga
     +   ,xi,eta
     +
     +   ,vnxa,vnya,vnza
     +   ,dfxx,dfyx,dfzx
     +   ,dfxy,dfyy,dfzy
     +   ,dfxz,dfyz,dfzz
     +   )

c===========================================
c FDLIB, BEMLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement.
c===========================================

c-------------------------------------------------
c Compute the Frechet derivative of the potential
c at a point on an element
c--------------------------------------------------

      Implicit Double Precision (a-h,o-z)

c--------
c prepare
c--------

      alc = 1.0D0-al
      bec = 1.0D0-be
      gac = 1.0D0-ga

      alalc = al*alc
      bebec = be*bec
      gagac = ga*gac

c-----------------------
c  xi derivatives of phi
c-----------------------

      dph2 =  (2.0D0*xi-al +eta*(al-ga)/gac)/alc
      dph3 =  eta*(be+ga-1.0)/(ga*bec)
      dph4 =  (1.0D0-2.0D0*xi-eta)/alalc
      dph5 =  eta/gagac
      dph6 = -eta/bebec
      dph1 = -dph2-dph3-dph4-dph5-dph6

c----------------------------
c  compute dx/dxi and df /dxi
c  from xi derivatives of phi
c----------------------------

      DxDxi = x1*dph1 + x2*dph2 + x3*dph3 + x4*dph4
     +      + x5*dph5 + x6*dph6

      DyDxi = y1*dph1 + y2*dph2 + y3*dph3 + y4*dph4
     +      + y5*dph5 + y6*dph6

      DzDxi = z1*dph1 + z2*dph2 + z3*dph3 + z4*dph4
     +      + z5*dph5 + z6*dph6

      dfxDxi = fx1*dph1 + fx2*dph2 + fx3*dph3
     +       + fx4*dph4 + fx5*dph5 + fx6*dph6

      dfyDxi = fy1*dph1 + fy2*dph2 + fy3*dph3
     +       + fy4*dph4 + fy5*dph5 + fy6*dph6

      dfzDxi = fz1*dph1 + fz2*dph2 + fz3*dph3
     +       + fz4*dph4 + fz5*dph5 + fz6*dph6

c------------------------
c  eta derivatives of phi
c------------------------

      pph2 =  xi*(al-ga)/(alc*gac)
      pph3 =  (2.0D0*eta-be+xi*(be+ga-1.0D0)/ga)/bec
      pph4 =  -xi/alalc
      pph5 =   xi/gagac
      pph6 =  (1.0D0-xi-2.0D0*eta)/bebec
      pph1 = -pph2-pph3-pph4-pph5-pph6

c----------------------------
c  compute dx/deta and df /deta
c  from xi derivatives of phi
c----------------------------

      DxDet = x1*pph1 + x2*pph2 + x3*pph3 + x4*pph4
     +      + x5*pph5 + x6*pph6

      DyDet = y1*pph1 + y2*pph2 + y3*pph3 + y4*pph4
     +      + y5*pph5 + y6*pph6

      DzDet = z1*pph1 + z2*pph2 + z3*pph3 + z4*pph4
     +      + z5*pph5 + z6*pph6

      dfxDet = fx1*pph1 + fx2*pph2 + fx3*pph3
     +       + fx4*pph4 + fx5*pph5 + fx6*pph6

      dfyDet = fy1*pph1 + fy2*pph2 + fy3*pph3
     +       + fy4*pph4 + fy5*pph5 + fy6*pph6

      dfzDet = fz1*pph1 + fz2*pph2 + fz3*pph3
     +       + fz4*pph4 + fz5*pph5 + fz6*pph6

c-------------------------------
c  raw normal vector (non-unit):
c  vn = (DxDxi)x(DxDeta)
c-------------------------------

      vnxr = DyDxi * DzDet - DyDet * DzDxi
      vnyr = DzDxi * DxDet - DzDet * DxDxi
      vnzr = DxDxi * DyDet - DxDet * DyDxi

c-------------------------------
c compute tangential derivatives
c by solving three linear systems
c-------------------------------

      A11 = DxDxi
      A12 = DyDxi
      A13 = DzDxi

      A21 = DxDet
      A22 = DyDet
      A23 = DzDet

      A31 = vnxa
      A32 = vnya
      A33 = vnza

c     A31 = vnxr
c     A32 = vnyr
c     A33 = vnzr

      B1 = dfxDxi
      B2 = dfxDet
      B3 = 0.0

      call cramer_33
     +
     +   (A11,A12,A13
     +   ,A21,A22,A23
     +   ,A31,A32,A33
     +   ,B1,B2,B3
     +   ,dfxx,dfyx,dfzx
     +   )

      B1 = dfyDxi
      B2 = dfyDet

      call cramer_33
     +
     +   (A11,A12,A13
     +   ,A21,A22,A23
     +   ,A31,A32,A33
     +   ,B1,B2,B3
     +   ,dfxy,dfyy,dfzy
     +   )

      B1 = dfzDxi
      B2 = dfzDet

      call cramer_33
     +
     +    (A11,A12,A13
     +    ,A21,A22,A23
     +    ,A31,A32,A33
     +    ,B1,B2,B3
     +    ,dfxz,dfyz,dfzz
     +    )

c---
c done
c---

      return
      end
