      program vs_3d

c============================================
c FDLIB, BEMLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c============================================

c-----------------------------------------------
c  Dynamical simulation of the self-induced motion
c  of a closed three-dimensional vortex sheet
c
c  SYMBOLS:
c  -------
c
c  cxp, cyp, czp:  coordinates of the center of the
c                  ellipsoidal vortex sheet
c
c  npts:  total number of points
c  nelm:  total number of elements
c
c  p(i,j):    coordinates of point i (j=1,2,3)
c  zeta(i,j): vorticity of point i (j=1,2,3)
c
c  n(k,i): connectivity table: 
c          global labels of points for element k
c          i = 1, ..., 6
c
c  nbe(k,j): the three neighboring elements of element k (j=1,2,3)
c
c  ne(k,j):  ne(k,1) is the number of elements adjacent to point k
c            ne(k,2), ... are the elements numbers, j = 2, ..., 7
c            for this triangulation, up to six
c
c  alpha, beta, gamma: triangle mapping coefficients
c
c smm: smoothing matrix
c------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension       p(1026,3)
      Dimension      ne(1026,7)
      Dimension     vna(1026,3)
      Dimension    zeta(1026,3)
      Dimension    circ(1026),dcircdt(1026)
      Dimension   gradf(1026,3)
      Dimension     bsi(1026,3)
      Dimension    crvm(1026)
      Dimension   aux_s(1026)       ! auxiliary scalar

      Dimension  ptl(1026,3),curl(1026,3)
      Dimension hptl(1026)

      Dimension dfxx(1026),dfyx(1026),dfzx(1026)
      Dimension dfxy(1026),dfyy(1026),dfzy(1026)
      Dimension dfxz(1026),dfyz(1026),dfzz(1026)

      Dimension du(1026,3),ddudt(1026,3)

      Dimension      n(512,6),nbe(512,3)
      Dimension   alpha(512),beta(512),gamma(512)
      Dimension    arel(512),xmom(512), ymom(512),zmom(512)
      Dimension  crvmel(512)

      Dimension zz(20),ww(20)
      Dimension xiq(20),etq(20),wq(20)

      Dimension jxy(100)

c---
c for smoothing
c---

      Dimension f_sm(1026)
      Dimension  smm(1026,1026)

c---
c for time stepping
c---

      Dimension     time(2000),  volume(2000),enrg_srf(2000)
      Dimension enrg_knt(2000),enrg_tot(2000)

c----
c save for RK2
c----

      Dimension ps(1026,3),us(1026,3)
      Dimension circs(1026),dcircdts(1026)

c---
c common blocks
c---

      common/points/p,ne
      common/elmnts/n,nbe

      common/albega/alpha,beta,gamma

      common/geo1/arel
      common/geo2/vna
      common/geo3/crvm
      common/geo6/crvmel

      common/zzetaa/zeta
      common/krasny/dels

      common/zwl/zz,ww
      common/trq/xiq,etq,wq

      common/pii/pi,piq,pih,pi2,pi4,pi6,pi8

c----------
c constants
c----------

      pi = 3.14159 265358 D0

      piq = 0.25D0*pi
      pih = 0.50D0*pi
      pi2 = 2.0D0*pi
      pi4 = 4.0D0*pi
      pi6 = 6.0D0*pi
      pi8 = 8.0D0*pi

      oot = 1.0D0/3.0D0

      null = 0

c-----------
c Input data
c-----------


      write (6,*)
      write (6,*) ' Enter:'
      write (6,*)
      write (6,*) ' 1 to read date from file: vs_3d.dat'
      write (6,*) ' 2 to type the data into the keyboard'
      write (6,*) ' 0 to quit'
      write (6,*) ' ---------'
      read  (5,*)   Ienrd

      if(Ienrd.eq.0) Go to 99

c------------------------
      if(Ienrd.eq.1) then
c------------------------

      open (2,file="vs_3d.dat")

      read (2,*) menu
      read (2,*) Iflow
      read (2,*)
      read (2,*) amp      ! oscillation amplitude
      read (2,*) nsh,msh  ! oscillation modes
      read (2,*)
      read (2,*) Ioctaicos
      read (2,*) ndiv 
      read (2,*)
      read (2,*) boa,coa
      read (2,*) req
      read (2,*) cx,cy,cz
      read (2,*)
      read (2,*) mint
      read (2,*) NGL
      read (2,*)
      read (2,*) Move
      read (2,*) Method
      read (2,*) IPBZ
      read (2,*)
      read (2,*) rho
      read (2,*) srtn
      read (2,*) 
      read (2,*) Iread
      read (2,*)
      read (2,*) Isym_xy
      read (2,*)
      read (2,*) Nprint_xy
      read (2,*) Nprint_xyz
      read (2,*)
      read (2,*) Nsm
      read (2,*) Ism_opt
      read (2,*) Ism_init
      read (2,*)
      read (2,*) dels
      read (2,*)
      read (2,*) IRK
      read (2,*) Dt
      read (2,*) Nstep

      close (2)

c---------
      else
c---------

  91  Continue

      write (6,*)
      write (6,*) ' Enter:'
      write (6,*)
      write (6,*) ' 1 for a dynamic simulation'
      write (6,*) ' 2 to compute the vector potential'
      write (6,*) '   and its curl normal to the vortex sheet'
      write (6,*) ' 3 to compute the harmonic potential'
      write (6,*) '   and its gradient tangential'
      write (6,*) '   to the vortex sheet'
      write (6,*) ' 4 to compute the Biot-Savart integral'
      write (6,*) '   using the Pozrikidis method'
      write (6,*) ' 5 to compute the Biot-Savart integral'
      write (6,*) '   using the Zinchenko method'
      write (6,*) ' 0 to quit'
      write (6,*) ' ---------'
      read  (5,*)  menu

      if(menu.eq.0) Go to 99

      write (6,*)
      write (6,*) '  Choose the flow'
      write (6,*)
      write (6,*) ' Enter:'
      write (6,*)
      write (6,*) ' 1 for a free spherical vortex sheet'
      write (6,*) ' 2 for drop oscillations induced by surface tension'
      write (6,*) ' 0 to quit'
      write (6,*) ' ---------'
      read  (5,*)  Iflow

      if(Iflow.eq.0) Go to 99

      if(Iflow.eq.2) then

       write (6,*)
       write (6,*) ' Please enter the shape modes n and m'
       write (6,*) ' ------------------------------------'
       read  (5,*)  nsh,msh

       write (6,*)
       write (6,*) ' Please enter the amplitude'
       write (6,*) ' ---------------------------'
       read  (5,*)  amp

      end if

      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 to discretize an octahedron"
      write (6,*) " 2 to discretize an icosahedron"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Ioctaicos

      if(Ioctaicos.eq.0) Go to 99

      write (6,*)
      write (6,*) " Select the level of triangulation"
      write (6,*)
      write (6,*) " Enter 0, 1, 2, 3; 99 to quit"
      write (6,*) " ----------------------------"
      read  (5,*) ndiv 

      if(ndiv.eq.99) Go to 99

      if(ndiv.gt.3)  then
        write (6,*)
        write (6,*) 'out of range; please try again.'
        write (6,*)
        Go to 91
      end if

      write (6,*)
      write (6,*) " Will discretize the surface of an ellipsoid"
      write (6,*)
      write (6,*) " Please enter the axes ratios b/a and c/a"
      write (6,*) " ----------------------------------------"
      read  (5,*) boa,coa

      write (6,*)
      write (6,*) " Please enter the equivalent radius"
      write (6,*) " ----------------------------------"
      read  (5,*) req

      write (6,*)
      write (6,*) " Please enter the coordinates"
      write (6,*) " of the center of the ellipsoid center"
      write (6,*) " -------------------------------------"
      read  (5,*) cxp,cyp,czp

      write (6,*)
      write (6,*) " Will use the m-point gauss-triangle rule"
      write (6,*) "      for integration over each triangle"
      write (6,*)
      write (6,*) " Please enter m"
      write (6,*)
      write (6,*) " choose from 3, 4, 5, 6, 9, 12, 13"
      write (6,*) "                   Enter 0 to quit"
      write (6,*) "----------------------------------"
      read  (5,*) mint

      write (6,*)
      write (6,*) " Enter the number of Gauss-Legendre"
      write (6,*) " nodes for integration in polar coordinates"
      write (6,*) " ------------------------------------------"
      read  (5,*) NGL

      write (6,*)
      write (6,*) " Motion of marker points"
      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 0 for total velocity"
      write (6,*) " 1 for normal velocity"
      write (6,*) " ---------------------"
      read  (5,*) Move

      write (6,*)
      write (6,*) " Choose the formulation"
      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 for the circulation formulation"
      write (6,*) " 2 for the sheet strength formulation"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) Method

      write (6,*)
      write (6,*) " Choose the method of computing the velocity"
      write (6,*)
      write (6,*) " Enter:"
      write (6,*)
      write (6,*) " 1 for the Pozrikidis method"
      write (6,*) " 2 for the Baker method"
      write (6,*) " 3 for the Zinchenko method"
      write (6,*) " 0 to quit"
      write (6,*) " ---------"
      read  (5,*) IPBZ

      if(IPBZ.eq.0) Go to 99

      write (6,*)
      write (6,*) " Enter the fluid density"
      write (6,*) " -----------------------"
      read  (5,*) rho

      write (6,*)
      write (6,*) " Enter the surface tension "
      write (6,*) " --------------------------"
      read  (5,*) srtn

      write (6,*)
      write (6,*) " Enter 0 to generate initial data"
      write (6,*) "       1 to read from file: vs_3d.inp"
      write (6,*) " -------------------------------------"
      read  (5,*) Iread

      write (6,*)
      write (6,*) " Enter 1 if the sheet is symmetric wrt the xy plane"
      write (6,*) "       0 otherwise"
      write (6,*) " -----------------"
      read  (5,*) Isym_xy

      write (6,*)
      write (6,*) " Will record a profile in the xy plane"
      write (6,*) " after N steps; please enter N"
      write (6,*) " -----------------------------"
      read  (5,*) Nprint_xy

      write (6,*)
      write (6,*) " Will record a 3D profile"
      write (6,*) " after N steps; please enter N"
      write (6,*) " -----------------------------"
      read  (5,*) Nprint_xyz

      write (6,*)
      write (6,*) " After how many steps smoothing ?"
      write (6,*)
      write (6,*) " Enter 0 for no smoothing"
      write (6,*) " ------------------------"
      read  (5,*) Nsm

      write (6,*)
      write (6,*) " Enter 1 to smooth the potential"
      write (6,*) "       2 to smooth potential and position"
      write (6,*) " ----------------------------------------"
      read  (5,*) Ism_opt

      write (6,*)
      write (6,*) " Smoothing at the initial instant ?"
      write (6,*)
      write (6,*) " Enter 1 for yes, 0 for no"
      write (6,*) " -------------------------"
      read  (5,*) Ism_init

      write (6,*)
      write (6,*) " Please enter the Krasny delta**2 parameter"
      write (6,*) " ------------------------------------------"
      read  (5,*) dels

      write (6,*)
      write (6,*) " Select the RK method: 1 or 2"
      write (6,*) " ----------------------------"
      read  (5,*) IRK

      write (6,*)
      write (6,*) " Enter the number of steps"
      write (6,*)
      write (6,*)
      write (6,*) " Please enter the time step"
      write (6,*) " --------------------------"
      read  (5,*) Dt

      write (6,*)
      write (6,*) " Enter the number of steps"
      write (6,*) " -------------------------"
      read  (5,*) Nstep

c-----------
      end if ! End of reading
c-----------

      if(menu.eq.0) Go to 99

c------------------
c open output files
c------------------

      open (1,file="vs_3d.xy")
      open (3,file="vs_3d.xyz")
      open (4,file="vs_3d.rst")
      open (7,file="vs_3d.diag")

c----------------------------
c triangulate the unit sphere
c
c Run even at restart to generate
c the connectivity table
c----------------------------

      if(Ioctaicos.eq.1) then

       call trgl6_octa
     +
     +  (ndiv
     +  ,npts,nelm
     +  )

      else

      call trgl6_icos
     +
     +   (ndiv
     +   ,npts,nelm
     +   )

      end if
      
      write (6,*)
      write (6,*) "Number of nodes:   ",npts
      write (6,*)
      write (6,*) "Number of elements: ",nelm
      write (6,*)

      if(Isym_xy.eq.1) czp = 0.0

c-------------------------
c AB INITIO
c
c Initialize time
c
c Assign the circulation
c or the vortex sheet strength 
c at the marker points
c-------------------------

c===
      if(Iread.eq.0) then  ! generate the initial condition
c===

      time(1) = 0.0

c--------------------------
      if(Method.eq.1) then    ! circulation method: assign circulation to nodes
c--------------------------

c------------------------
      if(Iflow.eq.1) then       ! flow past a sphere
c------------------------

       Do i=1,npts

        tmp = (p(i,1)-cx)/req
        if(tmp.gt. 1.0) tmp = 0.9999999
        if(tmp.lt.-1.0) tmp =-0.9999999
        theta = acos(tmp)
        circ(i) = 1.5*req**2 * cos(theta)

       End Do

c-----------------------------
      else if(Iflow.eq.2) then  ! drop oscillations
c-----------------------------

         arg  = srtn*req/rho 
     +        * nsh*(nsh+1.0)*(nsh-1.0)*(nsh+2.0)
     +          /(2.0*nsh+1)

         ccff = amp * (2*nsh+1.0)*sqrt(arg)/(nsh*(nsh+1))

c-------
        if(nsh.eq.2) then     ! mode n = 2
c-------

          cf = ccff * 0.5*sqrt(5.0/pi)

          If(msh.eq.0) then
           cf = cf
          else if(msh.eq.1) then
           cf = cf*sqrt(1.0/6.0)
          else if(msh.eq.2) then
           cf = cf*sqrt(1.0/24.0)
          else
           write (6,*) " This mode does not exist"
           stop
          End If

c-------
        else
c------
         write (6,*) " this oscollation  mode has not been implemented"
         stop
c-----
        end if  ! end of modes
c-----

         Do i=1,npts

         sigmas = p(i,2)**2+p(i,3)**2
         sigma  = sqrt(sigmas)

         if(sigma.le.0.000000001) then
          cosphi = 0.0                  ! arbitrary
         else
          cosphi = p(i,2)/sigma
         end if

         if(cosphi.gt. 0.99999999) cosphi = 0.9999999
         if(cosphi.lt.-0.99999999) cosphi =-0.9999999
         phi = acos(cosphi)
         if(p(i,3).lt.0) phi = pi2-phi

         costh = p(i,1)/req
         if(costh.gt. 0.99999999) costh = 0.9999999
         if(costh.lt.-0.99999999) costh =-0.9999999
         theta = acos(costh)
         sinth = sqrt(1.0-costh**2)

         if(nsh.eq.2.and.msh.eq.0) then              ! mode 2-0
            circ(i) = -cf*0.5*(3.0*costh**2-1.0)
         else if(nsh.eq.2.and.msh.eq.1) then         ! mode 2-1
            theta2 = 2.0*theta
            circ(i) = cf*1.5*sin(theta2)*cosphi
         else if(nsh.eq.2.and.msh.eq.2) then         ! mode 2-2
            theta2  = 2.0*theta
            phi2    = 2.0*phi
            circ(i) = cf*1.5*(1.0-cos(theta2))*cos(phi2)
         else
           write (6,*) " This mode has not been implemented"
           stop
         end if

        End Do

c-------------
        end if  ! end of Iflow for the circulation method
c-------------

c----------
       else     ! vortex sheet strength method
c----------

       Do i=1,npts

c------
        if(Iflow.eq.1) then  ! flow past a sphere
c------

        tmp = (p(i,1)-cx)/req
        if(tmp.gt. 1.0) tmp = 0.9999999
        if(tmp.lt.-1.0) tmp =-0.9999999

        theta = acos(tmp)

        zetmag = -1.5*req*sin(theta)

        par = sqrt(p(i,3)**2+p(i,2)**2)

        if(par.lt.0.000001) then
         zeta(i,1) =  0.0
         zeta(i,2) =  0.0
         zeta(i,3) =  0.0
        else
         zeta(i,1) =  0.0
         zeta(i,2) = -zetmag*p(i,3)/par
         zeta(i,3) =  zetmag*p(i,2)/par
        end if

c------
        end if   ! end of Iflow
c------

      End Do

c------------
      end if       ! end of assigning circulation or strength
c------------

c====
      end if  ! of Iread=0
c====

c---------------------------
c expand to specified radius;
c Deform to an ellipsoid;
c Translate the center
c to specified position
c---------------------------

      scale = req/(boa*coa)**oot

      Do i=1,npts
        p(i,1) = scale*    p(i,1) + cxp
        p(i,2) = scale*boa*p(i,2) + cyp
        p(i,3) = scale*coa*p(i,3) + czp
      End Do

c---------------------------------
c RESTART
c
c read node position, circulation 
c or vortex sheet strength
c---------------------------------

c====
      if(Iread.eq.1) then
c====

      open (9,file="vs_3d.inp")

       read (9,*) npts,time(1)

c--------------------------
       if(Method.eq.1) then
c--------------------------

         Do i=1,npts
           read (9,*) l,p(i,1),p(i,2),p(i,3),circ(i)
         End Do

c----------
       else 
c----------

         Do i=1,npts
           read (9,*) l,p(i,1),p(i,2),p(i,3)
     +                 ,zeta(i,1),zeta(i,2),zeta(i,3)
         End Do

c------------
       end if
c------------

       close (9)

c====
      end if  ! of Iread=1
c====

c-----------------
c read quadratures
c-----------------

      call gauss_leg (NGL,zz,ww)

      call gauss_trgl 
     +
     +   (mint
     +   ,xiq,etq,wq
     +   )

c-----------------------------
c compute the normal vector and the
c surface centroid
c to be used by slice_xy
c and for generating Du from zeta
c-----------------------------

      Do k=1,nelm

        i1 = n(k,1)
        i2 = n(k,2)
        i3 = n(k,3)
        i4 = n(k,4)
        i5 = n(k,5)
        i6 = n(k,6)

        call abc
     +
     +   (p(i1,1),p(i1,2),p(i1,3)
     +   ,p(i2,1),p(i2,2),p(i2,3)
     +   ,p(i3,1),p(i3,2),p(i3,3)
     +   ,p(i4,1),p(i4,2),p(i4,3)
     +   ,p(i5,1),p(i5,2),p(i5,3)
     +   ,p(i6,1),p(i6,2),p(i6,3)
     +   ,alpha(k),beta(k),gamma(k)
     +   )

      End Do

      call elm_geom
     +
     +   (nelm,npts,mint
     +   ,xmom,ymom,zmom
     +   ,area,vlm
     +   ,cx,cy,cz
     +   )

c--------------------------------
c Method 2:
c
c compute Du at the marker points
c from zeta
c
c Du = zeta x vna
c--------------------------------

      if(Method.eq.2) then

       Do i=1,npts
        Du(i,1) = zeta(i,2)*vna(i,3)-zeta(i,3)*vna(i,2)
        Du(i,2) = zeta(i,3)*vna(i,1)-zeta(i,1)*vna(i,3)
        Du(i,3) = zeta(i,1)*vna(i,2)-zeta(i,2)*vna(i,1)
       End Do

      end if

c-----------------------------------------------
c Find the nodes in the xy plane and order them
c
c nxy is number of points in the xy plane
c the vector jxy contains consecutive points
c-----------------------------------------------

      if(Isym_xy.eq.1) then

       cz = 0.0

       write (6,800) cx,cy,cz

       call slice_xy
     +
     +  (npts
     +  ,cx,cy,cz
     +  ,nxy,jxy
     +  )

      end if

c-----------------------------
c read in the smoothing matrix
c-----------------------------

      if(Nsm.gt.0) then

       open (2,file="sm_3d_cl_tr.table")

       write (6,*) "vs_3d: reading the smoothing matrix"
       write (6,*) "       please be patient"

       read (2,*) j_trunc    ! truncation level

       Do i=1,npts
        Do k=1,npts
          read (2,*) idle,idle,smm(k,i)
        End Do
       End Do

       close (2)

       write (6,*) " vs_3d: Done"

      end if

      if(Nsm.eq.0) Ism_init = 0        ! for smoothing
      if(Ism_init.eq.0)  Ism = 1       ! for smoothing
      if(Ism_init.eq.1)  Ism = Nsm     ! for smoothing

      Kstep = 1          ! step counter
      Istep = 1          ! step counter

      Iprint_xy  = Nprint_xy      ! printing
      Iprint_xyz = Nprint_xyz     ! printing

c-.-.-.-.-.-.-.-.-.-.
c BEGIN TIME STEPPING
c-.-.-.-.-.-.-.-.-.-.

  90  Continue         ! will return here after each step

      write (6,*) "Step: ",Istep
      write (6,*) "Time: ",time(Istep)

c----------------------------
c Compute alpha, beta, gamma,
c for the xi-eta mapping
c over each triangle
c---------------------------

      Do k=1,nelm

       i1 = n(k,1)
       i2 = n(k,2)
       i3 = n(k,3)
       i4 = n(k,4)
       i5 = n(k,5)
       i6 = n(k,6)

       call abc 
     +
     +   (p(i1,1),p(i1,2),p(i1,3)
     +   ,p(i2,1),p(i2,2),p(i2,3)
     +   ,p(i3,1),p(i3,2),p(i3,3)
     +   ,p(i4,1),p(i4,2),p(i4,3)
     +   ,p(i5,1),p(i5,2),p(i5,3)
     +   ,p(i6,1),p(i6,2),p(i6,3)
     +   ,alpha(k),beta(k),gamma(k)
     +   )

      End Do

c------------------------------------------------
c compute:
c
c     vlm:    surface area of the individual elements
c             x, y, and z surface moments over each element
c     area:   total surface area and volume
c     crvmel: mean curvature over each element
c     vna:    average normal vector at the nodes
c------------------------------------------------

      call elm_geom 
     +
     +  (nelm,npts,mint
     +  ,xmom,ymom,zmom
     +  ,area,vlm
     +  ,cx,cy,cz
     +  )

      arean = area/(pi4*req**2)        ! normalize
      vlmn  =  vlm/(pi4*req**3/3.0)    ! normalize

      write (6,*)
      write (6,110) arean
      write (6,111) vlmn
      write (6,*)

      write (6,800) cx,cy,cz

      volume(Kstep)   = vlmn
      enrg_srf(Kstep) = srtn*arean  ! normalized surface energy

c-------------------------------------------------
c Method 1:
c
c Compute the vortex sheet strength (zeta)
c from the circulation (circ)
c by differentiating in curvilinear coordinates
c-------------------------------------------------

      if(Method.eq.1) then

       call vs_3d_circ_vort
     +
     +   (npts,nelm
     +   ,circ
     +   ,zeta
     +   )

c      write (6,*)
c      write (6,*) " circulation and  zeta"
c      write (6,*)
c
c      Do i=1,npts
c       write (6,100) i,circ(i),zeta(i,1),zeta(i,2),zeta(i,3)
c      End Do

      end if

c------------------------------------------
c menu = 2
c
c compute: (a) the vector potential
c          (b) the component of the curl
c              of the vector potential
c              normal to the surface
c------------------------------------------

      if(menu.eq.2) then
 
       call lslp3_3d
     +
     +   (npts,nelm
     +   ,mint,NGL
     +   ,zeta
     +   ,Iflow
     +   ,ptl
     +   )
 
       call vs_3d_curl
     +
     +   (npts,nelm
     +   ,ptl
     +   ,curl
     +   )
 
       write (6,*)
       write (6,*)  "coordinates, vector potential and normal velocity"
       write (6,*)
 
c--------------
c Iflow = 1:
c
c add the normal component
c of the incident flow
c and compute the error
c--------------

       if(Iflow.eq.1) then

       open (8,file="vs_3d.test")

       error = 0.0
 
       Do i=1,npts

        curl(i,1) = curl(i,1) + vna(i,1)*vna(i,1)
        curl(i,2) = curl(i,2) + vna(i,1)*vna(i,2)
        curl(i,3) = curl(i,3) + vna(i,1)*vna(i,3)

        error = error + curl(i,1)**2 + curl(i,2)**2 + curl(i,3)**2
 
        write (6,100) i,p(i,1),p(i,2),p(i,3)
     +                ,ptl(i,1),ptl(i,2),ptl(i,3)
     +                ,curl(i,1),curl(i,2),curl(i,3)
        write (8,100) i,p(i,1),p(i,2),p(i,3)
     +                ,ptl(i,1),ptl(i,2),ptl(i,3)
     +                ,curl(i,1),curl(i,2),curl(i,3)
       End Do
  
       error = sqrt(error) /(3.0*npts)
 
       write (6,*)
       write (6,567) error
       write (6,*)

       close (8)
       end if

       stop

c---------
      end if     ! end of menu = 2
c---------
 
c-----------------------------------------
c menu = 3
c
c compute the principal value of the harmonic potential
c         and its tangential gradient
c--------------------------------------------
 
      if(menu.eq.3) then

       call ldlp_3d
     +
     +   (npts,nelm
     +   ,mint
     +   ,circ
     +   ,hptl
     +   )
 
       call sgrad_3d
     +
     +  (npts,nelm
     +  ,hptl
     +  ,gradf
     +  )

       write (6,*)
       write (6,*)  "coordinates, harmonic potential and tangential vel"
       write (6,*)
 
c---------------------------
c Iflow=1:
c
c add the tangential component
c of the incident flow
c and compute the error
c----------------------------

       if(Iflow.eq.1) then

       open (8,file="vs_3d.test")
       error = 0.0
  
       Do i=1,npts
        gradf(i,1) = gradf(i,1) + 1.0 - vna(i,1)*vna(i,1)
        gradf(i,2) = gradf(i,2)       - vna(i,1)*vna(i,2)
        gradf(i,3) = gradf(i,3)       - vna(i,1)*vna(i,3)
        error = error + gradf(i,1)**2 + gradf(i,2)**2 + gradf(i,3)**2
 
        write (6,100) i,p(i,1),p(i,2),p(i,3)
     +                ,hptl(i)
     +                ,gradf(i,1),gradf(i,2),gradf(i,3)
        write (8,100) i,p(i,1),p(i,2),p(i,3)
     +                ,hptl(i)
     +                ,gradf(i,1),gradf(i,2),gradf(i,3)
       End Do
 
       error = sqrt(error) /(3.0*npts)
 
       write (6,*)
       write (6,567) error
       write (6,*)

       close (8)
       End if

       stop

c----------------------------------
      end if    ! end of menu = 3
c----------------------------------

c---------------------------------
c menu = 4, 5
c
c compute the Biot-Savart integral
c---------------------------------

      if(menu.eq.4.or.menu.eq.5) then

      if(menu.eq.4) then
 
      call vs_3d_bsi
     +
     +    (npts,nelm
     +    ,mint,NGL
     +    ,Iflow
     +    ,bsi
     +    )
 
      else if(menu.eq.5) then

      call vs_3d_bsi_zin
     +
     +   (npts,nelm
     +   ,mint,NGL
     +   ,Iflow
     +   ,bsi
     +   )

      end if

c---
c Iflow = 1:
c
c add the incident velocity
c and compute the error
c---

       open (8,file="test.out")

       write (6,*)
       write (6,*)  "coordinates, Biot-Savart integral"
       write (6,*)

       error = 0.0
  
       Do i=1,npts

        if(Iflow.eq.1) bsi(i,1) = bsi(i,1)+1.0D0

        prj = bsi(i,1)*vna(i,1) 
     +      + bsi(i,2)*vna(i,2)
     +      + bsi(i,3)*vna(i,3)

        error = error + bsi(i,1)**2 + bsi(i,2)**2 + bsi(i,3)**2

        write (6,100) i,p(i,1),p(i,2),p(i,3)
     +                ,bsi(i,1),bsi(i,2),bsi(i,3),prj
        write (8,100) i,p(i,1),p(i,2),p(i,3)
     +                ,bsi(i,1),bsi(i,2),bsi(i,3),prj
       End Do
 
       error = sqrt(error) /(3.0*npts)

       close (8)
       stop

c----------------------------------
      end if    ! end of menu = 4,5
c----------------------------------

c-----------
c Simulation
c-----------

c-----------
c Pozrikidis
c-----------

      If(IPBZ.eq.1) then

      call vs_3d_bsi
     +
     +   (npts,nelm
     +   ,mint,NGL
     +   ,Iflow
     +   ,bsi
     +   )
 
      end if

c---------------------------------
c Baker' generalized vortex method
c---------------------------------

      if(IPBZ.eq.2) then

       call lslp3_3d
     +
     +     (npts,nelm
     +     ,mint,NGL
     +     ,zeta
     +     ,Iflow
     +     ,ptl
     +     )
 
       call vs_3d_curl
     +
     +   (npts,nelm
     +   ,ptl
     +   ,curl
     +   )

       call ldlp_3d
     +
     +   (npts,nelm
     +   ,mint
     +   ,circ
     +   ,hptl
     +   )
 
       call sgrad_3d
     +
     +   (npts,nelm
     +   ,hptl
     +   ,gradf
     +   )

       Do i=1,npts
         bsi(i,1) = curl(i,1)+gradf(i,1)
         bsi(i,2) = curl(i,2)+gradf(i,2)
         bsi(i,3) = curl(i,3)+gradf(i,3)
       End Do

      end if

c----------
c Zinchenko
c----------

      if(IPBZ.eq.3) then

      call vs_3d_bsi_zin
     +
     +    (npts,nelm
     +    ,mint,NGL
     +    ,Iflow
     +    ,bsi
     +    )
 
      end if

c--------------------------
c Iflow =1:
c Add the incident velocity
c along the x axis
c-------------------------

      if(Iflow.eq.1) then

       Do i=1,npts
        bsi(i,1) = bsi(i,1)+1.0
       End Do

      end if

c----------------------
c print and display all
c----------------------
 
c     write (6,*)
c     write (6,*)  "PV of the Biot-Savart integral"
c     write (6,*)
c
c     write (3,*) npts,time(Kstep)
c     write (6,*) npts,time(Kstep)
c
c     Do i=1,npts
c      write (6,100) i 
c    +              ,p(i,1),p(i,2),p(i,3)
c    +              ,bsi(i,1),bsi(i,2),bsi(i,3)
c    +              ,zeta(i,1),zeta(i,2),zeta(i,3)
c    +              ,du(i,1),du(i,2),du(i,3)
c      write (3,100) i 
c    +              ,p(i,1),p(i,2),p(i,3)
c    +              ,bsi(i,1),bsi(i,2),bsi(i,3)
c    +              ,zeta(i,1),zeta(i,2),zeta(i,3)
c    +              ,du(i,1),du(i,2),du(i,3)
c     End Do
 
c-----------------------------
c Kinetic energy:
c
c use formula (2.1.17) on p.44
c of Pozrikidis (1997)
c-----------------------------

      Do i=1,npts
       vnrm = vna(i,1)*bsi(i,1)
     +       +vna(i,2)*bsi(i,2)
     +       +vna(i,3)*bsi(i,3) ! normal velocity
       aux_s(i) = circ(i)*vnrm  ! auxilliary scalar
      End Do

      call srf_int_3d
     +
     +  (nelm,npts,mint
     +  ,aux_s
     +  ,en_int          ! energy integral
     +  )

      en_int = - 0.5*rho*en_int

      enrg_knt(Kstep) = en_int/(pi4*req**2)        ! normalize

c-------------
c total energy
c-------------

      enrg_tot(Kstep) = enrg_srf(Kstep)+enrg_knt(Kstep)

c-------------------------
c compute the maximum axes
c-------------------------

      dimx = 0.0
      dimy = 0.0
      dimz = 0.0

      Do i=1,npts
        xxxx = abs(p(i,1))
        yyyy = abs(p(i,2))
        zzzz = abs(p(i,3))
        if(xxxx.gt.dimx) dimx = xxxx
        if(yyyy.gt.dimy) dimy = yyyy
        if(zzzz.gt.dimz) dimz = zzzz
      End Do

      dimx = dimx/req
      dimy = dimy/req
      dimz = dimz/req

c------------------
c recording session
c------------------

      write (6,*) " time, volume, energy (s, k, t), dimx, dimy, dimz:"
      write (6,*)

      write (6,100) Kstep,time(Kstep),volume(Kstep)
     +               ,enrg_srf(Kstep),enrg_knt(Kstep)
     +               ,enrg_tot(Kstep)
     +               ,dimx,dimy,dimz

      write (7,100) Kstep,time(Kstep),volume(Kstep)
     +               ,enrg_srf(Kstep),enrg_knt(Kstep)
     +               ,enrg_tot(Kstep)
     +               ,dimx,dimy,dimz

c--------------------
c print cross section
c--------------------

      if(Isym_xy.eq.1) then
      if(Iprint_xy.eq.Nprint_xy) then

        Iprint_xy = 0

        write (1,100) nxy,time(Kstep)
        write (6,100) nxy,time(Kstep)
  
c-----
      if(Method.eq.1) then
c-----
        Do i = 1,nxy
          m = jxy(i)
          write (1,100) i,(p(m,j),j=1,2),circ(m)
c    +                   ,(zeta(m,j),j=1,2)
c    +                   ,(bsi(m,j),j=1,2)
c    +                   ,(vna(m,j),j=1,2)
c         write (6,100) i,(p(m,j),j=1,2),circ(m)
c    +                   ,(zeta(m,j),j=1,2)
c    +                   ,(bsi(m,j),j=1,2)
c    +                   ,(vna(m,j),j=1,2)
        End Do
c-----
      else if(Method.eq.2) then
c-----
        Do i = 1,nxy
          m = jxy(i)
          write (1,100) i,(p(m,j),j=1,2),(zeta(m,j),j=1,2)
c    +                   ,(bsi(m,j),j=1,2)
c    +                   ,(vna(m,j),j=1,2)
c         write (6,100) i,(p(m,j),j=1,2),zeta(m,3)  ! component in z
c    +                   ,(bsi(m,j),j=1,2)
c    +                   ,(vna(m,j),j=1,2)
        End Do
c-----
      End If
c-----

      end if
      end if

c----------------
c print all nodes
c----------------

      if(Iprint_xyz.eq.Nprint_xyz) then

        Iprint_xyz = 0

        write (3,100) npts,time(Kstep)

c-----
      if(Method.eq.1) then
c-----
         Do i=1,npts
           write (3,108) i,p(i,1),p(i,2),p(i,3),circ(i)
         End Do
c-----
      else if(Method.eq.2) then
c-----
         Do i=1,npts
           write (3,108) i,(p(i,j),j=1,3),(zeta(i,j),j=1,3)
         End Do
c-----
      End If
c-----

      end if

c-------------------------
      If(srtn.gt.0) then

c---
c compute the mean curvature at the nodes
c---

      call crvm_3d (nelm,npts)

      end if
c-------------------------


c-----------------------------
c Integrate evolution equation
c for the circulation
c for equal densities
c-----------------------------

      if(method.eq.1) then

       fc = 2.0*srtn/rho

       Do i=1,npts
        dcircdt(i) = fc*(crvm(i)-1.0/req)
       End Do

c---
c If points move with the normal velocity
c
c compute the gradient of the circulation
c and then compute the tangential derivative
c in the direction of (I-nn).bsi
c---

       if(Move.eq.1) then 

       call sgrad_3d
     +
     +   (npts,nelm
     +   ,circ
     +   ,gradf
     +   )

       Do i=1,npts

        prj = vna(i,1)*bsi(i,1)+vna(i,2)*bsi(i,2)+vna(i,3)*bsi(i,3)

        tngder = (bsi(i,1) - prj*vna(i,1)) * gradf(i,1)
     +         + (bsi(i,2) - prj*vna(i,2)) * gradf(i,2)
     +         + (bsi(i,3) - prj*vna(i,3)) * gradf(i,3)

        dcircdt(i) = dcircdt(i) - tngder

       End Do

       end if

c-----------
      end if
c-----------

c---------------------------
c evolution equation for Du
c--------------------------

      if(method.eq.2) then

c----
c compute the tangential derivative
c of the bsi in the direction of Du
c
c and then project it onto (-I+2nn)
c----

      call sgrad3_3d
     +
     +  (npts,nelm
     +  ,bsi
     +  ,dfxx,dfyx,dfzx
     +  ,dfxy,dfyy,dfzy
     +  ,dfxz,dfyz,dfzz
     +  )

      Do i=1,npts

       dbsix = Du(i,1)*dfxx(i) + Du(i,2)*dfyx(i) + Du(i,3)*dfzx(i)
       dbsiy = Du(i,1)*dfxy(i) + Du(i,2)*dfyy(i) + Du(i,3)*dfzy(i)
       dbsiz = Du(i,1)*dfxz(i) + Du(i,2)*dfyz(i) + Du(i,3)*dfzz(i)

       prj = vna(i,1)*dbsix + vna(i,2)*dbsiy + vna(i,3)*dbsiz

       ddudt(i,1) = - dbsix + 2.0*prj*vna(i,1)
       ddudt(i,2) = - dbsiy + 2.0*prj*vna(i,2)
       ddudt(i,3) = - dbsiz + 2.0*prj*vna(i,3)

      End Do
   
c------------
c Points move
c with the normal velocity
c
c compute the tangential derivative of Du
c in the direction of the tangential
c component of (I-nn)*bsi
c------------

      if(Move.eq.1) then

       call sgrad3_3d
     +
     +   (npts,nelm
     +   ,du
     +   ,dfxx,dfyx,dfzx
     +   ,dfxy,dfyy,dfzy
     +   ,dfxz,dfyz,dfzz
     +   )

       Do i=1,npts

        prj = vna(i,1)*bsi(i,1)+vna(i,2)*bsi(i,2)+vna(i,3)*bsi(i,3)

        tnvx = bsi(i,1) - prj*vna(i,1)
        tnvy = bsi(i,2) - prj*vna(i,2)
        tnvz = bsi(i,3) - prj*vna(i,3)

        ddux = tnvx*dfxx(i) + tnvy*dfyx(i) + tnvz*dfzx(i)
        dduy = tnvx*dfxy(i) + tnvy*dfyy(i) + tnvz*dfzy(i)
        dduz = tnvx*dfxz(i) + tnvy*dfyz(i) + tnvz*dfzz(i)

        ddudt(i,1) = ddudt(i,1)-ddux
        ddudt(i,2) = ddudt(i,2)-dduy
        ddudt(i,3) = ddudt(i,3)-dduz

       End Do

c---
      end if
c---

c-----------
      end if
c-----------

c-----------------------------------------------
c Integrate in time by the explicit Euler method
c-----------------------------------------------

      Do i=1,npts

        if(Move.eq.0) then      ! points move with total velocity

          Umove = bsi(i,1)
          Vmove = bsi(i,2)
          Wmove = bsi(i,3)

        else if(Move.eq.1) then  ! points move with normal velocity

          Uvel = bsi(i,1)*vna(i,1)+ bsi(i,2)*vna(i,2)
     +         + bsi(i,3)*vna(i,3)

          Umove = Uvel*vna(i,1)
          Vmove = Uvel*vna(i,2)
          Wmove = Uvel*vna(i,3)

        end if

c--------------------------------------
c save position and circulation for RK2
c--------------------------------------

        if(IRK.eq.2) then
               ps(i,1) =    p(i,1)
               ps(i,2) =    p(i,2)
               ps(i,3) =    p(i,3)
            circs(i)   = circ(i)
               us(i,1) = Umove
               us(i,2) = Vmove
               us(i,3) = Wmove
         dcircdts(i)   = dcircdt(i)
        end if

c-----------------
c advance position
c-----------------

        p(i,1) = p(i,1) + Dt*Umove   ! advance
        p(i,2) = p(i,2) + Dt*Vmove
        p(i,3) = p(i,3) + Dt*Wmove

c---
c advance circulation or Du
c---

        if(Method.eq.1) then
         circ(i) = circ(i) + Dt*dcircdt(i)
        else
         du(i,1) = du(i,1) + Dt*ddudt(i,1)
         du(i,2) = du(i,2) + Dt*ddudt(i,2)
         du(i,3) = du(i,3) + Dt*ddudt(i,3)
        end if

      End Do

c-------------------------
c compute the new geometry
c-------------------------

      Do k=1,nelm

       i1 = n(k,1)
       i2 = n(k,2)
       i3 = n(k,3)
       i4 = n(k,4)
       i5 = n(k,5)
       i6 = n(k,6)

       call abc 
     +
     +  (p(i1,1),p(i1,2),p(i1,3)
     +  ,p(i2,1),p(i2,2),p(i2,3)
     +  ,p(i3,1),p(i3,2),p(i3,3)
     +  ,p(i4,1),p(i4,2),p(i4,3)
     +  ,p(i5,1),p(i5,2),p(i5,3)
     +  ,p(i6,1),p(i6,2),p(i6,3)
     +  ,alpha(k),beta(k),gamma(k)
     +  )

      End Do

      call elm_geom 
     +
     +    (nelm,npts,mint
     +    ,xmom,ymom,zmom
     +    ,area,vlm
     +    ,cx,cy,cz
     +    )

c---------------------
c Extract zeta from Du
c---------------------

      if(Method.eq.2) then

c-----
c Project Du onto the tangential plane
c-----

       Do i=1,npts
        prj = du(i,1)*vna(i,1) + du(i,2)*vna(i,2)
     +                         + du(i,3)*vna(i,3)
        du(i,1) = du(i,1)-vna(i,1)*prj
        du(i,2) = du(i,2)-vna(i,2)*prj
        du(i,3) = du(i,3)-vna(i,3)*prj
c       write (6,100) i,du(i,1),du(i,2),du(i,3)
       End Do

c----
c recover zeta
c----

       Do i=1,npts
        zeta(i,1) = vna(i,2)*du(i,3)-vna(i,3)*du(i,2)
        zeta(i,2) = vna(i,3)*du(i,1)-vna(i,1)*du(i,3)
        zeta(i,3) = vna(i,1)*du(i,2)-vna(i,2)*du(i,1)
c       write (6,100) i,zeta(i,1),zeta(i,2),zeta(i,3)
       End Do

c----
c Project the vorticity onto the tangential plane
c----

       Do i=1,npts
        prj = zeta(i,1)*vna(i,1) + zeta(i,2)*vna(i,2)
     +                           + zeta(i,3)*vna(i,3)
        zeta(i,1) = zeta(i,1)-vna(i,1)*prj
        zeta(i,2) = zeta(i,2)-vna(i,2)*prj
        zeta(i,3) = zeta(i,3)-vna(i,3)*prj
       End Do

      End If

c----------------
c RK2: Second-order RK
c
c implemented only for
c limited options
c----------------

      if(IRK.eq.1) Go to 888
      if(method.eq.2) Go to 888
      if(Imove.eq.1) Go to 888

      call vs_3d_circ_vort
     +
     +   (npts,nelm
     +   ,circ
     +   ,zeta
     +   )

c---
      if(IPBZ.eq.1) then

      call vs_3d_bsi
     +
     +  (npts,nelm
     +  ,mint,NGL
     +  ,Iflow
     +  ,bsi
     +  )
 
      end if

c---
      if(IPBZ.eq.2) then

       call lslp3_3d
     +
     +  (npts,nelm
     +  ,mint,NGL
     +  ,zeta
     +  ,Iflow
     +  ,ptl
     +  )
 
       call vs_3d_curl
     +
     +   (npts,nelm
     +   ,ptl
     +   ,curl
     +   )

       call ldlp_3d
     +
     +  (npts,nelm
     +  ,mint
     +  ,circ
     +  ,hptl
     +  )
 
       call sgrad_3d
     +
     +   (npts,nelm
     +   ,hptl
     +   ,gradf
     +   )

       Do i=1,npts
         bsi(i,1) = curl(i,1)+gradf(i,1)
         bsi(i,2) = curl(i,2)+gradf(i,2)
         bsi(i,3) = curl(i,3)+gradf(i,3)
       End Do

      End If
c---
      If(IPBZ.eq.3) then

      call vs_3d_bsi_zin
     +
     +   (npts,nelm
     +   ,mint,NGL
     +   ,Iflow
     +   ,bsi
     +   )
 
      end if
c---

      if(Iflow.eq.1) then

      Do i=1,npts
       bsi(i,1) = bsi(i,1)+1.0
      End Do

      End If

c---

      if(srtn.gt.0) then
       call crvm_3d (nelm,npts)
       fc = 2.0*srtn/rho
       Do i=1,npts
        dcircdt(i) = fc*(crvm(i)-1.0/req)
       End Do
      end if

c---
      Dth = 0.50*Dt

      Do i=1,npts
         Umove = bsi(i,1)
         Vmove = bsi(i,2)
         Wmove = bsi(i,3)
         p(i,1) = ps(i,1) + Dth*(bsi(i,1)+us(i,1))
         p(i,2) = ps(i,2) + Dth*(bsi(i,2)+us(i,2))
         p(i,3) = ps(i,3) + Dth*(bsi(i,3)+us(i,3))
         circ(i) = circs(i) + Dth*(dcircdt(i)+dcircdts(i))
       End Do

  888 Continue


c-----------------
c SMOOTHING MODULE
c-----------------

      if(Ism.eq.Nsm) then

       Ism = 0

       write (6,*)
       write (6,*) " Smoothing"
       write (6,*)

c---
c smoothing the circulation
c---

       Do i=1,npts
         f_sm(i) = circ(i)
       End Do

       Do i=1,npts
        circ(i) = 0.0
        Do j=1,npts
         circ(i) = circ(i) + smm(i,j)*f_sm(j)
        End Do
       End Do

c---
c smoothing x
c---

      if(Ism_opt.eq.2) then

       Do i=1,npts
         f_sm(i) = p(i,1)
       End Do

       Do i=1,npts
         p(i,1) = 0.0
         Do j=1,npts
           p(i,1) = p(i,1) + smm(i,j)*f_sm(j)
         End Do
       End Do

      end if

c---
c smoothing y
c---

      if(Ism_opt.eq.2) then

       Do i=1,npts
         f_sm(i) = p(i,2)
       End Do

       Do i=1,npts
         p(i,2) = 0.0
         Do j=1,npts
           p(i,2) = p(i,2) + smm(i,j)*f_sm(j)
         End Do
       End Do

      end if

c---
c smoothing z
c---

      if(Ism_opt.eq.2) then

       Do i=1,npts
         f_sm(i) = p(i,3)
       End Do

       Do i=1,npts
         p(i,3) = 0.0
         Do j=1,npts
           p(i,3) = p(i,3) + smm(i,j)*f_sm(j)
         End Do
       End Do

       end if

      write (6,*)
      write (6,*) " Done smoothing"
      write (6,*)

      end if

c-----------------
c end of smoothing
c-----------------

c------------------------
c reset counters and time
c------------------------

      if(Kstep.eq.Nstep) Go to 99

      Kstep = Kstep+1
      Istep = Istep+1
      Ism   = Ism  +1      ! for smoothing

      Iprint_xy  = Iprint_xy+1
      Iprint_xyz = Iprint_xyz+1

      time(Istep) = time(Istep-1) + Dt

      Go to 90     ! return for another step

c-------------------------
c THE SIMULATION HAS ENDED
c-------------------------

  99  Continue

c------------------
c print final shape
c------------------

      write (1,100) nxy,time(Istep)
      write (3,100) npts,time(Istep)

c-----
      if(Method.eq.1) then
c-----

        if(Isym_xy.eq.1) then
         Do i = 1,nxy
          m = jxy(i)
          write (1,100) i,(p(m,j),j=1,2),circ(m)
c    +                   ,(zeta(m,j),j=1,2)
c    +                   ,(bsi(m,j),j=1,2)
c    +                   ,(vna(m,j),j=1,2)
         End Do
        end if

        Do i=1,npts
          write (3,108) i,p(i,1),p(i,2),p(i,3),circ(i)
        End Do

c-----
      else if(Method.eq.2) then
c-----

        if(Isym_xy.eq.1) then
         Do i = 1,nxy
          m = jxy(i)
          write (1,100) i,(p(m,j),j=1,2),(zeta(m,j),j=1,2)
c    +                   ,(zeta(m,j),j=1,2)
c    +                   ,(bsi(m,j),j=1,2)
c    +                   ,(vna(m,j),j=1,2)
         End Do
        End If

        Do i=1,npts
          write (3,108) i,(p(i,j),j=1,3)
     +                   ,(zeta(i,j),j=1,3)
        End Do

c-----------
      end if
c-----------

c--------------------------
c Generate a restart file
c--------------------------

      write (4,100) npts,time(Istep)

c-----
      if(Method.eq.1) then
c-----
         Do i=1,npts
           write (4,108) i,p(i,1),p(i,2),p(i,3),circ(i)
         End Do
c-----
      else if(Method.eq.2) then
c-----
         Do i=1,npts
           write (4,108) i,p(i,1),p(i,2),p(i,3)
     +              ,zeta(i,1),zeta(i,2),zeta(i,3)
         End Do

      end if
c-----

c----------
c finish up
c----------

      write (1,*) null

      write (1,200) ndiv 
      write (1,201) boa,coa
      write (1,202) req
      write (1,203) cx,cy,cz
      write (1,204) mint
      write (1,205) NGL
      write (1,206) Move
      write (1,207) Method,IPBZ
      write (1,208) Nprint_xy
      write (1,209) Nprint_xyz
      write (1,210) Iread
      write (1,211) Nsm,Ism_opt,Ism_init
      write (1,212) eps_sm
      write (1,213) dels
      write (1,214) Dt
      write (1,215) rho
      write (1,216) srtn
      write (1,220) Isym_xy
      write (1,221) IRK

      write (3,*) null

      write (3,200) ndiv 
      write (3,201) boa,coa
      write (3,202) req
      write (3,203) cx,cy,cz
      write (3,204) mint
      write (3,205) NGL
      write (3,206) Move
      write (3,207) Method,IPBZ
      write (3,208) Nprint_xy
      write (3,209) Nprint_xyz
      write (3,210) Iread
      write (3,211) Nsm,Ism_opt,Ism_init
      write (3,212) eps_sm
      write (3,213) dels
      write (3,214) Dt
      write (3,215) rho
      write (3,216) srtn
      write (3,220) Isym_xy
      write (3,221) IRK

c------------
c close files
c------------

      close (1)
      close (3)
      close (4)
      close (7)

  100 Format(1x,i4,100(1x,f10.6))
  108 Format(1x,i4,100(1x,f15.10))
  109 Format(1x,i4,100(1x,f7.4))
  101 Format(10(1x,f12.5))

  110 Format(" Surface Area :",F15.10)
  111 Format(" Volume       :",F15.10)

  200 Format(" Ndiv        = ",i1)
  201 Format(" axis ratios = ",f10.5,1x,f10.5)
  202 Format(" eq radius   = ",f10.5)
  203 Format(" centroid    = ",f10.5,1x,f10.5,1x,f10.5)
  204 Format(" mint        = ",i1)
  205 Format(" Ngl         = ",i2)
  206 Format(" Move        = ",i1)
  207 Format(" Method      = ",i1,/
     +      ," IPBZ        = ",i1)
  208 Format(" Nprint_xy   = ",i2)
  209 Format(" Nprint_xyz  = ",i2)
  210 Format(" Iread       = ",i1)
  211 Format(" Nsm         = ",i4,/
     +      ," Ism_opt     = ",i1
     +      ," Ism_init    = ",i1)
  212 Format(" eps_sm      = ",f15.10)
  213 Format(" delta**2    = ",f15.10)
  214 Format(" Dt          = ",f15.10)
  215 Format(" Density     = ",f15.10)
  216 Format(" Surface ten = ",f15.10)
  220 Format(" Isym_xy     = ",i1)
  221 Format(" IRK         = ",i1)

  567 Format(" Error: ",f15.10)

  800 Format(" centroid:",3(1x,f10.5))

      stop
      end
