      subroutine vs_3d_bsi_integrate_flat_sing
     +
     +      (NGL,Iflow
     +      ,zetx0,zety0,zetz0
     +      ,vnx0,vny0,vnz0
     +      ,x1,y1,z1
     +      ,x2,y2,z2
     +      ,x3,y3,z3
     +      ,vnx1,vny1,vnz1
     +      ,vnx2,vny2,vnz2
     +      ,vnx3,vny3,vnz3
     +      ,ztx1,zty1,ztz1
     +      ,ztx2,zty2,ztz2
     +      ,ztx3,zty3,ztz3
     +      ,crvm_cntr
     +      ,uux,uuy,uuz
     +      ,area
     +      )

c===========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c===========================================

c--------------------------------------------------------
c Compute the modified Biot-Savart integral over
c a linear (flat) triangle defined by three points 1-2-3
c using local polar coordinates with origin at point 1
c
c SYMBOLS:
c -------
c
c asm: triangle area computed by numerical integration
c-------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension zz(20),ww(20)

      common/zwl/zz,ww
      common/pii/pi,piq,pih,pi2,pi4,pi6,pi8

c---
c flags
c---

c     write (6,*) "mean curvature at element centroid",crvm_cntr

      Iopt_lgf = 2    ! need G and gradient

c---
c compute triangle area,
c         surface metric
c         normal vector
c---

      dx = sqrt( (x2-x1)**2+(y2-y1)**2+(z2-z1)**2 )
      dy = sqrt( (x3-x1)**2+(y3-y1)**2+(z3-z1)**2 )

      vnx = (y2-y1)*(z3-z1) - (z2-z1)*(y3-y1)
      vny = (z2-z1)*(x3-x1) - (x2-x1)*(z3-z1)
      vnz = (x2-x1)*(y3-y1) - (y2-y1)*(x3-x1)

      area = 0.5*sqrt( vnx**2+vny**2+vnz**2 )
      hs   = 2.0*area

      vnx = vnx/hs
      vny = vny/hs
      vnz = vnz/hs

c---
c initialize
c---

      asm = 0.0D0

      uux = 0.0D0
      uuy = 0.0D0
      uuz = 0.0D0

c---
c Double Gaussian quadrature
c---

      Do i=1,NGL

       ph    = piq*(1.0D0+zz(i))
       cph   = cos(ph)
       sph   = sin(ph)

       rmax  = 1.0D0/(cph+sph)
       rmaxh = 0.5D0*rmax

       bsm = 0.0D0

       bix = 0.0D0
       biy = 0.0D0
       biz = 0.0D0

       Do j=1,NGL

        r = rmaxh*(1.0D0+zz(j))

        xi = r*cph
        et = r*sph
        zt = 1.0D0-xi-et

        x = x1*zt + x2*xi + x3*et
        y = y1*zt + y2*xi + y3*et
        z = z1*zt + z2*xi + z3*et

        zetx = ztx1*zt + ztx2*xi + ztx3*et
        zety = zty1*zt + zty2*xi + zty3*et
        zetz = ztz1*zt + ztz2*xi + ztz3*et

        call lgf_3d_fs 
     +
     +    (Iopt_lgf
     +    ,x,y,z
     +    ,x1,y1,z1
     +    ,G
     +    ,Gx,Gy,Gz
     +    )

c------------
c Integral I1
c------------

        zzx = zetx-zetx0
        zzy = zety-zety0
        zzz = zetz-zetz0

        rintx = zzy*Gz - zzz*Gy 
        rinty = zzz*Gx - zzx*Gz 
        rintz = zzx*Gy - zzy*Gx 

c-----------
c integral I3
c-----------

        vnx = vnx1*zt + vnx2*xi + vnx3*et
        vny = vny1*zt + vny2*xi + vny3*et
        vnz = vnz1*zt + vnz2*xi + vnz3*et

        tmp = 2.0D0*G*crvm_cntr

        rintx = rintx + (zety0*vnz-zetz0*vny)*tmp
        rinty = rinty + (zetz0*vnx-zetx0*vnz)*tmp
        rintz = rintz + (zetx0*vny-zety0*vnx)*tmp

c---
c finish up the quadrature
c---

        cf = r*ww(j)

        bsm = bsm+cf

        bix = bix + rintx*cf
        biy = biy + rinty*cf
        biz = biz + rintz*cf

       End Do

       cf = ww(i)*rmaxh

       asm = asm + bsm*cf

       uux = uux + bix*cf
       uuy = uuy + biy*cf
       uuz = uuz + biz*cf

      End Do

      cf = piq*hs

      uux = uux*cf
      uuy = uuy*cf
      uuz = uuz*cf

      asm = asm*cf

c-----------------------------
c  if all went well,
c  asm should be equal to area
c
c     write (6,100) i,area,asm
c-----------------------------

 100  Format (1x,i3,2(f10.5))

      return
      end
