close all
clear all

%===
% Boundary layer due to a stretching sheet
%
% will solve the boundary-value problem
% with newton iterations
%
% will solve an extended system of six ODEs 
%===

global m

figure(1)
hold on

%---
% initial guess
%----

 options = odeset('RelTol',1e-8,'AbsTol',1e-08);

 y0 = [0.0 1.0 -0.1 0.0 0.0 1.0];

%---
for repeat=-3:12
%---

if(repeat==0)
 m = -0.10; tspan = [0, 9.0];
 y0 = [0.0 1.0 -0.443748 0.0 0.0 1.0];
elseif(repeat==-1)
 m = -0.20; tspan = [0, 9.0];
elseif(repeat==-2)
 m = -0.28; tspan = [0, 8.0];
elseif(repeat==-3)
 m = -0.33; tspan = [0, 8.0];
elseif(repeat==1)
 m = 0.0; tspan = [0, 10.0];
 y0 = [0.0 1.0 -0.443748 0.0 0.0 1.0];
elseif(repeat==2)
 m = 0.2; tspan = [0, 9.0];
elseif(repeat==3)
 m = 0.2; tspan = [0, 9.0];
elseif(repeat==4)
 m = 0.4; tspan = [0, 9.0];
elseif(repeat==5)
 m = 0.4; tspan = [0, 9.0];
elseif(repeat==6)
 m = 0.6; tspan = [0, 8.0];
elseif(repeat==7)
 m = 0.8; tspan = [0, 8.0];
elseif(repeat==8)
 m = 1.0; tspan = [0, 8.0];
elseif(repeat==9)
 m = 1.2; tspan = [0, 8.0];
elseif(repeat==10)
 m = 1.4; tspan = [0, 7.0];
elseif(repeat==11)
 m = 1.6; tspan = [0, 7.0];
elseif(repeat==12)
 m = 1.8; tspan = [0, 7.0];
end

%---
% iterate
%---

 for iter=1:10

  [t, y]= ode23(@crane_fnc,tspan,y0,options);
  NN = size(y);
  N  = NN(1);
  residual = y(N,2)-0.0;
  y0 = [0.0; ...
        1.0; ...
        y0(3)-residual/y(N,5);...
        0.0; ...
        0.0; ...
        1.0];

  if(abs(residual)<0.00000001) break; end

 end

%---
% converged solution
%---

vel = y(:,2);
nsize = size(vel);
n = nsize(1);

%---
% compute the coefficient alpha
% for the displacement thickness
%---

alpha1 = y(n,1);
alpha2 = y(n-30,1);

%---
% compute the coefficient beta
% for the momentum thickness
% by the trapezoidal rule
%---

beta = 0.0;

for i=1:n-1
 v1 = vel(i)^2;
 v2 = vel(i+1)^2;
 beta = beta + (t(i+1)-t(i))*0.5*(v1+v2);
end

%---
% now record
%---

format long
y0(3)
mplot(repeat+4) = m;
yplot(repeat+4) = -y0(3);
a1plt(repeat+4) = alpha1;
a2plt(repeat+4) = alpha2;
b1plt(repeat+4) = beta;
lambd(repeat+4) = m*beta^2;
format short

figure(1)

if(abs(m-1.8)<0.000001)
 plot(y(:,2),t,'k:')
elseif(abs(m-0.0)<0.000001)
 plot(y(:,2),t,'k--')
elseif(abs(m-1.0)<0.000001)
 plot(y(:,2),t,'k-.')
else
 plot(y(:,2),t,'k')
end

%==========
end % of repeat
%==========

%---
% plot the solution
%---

for i=1:32
 cranex(i) = (i-1.0)*0.1;
 craney(i) = exp(-cranex(i));
end

figure(1)
%plot(craney,cranex,'rx')
xlabel('u/U','fontsize',13)
ylabel('\eta','fontsize',13)
set(gca,'fontsize',13)
axis([0, 1.1, 0, 8])
box

figure(2)
hold on
plot(mplot,yplot,'k-')
plot(mplot,a1plt,'k--')
plot(mplot,b1plt,'k:')
%plot(mplot,a2plt,'k--')
xlabel('m','fontsize',13)
ylabel('-x_3(0),  \alpha,  \beta','fontsize',13)
set(gca,'fontsize',13)
box

%----------------
% compute and plot
% the Holstein-Bohlen parameter lambda
%----------------

nsize = size(mplot);
n = nsize(2);

lpohl = -0.1;

for i=4:n
 m = mplot(i);
 [Lpohl,lpohl] = crane_pohl(m,lpohl);
 holbo1(i) = lpohl;
 holbo2(i) = Lpohl;
end

figure(3)
hold on
plot(mplot,lambd,'k-')
plot(mplot(4:n),holbo1(4:n),'k--')
%plot(mplot,holbo2,'k--')
plot([-0.2 2.0],[0, 0],'b:')
plot([0 0],[-0.1 0.15],'b:')
xlabel('m','fontsize',13)
ylabel('\lambda','fontsize',13)
set(gca,'fontsize',13)
axis([-0.5 2 -0.9 0.35])
box on


