      program falskan

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c-------------------------------------------------------
c Solution of the Falkner--Skan boundary-layer equations
c
c Legend:
c
c IRK    Runge-Kutta selector (1 or 2)
c Tmax   Max integration limit
c Nstep  Max steps
c N      Number of equations
c
c sol(i,j)   jth component of the ith function
c-------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension x(3),u(3),xs(3),us(3)
      Dimension xinit(3),sol(3,1000)

      Parameter (tol=0.00000001)

      common/fs/rm

c---
c constants
c---

      Null = 0

c---
c select the problem
c---

      write (6,*) 
      write (6,*) 'Enter 1 for the Falkner-Skan equation'
      write (6,*) '      0 to quit'
      write (6,*) '---------------'
      read  (5,*) menu

      If(menu.eq.0) Go to 99

c---
c number of ODEs
c---

      N=3

 97   Continue   ! return to repeat

c-----------
c initialize
c-----------

      Do j=1,N
        xinit(j) = 0.0
        x(j)     = xinit(j)
      End do

c---
c parameters
c---

      open (1,file="falskan.dat")

       read (1,*) IRK
       read (1,*) Tmax
       read (1,*) Nstep
       read (1,*)

      close (1)

c---
c initial conditions
c---

      If(menu.eq.1) then

         write (6,*) 
         write (6,*) " Please enter the malue of m"
         write (6,*) 
         write (6,*) " Choose from  0.0 (Blasius)"
         write (6,*) "             -0.0904"
         write (6,*) "              0.176"
         write (6,*) "              0.25"
         write (6,*) "              1.0 (orthogonal - exact)"
         write (6,*) "              1.5"
         write (6,*) "--------------------------------------" 
         read (5,*) rm
c        read (1,*) xinit(3)

         write (6,*) "The initial condition will be chosen wisely" 

         if(abs(rm+0.0904).lt.tol) then
          xinit(3) = 0.0

         elseif(abs(rm).lt.tol) then
          xinit(3) = 0.332

         elseif(abs(rm-0.176).lt.tol) then
          xinit(3) = 0.5936

         elseif(abs(rm-0.250).lt.tol) then
          xinit(3) = 0.675

         Else If(abs(rm-1.0).lt.tol) then
          xinit(3) = 1.2322

         elseif(abs(rm-1.5).lt.tol) then
          xinit(3) = 1.493

         else
          xinit(3) = 0.4

         end if

         x(3) = xinit(3)

      End If

c---------------
c prepare to run
c---------------

      Nstep1 = Nstep+1
      dt = Tmax/(Nstep1-1.0)

c---
c begin the shootings
c---

      open (2,file="falskan.out")

  98  Continue 

      i    = 1
      time = 0.0

c---
c print initial conditions
c---

      write (6,100) Nstep1
      write (2,100) Nstep1
      write (6,100) i,time,(x(j),j=1,n)
      write (2,100) i,time,(x(j),j=1,n)

      Do j=1,N
       sol(j,i) = x(j)
      End Do

c--------------------------------
c integrate odes
c using the explicit Euler method
c--------------------------------

      Do 2 i=2,Nstep1

       call falskan_ode (menu,N,x,u)

       If(IRK.eq.2) then
          Do j=1,N
            xs(j) = x(j)
            us(j) = u(j)
             x(j) = x(j) + u(j)*dt
          End Do
          call falskan_ode (menu,N,x,u)
          Do j=1,N
            u(j) = 0.5*(us(j)+u(j))
          End Do
       End If 

       Do j=1,N
         x(j)     = xs(j) + u(j)*dt
         sol(j,i) = x(j)
       End Do

       time = time + dt

       write (6,100) i,time,(x(j),j=1,n)
       write (2,100) i,Time,(x(j),j=1,n)

   2  Continue

c-----------------------
c new initial conditions
c-----------------------

      write (6,*) ' Try a different initial condition ?'
      write (6,*) ' 0 for no, 1 for yes'
      write (6,*) ' -------------------'
      read  (5,*) Imore

      if(Imore.eq.1) then

         write (6,*) " Old initial values were:"

         Do i=1,N
           write (6,100) i,xinit(i)
         End Do

         Do j=1,N
           xinit(j) = 0.0
               x(j) = xinit(j)
         End Do

         if(menu.eq.1) then
          write (6,*) " Please enter new value for xinitial(3)"
          read  (5,*) xinit(3)
          x(3) = xinit(3)
         end If

         Go to 98

      End If

      Go to 97

c-----
c done
c-----

   99 Continue

      write (2,100) Null

      close (2)

  100 Format (1x,i3,10(1x,f9.5))

      Stop 
      End
