function [Lam,lam] = fskan_pohl(m,lam)

%--------
% find the root of the function:
% F = 2mT+(m-1)*lam = 0
% determining the Holstein-Bohlen parameter
%--------

tol = 0.000001;
eps = 0.000001;

  Ic=0;
  for i=1:50
    fnc  = pscan(m,lam);
    fnc1 = pscan(m,lam+eps);
    fncp = (fnc1-fnc)/eps;
    corr = -fnc/fncp;
    lam  = lam+corr;
    Ic=Ic+1;
    if(abs(corr)<tol)
      Lam = -6.0;
      Lam = solveforL(lam,Lam);
      return; end
   end

   disp 'fskan_pohl: could not find a root in 50 iterations'
   pause

%---
% done
%---

return

%================================

function F = pscan(m,lam)

%---
% compute the function F
%---

Lam = -6.0;
Lam = solveforL(lam,Lam);

tmp = 37.0-Lam/3.0-5.0*Lam*Lam/144.0;
H = 31.5*(3.0-Lam/12.0)/tmp;
S = (2.0+Lam/6.0)*tmp/315.0;
T = S-(2.0+H)*lam;
F = 2*m*T + (m-1)*lam

%---
% done
%---

return

%================================

function Lam = solveforL(lam,Lam)

%---
% solve for Lambda, given lambda
%---

tol = 0.000001;
eps = 0.000001;

 if(abs(lam)<0.000005)
   Lam = 0.00001;
   return
 end

%---
% Newton's method
%---

  Ic=0;
  for i=1:50
    fnc   = ltoL(lam,Lam);
    fnc1  = ltoL(lam,Lam+eps);
    fncp  = (fnc1-fnc)/eps;
    corr  = -fnc/fncp;
    Lam   = Lam+corr;
    Ic=Ic+1;
    if(abs(corr)<tol) return; end
   end

   disp 'solveforL: could not find a root after 50 iterations'

%---
% done
%---

return

%================================

function f = ltoL(lam,Lam)

%---
% lambda to Lambda
%---

  f = Lam*( (37.0-Lam/3.0-5.0*Lam*Lam/144.0)/315.0 )^2 ...
   - lam;

%---
% done
%---

return
