      subroutine pois_gs1
     +
     +  (N,M
     +  ,Dx,Dy
     +  ,source
     +  ,Itermax
     +  ,tol
     +  ,T
     +  )

c-----------------------------------------
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c------------------------------------------
c Gauss-Siedel solution of Poisson's equation
c in a rectangular domain
c with Neumann boundary conditions
c
c Equation is applied at the interior
c AND boundary nodes
c
c Boundary conditions
c involving inward differences:
c ----------------------------
c
c     df/dx =  pnbcl   at x = ax
c     df/dx = -pnbcr   at x = bx
c     df/dy =  pnbcb   at y = ay
c     df/dy = -pnbct   at y = by
c
c SYMBOLS:
c -------
c
c N=Nx:   Grid size in the x direction
c M=Ny:   Grid size in the y direction
c
c------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension     T(0:130,0:130)
      Dimension Tsave(0:130,0:130)

      Dimension pnbcl(129),pnbcr(129)
      Dimension pnbcb(129),pnbct(129)

      Dimension source(16666)

      Parameter (relax=1.0D0)

c--------------
c common blocks
c--------------

      common/pnbc/pnbcl,pnbcr,pnbcb,pnbct

c--------
c prepare
c--------

      Na = N-1
      N1 = N+1
      N2 = N+2

      Ma = M-1
      M1 = M+1
      M2 = M+2

      Nodes = N1*M1     ! total number of nodes

      Dxs   = Dx**2
      Dys   = Dy**2
      beta  = Dxs/Dys
      beta1 = 2.0D0*(beta+1.0D0)

      open (9,file="error.out")

c------------------------
c Gauss-Siedel iterations
c------------------------

      Do 97 Iter=1,Itermax

c---
c Implement the homogeneous Neumann
c boundary condition
c using centered differences
c
c the inhomogeneity is incorporated
c into the RHS (source term)
c---

       Do j=1,M1
         T( 0,j) = T(2,j)
         T(N2,j) = T(N,j)
       End Do

       Do i=1,N1
         T(i, 0) = T(i,2)
         T(i,M2) = T(i,M)
       End Do

c------------------------
c update nodes row-by-row
c------------------------

c---
c save
c---
       Do JL=1,M1
        Do IL=1,N1
          Tsave(IL,JL) = T(IL,JL)
        End Do
       End Do

c---
c iterate
c---

       Ic = 0

       Do JL=1,M1
        Do IL=1,N1
         Ic = Ic+1
         corr = (       T(IL+1,JL)+T(IL-1,JL)
     +           +beta*(T(IL,JL+1)+T(IL,JL-1))
     +           -source(Ic) )/beta1 - T(IL,JL)
         T(IL,JL) = Tsave(IL,JL) + relax*corr
        End Do
       End Do

c-----------------------
c shift to prevent drift
c-----------------------

       Toff = T(N/2+1,M/2+1)
c      Toff = 0.0D0

       Do JL=1,M1
        Do IL=1,N1
         T(IL,JL) = T(IL,JL)-Toff
        End Do
       End Do

c-------------------
c maximum correction
c-------------------

       corrmax = 0.0D0

       Do JL=1,M1
        Do IL=1,N1
         testo = T(IL,JL)-Tsave(IL,JL)
         If(abs(testo).gt.corrmax) corrmax = abs(testo)
        End Do
       End Do

c      write (6,*) iter,T(3,3)

c---
c      write (6,*)
c      write (6,117) iter
c      Do i=1,N1
c        write (6,110) (T(i,j),j=1,M1)
c      End Do
c---

c      write (6,201) Iter,corrmax,log10(abs(corrmax))
c      write (9,201) Iter,corrmax,log10(abs(corrmax))

       If(corrmax.lt.tol) Go to 98

  97  Continue

c-------
c failed
c-------

      write (6,*) " pois_gs1: Gauss-Siedel iterations did not converge"

      stop

c------------------
c solution computed
c------------------

  98  Continue

c     write (6,*) " pois_gs1: Gauss-Siedel required "
c    +             ,iter," iterations"

      close (9)

c-----
c Done
c-----

 100  Format (50(1x,f8.4))
 101  Format (2(1x,i3),2(1x,f5.3))
 102  Format (" Correction factor           = ",f10.5)
 103  Format (" Residuals = ",2(1x,f15.4))

 110  Format (30(1x,f6.3))
 117  Format ("Iteration: ",i5,1x,f15.8)

 201  Format (1x,i5,3(1x,f15.10))

      Return
      End
